@@ -1,11 +1,13 @@
 /* App.m
  * Application class of Cenon
  *
- * Copyright (C) 1995-2012 by vhf interservice GmbH
+ * Copyright (C) 1995-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2012-06-29 (-terminate:, terminate sub processes)
+ * modified: 2014-07-23 (Import of cenon files added)
+ *           2013-06-14 (-listFromPSFile: AI with header will be loaded as UTF8 and lossy)
+ *           2012-06-29 (-terminate:, terminate sub processes)
  *           2012-06-22 (-openFile:, remove i-cut Import Stuff)
  *           2012-02-06 (systemLibrary: return nil on Apple)
  *           2011-12-02 (i-cut Import Stuff added)
@@ -20,26 +22,29 @@
  *           2009-02-25 (-listFromFile: check extensions for any case)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
+ * http://www.cenon.info
  */
 
 #include <AppKit/AppKit.h>
+//#include <objc/runtime.h> // class_addMethod, objc_getMetaClass
 #include <VHFShared/vhfCommonFunctions.h>
+#include <VHFShared/VHFStringAdditions.h>
 #include <VHFShared/VHFSystemAdditions.h>
 #include "messages.h"
 #include "functions.h"
@@ -75,7 +80,39 @@
  */
 + (void)initialize
 {   NSMutableDictionary	*registrationDict = [NSMutableDictionary dictionary];
+    NSUserDefaults      *defaults = [NSUserDefaults standardUserDefaults];
 
+    /* call transition script, allowing us to handle certain migrations */
+    /*{ NSTask      *task = [[NSTask alloc] init];
+        NSString    *path = [[NSBundle mainBundle] resourcePath];
+
+        task.launchPath = [path stringByAppendingPathComponent:@"CenonTransition.sh"];
+        [task launch];
+        [task release];
+    }*/
+
+    /* transition to new App Id (version >= 4.0.3)
+     * we copy our defaults from "de.vhf.Cenon" -> "com.Cenon"
+     */
+    if ( [defaults persistentDomainForName:@"com.Cenon"] == nil )
+    {   NSArray *domains = [defaults persistentDomainNames];
+
+        if ( [domains containsObject:@"de.vhf.Cenon"] )
+        {   NSDictionary    *dict = [defaults persistentDomainForName:@"de.vhf.Cenon"];
+            NSArray         *keys = [dict allKeys];
+            int             i;
+
+            NSLog(@"Copy defaults from \"de.vhf.Cenon\" -> \"com.Cenon\"");
+            for ( i=0; i < [keys count]; i++ )
+            {   NSString    *key = [keys objectAtIndex:i];
+
+                [defaults setObject:[dict objectForKey:key] forKey:key];
+            }
+        }
+    }
+
+
+    // TODO: we could load this from a property list resource
     [registrationDict setObject:@"." forKey:@"NSDecimalSeparator"];
 
     /* General Preferences defaults */
@@ -104,6 +141,25 @@
     [registrationDict setObject:@"NO"        forKey:@"exportFlattenText"];
 
     [[NSUserDefaults standardUserDefaults] registerDefaults:registrationDict];
+
+
+#   if 0   // TODO: add missing methods
+    //[CenMissingMethods add];
+    //#include <objc/runtime.h>
+    /* add missing methods (for older os x versions)
+     */
+    /*NSColor *_NSColor_colorWithCGColor_(Class self, SEL cmd, CGColorRef cgColor)
+    {
+        // code
+        return nsColor;
+    }*/
+
+    if ( ! [[NSColor class] respondsToSelector:@selector(colorWithCGColor:)] )
+    {
+        // older system, add your own category
+        class_addMethod(objc_getMetaClass("NSColor"), @selector(colorWithCGColor:), (IMP)_NSColor_colorWithCGColor_, "@@:@");
+    }
+#   endif
 }
 
 /* modified: 2004-02-13
@@ -111,7 +167,7 @@
 - init
 {
     if ( (self = [super init]) )
-        [self setDelegate:self]; // so that we get NSApp delegation methods
+        [self setDelegate:self];    // so that we get NSApp delegation methods
 
     modules = [NSMutableArray new];
 
@@ -234,29 +290,14 @@
     NSFileManager   *fileManager = [NSFileManager defaultManager];
     NSMutableArray  *loadedFiles = [NSMutableArray array];
 
-    /* load modules from application bundle
-     * FIXME: The CAM module should be integrated below (case 0)
-     */
-    /*path = [mainBundle pathForResource:@"CAM" ofType:@"bundle"];
-    if ( (bundle = [NSBundle bundleWithPath:path]) )
-    {   NSString	*file = @"CAM.bundle";
-
-        bundleClass = [bundle principalClass];  // controller
-        [modules addObject:bundle];             // our loaded modules
-        [bundleClass instance];                 // create instance
-        camModuleLoaded = YES;
-        [loadedFiles addObject:file];
-        NSLog(@"Load Module: %s/%s\n", [path UTF8String], [file UTF8String]);
-    }*/
-
-    /* load additional modules */
+    /* load modules */
     for (i=0; i <= 5; i++)
     {   NSArray	*files;
 
         switch (i)
         {
             default:
-            //case 0: path = [[mainBundle resourcePath] stringByAppendingPathComponent:BUNDLEFOLDER]; break;
+            //case 0: path = [[mainBundle resourcePath] stringByAppendingPathComponent:@"Modules"]; break;
             case 0: path = [mainBundle resourcePath]; break;    // load modules from main bundle
             case 1: path = [userLibrary()  stringByAppendingPathComponent:@"Bundles"]; break;
             case 2: path = [localLibrary() stringByAppendingPathComponent:@"Bundles"]; break;
@@ -268,7 +309,10 @@
             continue;
 
         /* we load the bundles in alphabetic order */
-        files = [[fileManager directoryContentsAtPath:path] sortedArrayUsingSelector:@selector(compare:)];
+        //files = [fileManager contentsOfDirectoryAtPath:path error:NULL];  // >= OSX 10.6
+        //files = [fileManager directoryContentsAtPath:path];               // <= OSX 10.5
+        files = [fileManager contentsOfDirectoryAtPath:path];
+        files = [files sortedArrayUsingSelector:@selector(compare:)];
         for (j=0; j<(int)[files count]; j++)
         {   NSString	*file = [files objectAtIndex:j];
 
@@ -286,6 +330,8 @@
             }
         }
     }
+
+    //[self activateIgnoringOtherApps: YES];  // bring Cenon back to the front, if some module loading pushed it to the back
 }
 - (NSArray*)modules
 {
@@ -297,7 +343,7 @@
  */
 - (void)applicationDidFinishLaunching:(NSNotification *)notification
 {   NSApplication	*theApplication = [notification object];
-    NSString		*path;
+    NSString		*path, *pathDoc;
     NSFileManager	*fileManager = [NSFileManager defaultManager];
 
     [[NSUserDefaults standardUserDefaults] setObject:@"." forKey:@"NSDecimalSeparator"];
@@ -341,10 +387,10 @@
         {   NSString	*from, *to;
 
             /* create HOME/Cenon */
-            [fileManager createDirectoryAtPath:path attributes:nil];
+            [fileManager createDirectoryAtPath:path recursive:NO attributes:nil error:NULL];
             /* create HOME/Cenon/Projects */
             [fileManager createDirectoryAtPath:vhfPathWithPathComponents(path, @"Projects", nil)
-                                    attributes:nil];
+                                     recursive:NO attributes:nil error:NULL];
             /* copy Cenon/Projects/.dir.tiff */
             from = vhfPathWithPathComponents(localLibrary(), @"Projects", @".dir.tiff", nil);
             to   = vhfPathWithPathComponents(path,           @"Projects", @".dir.tiff", nil);
@@ -355,6 +401,14 @@
             [fileManager copyPath:from toPath:to handler:nil];
         }
     }
+    /* create symbolic link from FOME/Documents/Cenon -> HOME/Library/../Cenon
+     * we want symbolic link, because
+     * - if Library would be renamed to a backup file, our link would point to the backup
+     * - hard links can't be rsynced
+     */
+    pathDoc = vhfUserDocuments(APPNAME);
+    if ( pathDoc && ! [fileManager fileExistsAtPath:pathDoc] )
+        [fileManager createSymbolicLinkAtPath:pathDoc withDestinationPath:path error:NULL];
 
     [theApplication setDelegate:self];
 
@@ -406,7 +460,8 @@
     document = [Document new];
 }
 
-/* modified: 2011-09-16 (icut format added)
+/* modified: 2014-07-23 (cenon format added)
+ *           2011-09-16 (icut format added)
  *           2010-09-10 (defaultManager renamed to fileManager)
  *           2009-02-25 (fileNameLC)
  */
@@ -416,9 +471,23 @@
     NSFileManager   *fileManager = [NSFileManager defaultManager];
     NSString        *fileNameLC = [fileName lowercaseString];
 
+    if ( [fileNameLC hasSuffix:@".cenon"] || [fileNameLC hasSuffix:@".cen"] )
+    {   NSString        *fileDirectory = fileName;
+        NSDictionary	*plist = nil;
+
+        fileName = [fileName stringByAppendingPathComponent:DOCUMENT_NAME];
+
+        plist = [NSDictionary dictionaryWithContentsOfFile:fileName];
+        if ( [[plist objectForKey:@"Data"] isKindOfClass:[NSDictionary class]] )
+            list = [DocView readList:[plist objectForKey:@"Data"]
+                         inDirectory:fileDirectory];
+        else
+            list = [DocView readList:[[[NSUnarchiver alloc] initForReadingWithData:[plist objectForKey:@"Data"]] autorelease]
+                         inDirectory:fileDirectory];
+    }
     //if ( [fileName rangeOfString:@".dxf" options:NSCaseInsensitiveSearch].length )
     //if ( [fileName compare:@".dxf" options:NSAnchoredSearch|NSBackwardsSearch|NSCaseInsensitiveSearch] )
-    if ( [fileNameLC hasSuffix:@".dxf"] )
+    else if ( [fileNameLC hasSuffix:@".dxf"] )
     {	id	dxfImport = [[DXFImportSub allocWithZone:[self zone]] init];
         NSData	*data = [NSData dataWithContentsOfFile:fileName];	// get data object
 
@@ -430,7 +499,7 @@
     {	id      icutImport = [[ICUTImportSub allocWithZone:[self zone]] init];
         NSData  *data = [NSData dataWithContentsOfFile:fileName];   // get data object
 
-        // FIXME: should come from Somewher with the possibility to change
+        // FIXME: should come from Somewhere with the possibility to change
         //        [icutImport fillClosedPaths:Prefs_ICUTFillClosedPaths];
         //        [icutImport originUL:Prefs_ICUTOriginUL];
         list = [[icutImport importICUT:data] retain];	// get list of graphic objects from import
@@ -613,6 +682,8 @@
     return [list autorelease];
 }
 
+/* modified: 2013-06-14 (AI with header will be loaded as UTF8 and lossy for more tolerance)
+ */
 - (NSArray*)listFromPSFile:(NSString*)fileName
 {   PSImportSub		*psImport;
     NSData          *data = nil;
@@ -624,9 +695,10 @@
     {   NSString	*string, *header;
         NSRange		range;
 
-        string = [NSString stringWithContentsOfFile:fileName];	/* get file */
+        // FIXME: this is actually covered by (VHFStringAdditions +stringWithContentsOfFile:) ???
+        string = [NSString stringWithContentsOfFile:fileName];   // get file
         range = [string rangeOfString:@"%%BeginResource"];
-        if ( !range.length )	/* no ps-header */
+        if ( !range.length )	// no ps-header
         {
             path = [[[NSBundle bundleForClass:[PSImport class]] resourcePath]
                     stringByAppendingPathComponent:AI_HEADER];
@@ -634,11 +706,11 @@
             {   NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                 return nil;
             }
-            header = [NSString stringWithContentsOfFile:path];	/* get file */
+            header = [NSString stringWithContentsOfFile:path];	// get file
 
             string = [header stringByAppendingString:string];
         }
-        data = [string dataUsingEncoding:NSASCIIStringEncoding];
+        data = [string dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES];
     }
     else
         data = [NSData dataWithContentsOfFile:fileName];	/* get file */
@@ -923,7 +995,9 @@ typedef enum
 
 - (NSSavePanel*)saveAsPanelWithSaveType:(NSString*)ext
 {   NSSavePanel		*savePanel = [NSSavePanel savePanel];
-    NSDictionary	*dict = [NSDictionary dictionaryWithObjectsAndKeys:@"0", DOCUMENT_EXT, @"1", EPS_EXT, @"2", GERBER_EXT, @"3", DXF_EXT, @"4", HPGL_EXT, @"5", FONT_EXT, @"6", DIN_EXT, nil];
+    NSDictionary	*dict = [NSDictionary dictionaryWithObjectsAndKeys:
+                             @"0", DOCUMENT_EXT, @"1", EPS_EXT,  @"2", GERBER_EXT,
+                             @"3", DXF_EXT,      @"4", HPGL_EXT, @"5", FONT_EXT,   @"6", DIN_EXT, nil];
 
     [savePanel setAccessoryView:[savePanelAccessory retain]];
     [savePanel setRequiredFileType:ext];
@@ -1025,7 +1099,7 @@ typedef enum
         NSSavePanel *savePanel = [NSSavePanel savePanel];
 
         if ( ! [[NSFileManager defaultManager] fileExistsAtPath:path])
-            [[NSFileManager defaultManager] createDirectoryAtPath:path attributes:nil];
+            [[NSFileManager defaultManager] createDirectoryAtPath:path recursive:NO attributes:nil error:NULL];
         file = vhfPathWithPathComponents(path, name, nil);
 
         [savePanel setRequiredFileType:@"plist"];
@@ -1312,7 +1386,7 @@ typedef enum
         default: site = NSLocalizedString(@"http://www.cenon.info", Web Site); break;
         case 1:  site = NSLocalizedString(@"http://www.cenon.info/support_faq_gb.html", Web FAQ); break;
         case 2:  site = NSLocalizedString(@"http://www.cenon.info/releaseNotes_gb.html", Web Releae Notes); break;
-        case 3:  site = NSLocalizedString(@"mailto:service@vhf.de?subject=Cenon%20Feedback", eMail Address); break;
+        case 3:  site = NSLocalizedString(@"mailto:info@cenon.de?subject=Cenon%20Feedback", eMail Address); break;
         case 4:  site = NSLocalizedString(@"http://www.cenon.info/news/news_gb.html", News); break;
                  //site = NSLocalizedString(@"http://www.cenon.info/dApple_gb.html", Download); break;
     }
@@ -1625,10 +1699,10 @@ typedef enum
 
     /* terminate sub processes */
     for (count=[modules count]-1; count>=0; count--)
-    {  NSBundle     *module = [modules objectAtIndex:count];
+    {   NSBundle     *module = [modules objectAtIndex:count];
 
         if ( [[[module principalClass] instance] respondsToSelector:@selector(terminate)] )
-            [[[module principalClass] instance] terminate];
+            [(id <CenonModuleMethods>)[[module principalClass] instance] terminate];
     }
 
     [super terminate:sender];
