@@ -1,36 +1,39 @@
 /* DocView.h
  * Cenon document view class
  *
- * Copyright (C) 1996-2012 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  *
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2012-02-13 (separated gridList for x and y)
+ * modified: 2014-10-21 (-tileXYDirections: instance tileXYDirections new)
+ *           2014-06-14 (scaleSize added for x = y scaling)
+ *           2013-10-30 (cacheImage added, cache removed)
+ *           2012-02-13 (separated gridList for x and y)
  *           2011-04-06 (pathSetStartPoint: added)
  *           2011-03-03 (scaleGTo: added)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.cenon.de
  */
 
-#ifndef VHF_H_DOCVIEW
-#define VHF_H_DOCVIEW
+#ifndef CEN_H_DOCVIEW
+#define CEN_H_DOCVIEW
 
 #include <AppKit/AppKit.h>
 #include "FlippedView.h"
@@ -48,6 +51,7 @@
 #define DocViewDrawDecoration		@"DocViewDrawDecoration"        // draw stuff which is not inside cache
 #define DocViewMouseDown            @"DocViewMouseDown"             // inform modules about mouse down
 #define DocViewDragSelect           @"DocViewDragSelect"            // inform modules about drag region
+#define DocViewCrossHairMoved       @"DocViewCrossHairMoved"        // inform modules about crosshair moved
 
 extern NSString *e2PboardType;
 
@@ -57,10 +61,11 @@ extern NSString *e2PboardType;
     NSColor         *backgroundColor;
     NSMutableDictionary	*statusDict;        // dictionary used for storing status information
 
-    NSWindow        *cache;                 // the cache window and its view
-    NSView          *cacheView;
+    //NSWindow        *cache;                 // the cache window and its view
+    NSView          *cacheView;             // the cacheView
+    NSImage         *cacheImage;            // the cache image if there is no gState (for Mavericks OS 10.9)
     BOOL            doCaching;              // wether we have to cache
-    id              betaCache;              // buffer for moving objects
+    id              betaCache;              // buffer for moving objects (TODO: replace this with [NSWindow –cacheImageInRect:, -restoreCachedImage])
     FlippedView     *editView;              // superview for text editing
 
     BOOL            inTimerLoop;            // used for scrolling
@@ -82,20 +87,20 @@ extern NSString *e2PboardType;
     NSPoint         tileLimits;
     BOOL            tileLimitSize;          // whether to limit by size instead of item-number
     NSPoint         tileDistance;           // distance between parts
-    //BOOL          tileAbsoluteDistance;   // absolute instead of relative distance between tiles
     NSSize          tileSize;               // size of a part
-    //BOOL          tileMoveMasterToOrigin; // whether to move the master graphics to origin
+    NSPoint         tileXYDirections;        // whether to move the tiles to the right/up (or NO left/down)
     id              serialNumber;           // the serialnumber object
 
     BOOL            gridIsEnabled;
     int             gridUnit;
-    float           gridSpacing;
+    VFloat          gridSpacing;
     NSRect          *gridListX, *gridListY;
     int             numGridRectsX, numGridRectsY;
 
     VCrosshairs     *origin;                // the crosshairs for the offset
 
-    float           scale;                  // the scale factor 1.0->100%, 0.5->50%
+    VFloat          scale;                  // the scale factor 1.0->100%, 0.5->50%
+    NSSize          scaleSize;              // the scale factor 1.0->100%, 0.5->50%
 
     VGraphic        *originalPaste;         // the first pasted graphic
     int             consecutivePastes;      // number of consecutive pastes
@@ -130,7 +135,7 @@ extern NSString *e2PboardType;
 - (NSWindow*)cache;
 - (void)setCaching:(BOOL)flag redraw:(BOOL)rd;
 - (void)sizeCacheWindow:(float)width :(float)height;
-- (void)scaleCacheWindow:(float)x :(float)y;
+- (void)scaleCacheWindow:(NSSize)newUnitSize;
 - (void)scaleUnitSquareToSize:(NSSize)_newUnitSize;
 - (void)setFrameSize:(NSSize)_newSize;
 - (void)scrollPointToVisible:(NSPoint)point;
@@ -158,8 +163,9 @@ extern NSString *e2PboardType;
 
 - (id)clipObject;
 
-- (float)scaleFactor;
-- (float)controlPointSize;
+- (VFloat)scaleFactor;  // scale factor for x == y scaling
+- (NSSize)scaleSize;    // scale factor with width + height
+- (VFloat)controlPointSize;
 - (void)drawControls:(NSRect)rect;
 //- (void)drawControl:(NSRect)r for:object;
 - (void)drawRect:(NSRect)rect;
@@ -296,11 +302,12 @@ extern NSString *e2MatchTypes(NSArray *typesToMatch, NSArray *orderedTypes);
 - (NSPoint)tileDistance;
 - (BOOL)tileLimitSize;
 - (NSPoint)tileLimits;
+- (NSPoint)tileXYDirections;
 - (void)setTileWithLimits:(NSPoint)limits limitSize:(BOOL)limitSize distance:(NSPoint)dist
-             moveToOrigin:(BOOL)moveToOrigin;
+               directions:(NSPoint)dirs moveToOrigin:(BOOL)moveToOrigin;
 - (void)removeTiles;
 - (void)buildTileCopies:(NSPoint)limits limitSize:(BOOL)limitSize distance:(NSPoint)dist
-           moveToOrigin:(BOOL)moveToOrigin;
+             directions:(NSPoint)dirs moveToOrigin:(BOOL)moveToOrigin;
 - (id)serialNumber;
 - (void)incrementSerialNumbers;
 - (int)numberOfTiles;
@@ -334,4 +341,4 @@ extern NSString *e2MatchTypes(NSArray *typesToMatch, NSArray *orderedTypes);
 - (void)addPointTo:(VGraphic*)g atPoint:(NSPoint)p redraw:(BOOL)redraw;
 @end
 
-#endif // VHF_H_DOCVIEW
+#endif // CEN_H_DOCVIEW
