@@ -1,33 +1,34 @@
 /* DocWindow.h
  * Cenon document window class
  *
- * Copyright (C) 1995-2010 by vhf interservice GmbH
+ * Copyright (C) 1995-2014 by Cenon GmbH
  * Author: Georg Fleischmann
  *
  * Created:  1995-12-02
- * Modified: 2010-04-19 (unfoldedHeight)
+ * Modified: 2013-07-04 (-windowDidBecomeKey: added)
+ *           2010-04-19 (unfoldedHeight)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
-#ifndef VHF_H_DOCWINDOW
-#define VHF_H_DOCWINDOW
+#ifndef CEN_H_DOCWINDOW
+#define CEN_H_DOCWINDOW
 
 #include <AppKit/AppKit.h>
 
@@ -76,10 +77,11 @@
 /* delegate methods
  */
 - (void)windowDidBecomeMain:(NSNotification *)notification;
+- (void)windowDidBecomeKey:(NSNotification *)notification;
 //- (void)windowDidResignMain:(NSNotification *)notification;
 - (void)windowDidResignKey:(NSNotification *)notification;
 //- (BOOL)windowShouldClose:(id)sender;
 
 @end
 
-#endif // VHF_H_DOCWINDOW
+#endif // CEN_H_DOCWINDOW
