@@ -1,33 +1,36 @@
 /* Document.m
  * Cenon document class
  *
- * Copyright (C) 1996-2011 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author: Georg Fleischmann
  *
  * Created:  1996-02-09
- * Modified: 2011-03-30 (-exportLock, -setExportLock:)
+ * Modified: 2013-06-29 (+newFromFile: scale to saved scaleFactor)
+ *           2013-02-05 (-save  Cenon Version saved with file)
+ *           2013-01-02 (-printDocument: remove paper margins from view, which are not ignored any more since OS 10.8)
+ *           2011-03-30 (-exportLock, -setExportLock:)
  *           2010-11-26 (-saveTiff text work)
  *           2010-06-30 (-saveTiff added)
  *           2008-10-21 (clean-up)
  *           2008-07-24 (document units, update units)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.cenon.info
  */
 
 #include <AppKit/AppKit.h>
@@ -58,14 +61,13 @@
 #define	MAXWINHEIGHT        750
 
 #define DOCUMENT_VERSION    102
-#define DOCUMENT_NAME       @"document"
-#define OUTPUT_NAME         @"output"
 
 /* Private methods */
 @interface Document(PrivateMethods)
 - initWindow;
 - (void)initializePrintInfo;
-- (BOOL)setWindowSize:(NSSize)size;
+//- (BOOL)setWindowSize:(NSSize)size;
+- (void)scale:(NSSize)scaleSize withCenter:(NSPoint)center;
 //- (BOOL)loadOutputListsFromFile:(const char*)fileName;
 - (BOOL)saveTiff:(NSString*)filename;
 - (BOOL)saveEPS:(NSString*)filename;
@@ -194,7 +196,7 @@ static NSMutableArray *zoneList = nil;
 {   NSRect	frameRect;
     DocView	*view;
 
-    [window setDelegate:self];
+    [window setDelegate:(id)self];
     [scrollView setHasHorizontalScroller:YES];
     [scrollView setHasVerticalScroller:YES];
 
@@ -295,7 +297,7 @@ static NSMutableArray *zoneList = nil;
             directoryContainingDocument = fileDirectory ? fileDirectory
                                                         : [fileName stringByDeletingLastPathComponent];
 
-            [newDocument->window setDelegate:newDocument];
+            [newDocument->window setDelegate:(id)newDocument];
             [newDocument->scrollView setHasHorizontalScroller:YES];
             [newDocument->scrollView setHasVerticalScroller:YES];
 
@@ -340,7 +342,7 @@ static NSMutableArray *zoneList = nil;
 #endif
 
             /* workaround: something (NSWindow according documentation) doesn't work with size>10000.0 */
-            // #if VHF_IS_DOUBLE == 0
+            // #if CEN_IS_DOUBLE == 0
             fRect = rectFromPropertyList([plist objectForKey:@"ViewSize"]);
             if ( fRect.size.width > 10000.0 )
                 fRect.size.width = 10000.0;
@@ -400,6 +402,14 @@ static NSMutableArray *zoneList = nil;
 
             newDocument->haveSavedDocument = YES;
 
+            /* zoom */
+            {   NSMutableDictionary *plist = [[(App*)NSApp currentDocument] docSettingsDict];
+                VFloat              s = vfloatFromPropertyList([plist objectForKey:@"scaleFactor"]);    // [plist floatForKey:@"scaleFactor"]
+
+                if ( s > 0.0 && s != 1.0 )
+                    [[newDocument scrollView] scaleTo:s];
+            }
+
             [[NSNotificationCenter defaultCenter] postNotificationName:DocumentDidOpen
                                                                 object:view];
         }
@@ -471,10 +481,10 @@ static NSMutableArray *zoneList = nil;
     [(App*)NSApp setCurrentDocument:newDocument];
     [newDocument setName:nil andDirectory:nil];
 
-    win = [newDocument window];
+    win   = [newDocument window];
     sView = [newDocument scrollView];
 
-    [win setDelegate:newDocument];
+    [win setDelegate:(id)newDocument];
     [sView setHasHorizontalScroller:YES];
     [sView setHasVerticalScroller:YES];
 
@@ -493,6 +503,9 @@ static NSMutableArray *zoneList = nil;
     //newDocument->printInfo = [[NSPrintInfo allocWithZone:[newDocument zone]] init];
     [newDocument initializePrintInfo];
 
+    /* Prefs_RotateObjects */
+    /* rotate list with given Degree */
+
     /* calculate size of view */
     frameRect = [DocView boundsOfArray:list];	/* get bounds of list */
     frameRect.size.width  += frameRect.origin.x + MMToInternal(20.0);
@@ -501,7 +514,7 @@ static NSMutableArray *zoneList = nil;
     frameRect.origin.y = 0.0;
 
     /* workaround: something doesn't work with size > 10000.0 */
-    // #if VHF_IS_DOUBLE == 0
+    // #if CEN_IS_DOUBLE == 0
     fRect = frameRect;
     if ( fRect.size.width > 10000.0 )
         fRect.size.width = 10000.0;
@@ -783,7 +796,7 @@ static NSMutableArray *zoneList = nil;
 
 /* Document setting
  * created:  2008-07-19
- * modified: 2008-07-24
+ * modified: 2013-10-17 (convert... withVFloat instead of float)
  */
 - (NSMutableDictionary*)docSettingsDict
 {
@@ -797,7 +810,7 @@ static NSMutableArray *zoneList = nil;
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
     return (baseUnit == UNIT_NONE) ? [defaults integerForKey:@"unit"] : baseUnit;
 }
-- (float)convertToUnit:(float)iValue
+- (VFloat)convertToUnit:(VFloat)iValue
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
     CenonUnit       unit = (baseUnit == UNIT_NONE) ? [defaults integerForKey:@"unit"] : baseUnit;
 
@@ -810,7 +823,7 @@ static NSMutableArray *zoneList = nil;
     }
     return iValue;
 }
-- (float)convertFrUnit:(float)uValue
+- (VFloat)convertFrUnit:(VFloat)uValue
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
     CenonUnit       unit = (baseUnit == UNIT_NONE) ? [defaults integerForKey:@"unit"] : baseUnit;
 
@@ -823,7 +836,7 @@ static NSMutableArray *zoneList = nil;
     }
     return uValue;
 }
-- (float)convertMMToUnit:(float)mmValue
+- (VFloat)convertMMToUnit:(VFloat)mmValue
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
     CenonUnit       unit = (baseUnit == UNIT_NONE) ? [defaults integerForKey:@"unit"] : baseUnit;
 
@@ -836,7 +849,7 @@ static NSMutableArray *zoneList = nil;
     }
     return mmValue;
 }
-- (float)convertUnitToMM:(float)uValue
+- (VFloat)convertUnitToMM:(VFloat)uValue
 {   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
     CenonUnit       unit = (baseUnit == UNIT_NONE) ? [defaults integerForKey:@"unit"] : baseUnit;
 
@@ -1052,7 +1065,8 @@ static NSMutableArray *zoneList = nil;
  * 3. move file name (old) file to backup (fileName.cenon -> fileName.cenon~)
  * 4. move temporary file to file name (#fileName.cenon -> fileName.cenon)
  *
- * modified: 2007-07-24 (save baseUnit and project settings)
+ * modified: 2013-07-04 (save Cenon version to creator)
+ *           2007-07-24 (save baseUnit and project settings)
  */
 - (BOOL)save
 {   BOOL                savedOk = NO;
@@ -1071,12 +1085,13 @@ static NSMutableArray *zoneList = nil;
         return NO;
     }
     /* rename to backup */
-    if ( ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename handler:nil]) || ([fileManager fileExistsAtPath:filename] && ![fileManager movePath:filename toPath:backupFilename handler:nil]) )
+    if (    ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename handler:nil])
+         || ([fileManager fileExistsAtPath:filename]       && ![fileManager movePath:filename toPath:backupFilename handler:nil]) )
     {   NSRunAlertPanel(SAVE_TITLE, CANT_CREATE_BACKUP, nil, nil, nil);
         return NO;
     }
     /* create file directory */
-    else if ( isDirectory && [fileManager createDirectoryAtPath:filename attributes:nil] )
+    else if ( isDirectory && [fileManager createDirectoryAtPath:filename recursive:NO attributes:nil error:NULL] )
     {
         fileDirectory = filename;
         filename = [fileDirectory stringByAppendingPathComponent:DOCUMENT_NAME];
@@ -1085,7 +1100,8 @@ static NSMutableArray *zoneList = nil;
     pool = [NSAutoreleasePool new];
     /* save */
     if ([fileManager isWritableFileAtPath:fileDirectory])
-    {   NSArchiver	*archiver;
+    {   NSArchiver      *archiver;
+        NSString        *version  = [(App*)NSApp version];
 
         NS_DURING
             [window makeFirstResponder:[self documentView]];
@@ -1102,7 +1118,8 @@ static NSMutableArray *zoneList = nil;
             [fileContents setInt:[window hasCoordDisplay] forKey:@"hasCoordBox"];
             [fileContents setInt:magazineIndex forKey:@"MagazineIndex"];
             //[fileContents setObject:propertyListFromInt([VGraphic currentGraphicIdentifier]) forKey:@"GraphicIdentifier"];
-            [fileContents setObject:@"Cenon" forKey:@"Creator"];
+            //[fileContents setObject:@"Cenon" forKey:@"Creator"];
+            [fileContents setObject:[NSString stringWithFormat:@"Cenon %@", version] forKey:@"Creator"];
             [fileContents setObject:[[NSCalendarDate date] descriptionWithCalendarFormat:@"%Y-%m-%d %H:%M"] forKey:@"CreationDate"];
             /* Project Info */
             if (docVersion)
@@ -1452,7 +1469,7 @@ static NSMutableArray *zoneList = nil;
 
 - (void)printSeparation:(NSPrintOperation*)op
 {   NSColor             *sepColor;
-    NSPrintOperation    *sop=nil;
+    NSPrintOperation    *sop = nil;
     NSPrintInfo         *pi = [op printInfo], *spi;
     NSMutableDictionary *piDict = [pi dictionary];
     NSAutoreleasePool   *pool0 = [NSAutoreleasePool new], *pool;
@@ -1463,7 +1480,7 @@ static NSMutableArray *zoneList = nil;
     NSPoint             mPosition = NSZeroPoint, mOffset, lr, ll, ur, ul, center, dm, pOffset = {55.0, 55.0};
     int                 i, orientation = [pi orientation];
     DocView             *docView = [self documentView];
-    float               width, height, scale = [docView scaleFactor];
+    VFloat              width, height, scale = [docView scaleFactor];
 
     cropPath = vhfPathWithPathComponents([[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER, nil);
 
@@ -1753,7 +1770,7 @@ static NSMutableArray *zoneList = nil;
     [docView setSeparationColor:sepColor];
     [sepColor release];
     /* op is allready currentOperation !!!! */
-    [op setShowPanels:NO]; // else the user get a second printpanel
+    [op setShowsPrintPanel:NO]; // else the user get a second printpanel
     [op runOperation];
     [pool release];
 
@@ -1763,7 +1780,7 @@ static NSMutableArray *zoneList = nil;
     ml = [[layerList objectAtIndex:0] list]; // set Color string
     [[ml objectAtIndex:[ml count]-1] setString:[NSString stringWithFormat:@"Magenta  "]];
     sop = [NSPrintOperation printOperationWithView:docView printInfo:pi];
-    [sop setShowPanels:NO];
+    [sop setShowsPrintPanel:NO];
     spi = [sop printInfo]; // from here we add a "_c" to the NSPrintSavePath string in printInfo dictionary
     piDict = [spi dictionary];
     if ([piDict objectForKey:NSPrintSavePath])
@@ -1782,7 +1799,7 @@ static NSMutableArray *zoneList = nil;
     ml = [[layerList objectAtIndex:0] list]; // set Color string
     [[ml objectAtIndex:[ml count]-1] setString:[NSString stringWithFormat:@"Yellow  "]];
     sop = [NSPrintOperation printOperationWithView:docView printInfo:pi];
-    [sop setShowPanels:NO];
+    [sop setShowsPrintPanel:NO];
     spi = [sop printInfo];
     piDict = [spi dictionary];
     if ([piDict objectForKey:NSPrintSavePath])
@@ -1801,7 +1818,7 @@ static NSMutableArray *zoneList = nil;
     ml = [[layerList objectAtIndex:0] list]; // set Color string
     [[ml objectAtIndex:[ml count]-1] setString:[NSString stringWithFormat:@"Black  "]];
     sop = [NSPrintOperation printOperationWithView:docView printInfo:pi];
-    [sop setShowPanels:NO];
+    [sop setShowsPrintPanel:NO];
     spi = [sop printInfo];
     piDict = [spi dictionary];
     if ([piDict objectForKey:NSPrintSavePath])
@@ -1862,31 +1879,48 @@ static NSMutableArray *zoneList = nil;
 }
 
 /*
- * modified: 2005-09-01
+ * modified: 2013-01-02 (10.8: remove paper margins by adjusting print view)
  */
 - (void)printDocument:sender
 {   NSPrintOperation    *op;
     NSPrintPanel        *pp;
     NSPrintInfo         *pi;
     DocView             *docView = [self documentView];
-    NSRect              visibleRect, oldFrame;
-    float               width, height, scale = [docView scaleFactor];
+    NSRect              visibleRect, oldFrame, pageBounds = [printInfo imageablePageBounds];
+    VFloat              width, height, scale = [docView scaleFactor];
 
     /* set scale to 1.0 */
     scale = [docView scaleFactor];
     visibleRect = [docView visibleRect]; // needed later to calc the center to scroll back
-    oldFrame = [docView frame];
+    oldFrame  = [docView frame];
     width  = oldFrame.size.width  / scale;
     height = oldFrame.size.height / scale;
+#ifdef __APPLE__
+    /* Apple 10.8: the PrintInfo margins are 0, but there are paper margins PMPaperMargins()
+     * we could create a custom paper with PMPaperCreateCustom() or adjust our print view
+     */
+    if ( NSAppKitVersionNumber > NSAppKitVersionNumber10_7 )    // Workaround Max OS 10.8: make view smaller by paper margins
+    {   width  -= ([printInfo paperSize].width  - pageBounds.size.width);
+        height -= ([printInfo paperSize].height - pageBounds.size.height);
+    }
+#endif
     [window disableFlushWindow];
     [docView scaleUnitSquareToSize:NSMakeSize(1.0/scale, 1.0/scale)];
     [docView setFrameSize:NSMakeSize(width, height)];
+    /* Workaround Mac OS 10.8: we have to translate printing by the page margin to remove the margin */
+#ifdef __APPLE__
+    if ( NSAppKitVersionNumber > NSAppKitVersionNumber10_7 )
+        [docView translateOriginToPoint:NSMakePoint(-pageBounds.origin.x, -pageBounds.origin.y)];
+#endif
+
+    //[printInfo setHorizontalPagination:NSClipPagination];
+    //[printInfo setVerticalPagination:NSClipPagination];
 
     /* multi page document */
     if ([docView isMultiPage])
     {
         op = [NSPrintOperation printOperationWithView:[self documentView] printInfo:printInfo];
-        [op setShowPanels:YES];
+        [op setShowsPrintPanel:YES];
         [op runOperation];
 
         // FIXME: restore enabled/disabled pages
@@ -1904,21 +1938,22 @@ static NSMutableArray *zoneList = nil;
             [printInfo setPrinter:[NSPrintInfo defaultPrinter]];	// set default printer for the first time
 
         op = [NSPrintOperation printOperationWithView:[self documentView] printInfo:printInfo];
-        [op setShowPanels:YES];
+        [op setShowsPrintPanel:YES];
         [NSPrintOperation setCurrentOperation:op];
 
         pp = [NSPrintPanel printPanel];
         [pp setAccessoryView:[[(App*)NSApp printPanelAccessory] retain]];
+        //[pp addAccessoryController:[[(App*)NSApp printPanelAccessory] retain]];
         [op setPrintPanel:pp];
 
         if ([pp runModal] == NSCancelButton)
         {
-            [op setShowPanels:NO];	// else the user get a second printpanel
+            [op setShowsPrintPanel:NO]; // else the user get a second printpanel
             //[op runOperation];	// this will end the operation ! (this deletes a saved file from the prev. operation !)
         }
         else if (![[(App*)NSApp ppaRadio] selectedRow])		// composite
         {
-            [op setShowPanels:NO];	// else the user get a second printpanel
+            [op setShowsPrintPanel:NO]; // else the user get a second printpanel
             [op runOperation];
         }
         else							// color separation
@@ -1930,7 +1965,13 @@ static NSMutableArray *zoneList = nil;
         printInfo = [pi retain];	// remember printInfo values for the next time ! (2005-05-06)
 #endif
     }
+    [op cleanUpOperation];
 
+#ifdef __APPLE__
+    /* translate back */
+    if ( NSAppKitVersionNumber > NSAppKitVersionNumber10_7 )
+        [docView translateOriginToPoint:NSMakePoint(pageBounds.origin.x, pageBounds.origin.y)];
+#endif
     /* scale view back to old scale */
     [docView scaleUnitSquareToSize:NSMakeSize(scale, scale)];
     [docView setFrameSize:oldFrame.size];
@@ -2005,7 +2046,7 @@ static NSMutableArray *zoneList = nil;
         switch (NSRunAlertPanel(CLOSEWINDOW_STRING, SAVECHANGES_STRING, SAVE_STRING, DONTSAVE_STRING, CANCEL_STRING, [sender title]))
         {
             case NSAlertDefaultReturn:      // save
-                [self save];
+                [self save:nil];            // brings up a Save Panel, in case doc hasn't been saved before
                 break;
             case NSAlertAlternateReturn:    // dont save
                 break;
