@@ -1,6 +1,6 @@
 /* EPSExport.m
  *
- * Copyright (C) 2000-2012 by vhf interservice GmbH
+ * Copyright (C) 2000-2014 by Cenon GmbH
  * Author:   Ilonka Fleischmann
  *
  * created:  2000-12-21
@@ -10,27 +10,27 @@
  * This file is part of the vhf Export Library.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
@@ -283,15 +283,15 @@ epsState->noPoint = 1;
 }
 
 - (NSString*)epsVText:(VText*)g :(EPSState*)epsState
-{   NSString	*str = @"";
-    NSRect	b = [g bounds];
-    NSPoint	o;
-    NSString	*curStr; // , *text = [g string];
-    BOOL	filled = [g filled];
-    float	lineHeight = [g lineHeight];
+{   NSString        *str = @"";
+    NSRect          b = [g bounds];
+    NSPoint         o;
+    NSString        *curStr; // , *text = [g string];
+    BOOL            filled = [g filled];
+    float           lineHeight = [g lineHeight];
     NSScanner		*scanner = [NSScanner scannerWithString:[g string]];
     NSCharacterSet	*skipSet = [NSCharacterSet characterSetWithCharactersInString:@"\n\r"];
-    int			first = 1;
+    int             first = 1;
 
     o = b.origin;
     o.y += b.size.height - lineHeight;  // ???
@@ -318,7 +318,7 @@ epsState->noPoint = 1;
             str = [str stringByAppendingFormat:@"eofill\n"];
         }
         else
-        {   str = [str stringByAppendingFormat:@"(%@) false charpath\n"];
+        {   str = [str stringByAppendingFormat:@"(%@) false charpath\n", curStr];
             str = [str stringByAppendingFormat:@"stroke\n"];
         }
         [scanner scanCharactersFromSet:skipSet intoString:NULL];
@@ -350,7 +350,8 @@ epsState->noPoint = 1;
         return NO;
     }
     /* rename to backup */
-    if ( ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename handler:nil]) || ([fileManager fileExistsAtPath:filename] && ![fileManager movePath:filename toPath:backupFilename handler:nil]) )
+    if (   ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename handler:nil])
+        || ([fileManager fileExistsAtPath:filename]       && ![fileManager movePath:filename toPath:backupFilename handler:nil]) )
     {   NSRunAlertPanel(SAVE_TITLE, CANT_CREATE_BACKUP, nil, nil, nil);
         return NO;
     }
@@ -396,7 +397,7 @@ epsState->noPoint = 1;
     }
 
 #if 0
-    //else if ( isDirectory && [fileManager createDirectoryAtPath:filename attributes:nil] )
+    //else if ( isDirectory && [fileManager createDirectoryAtPath:filename recursive:NO attributes:nil error:NULL] )
     //fileDirectory = [filename stringByDeletingLastPathComponent];
     /* save */
     //if ([fileManager isWritableFileAtPath:filename])
