@@ -1,29 +1,29 @@
 /* HiddenArea.m
  * Object for calculation of hidden areas
  *
- * Copyright (C) 1996-2004 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Ilonka Fleischmann
  *
  * created:  1996-09-24
- * modified: 2004-12-13
+ * modified: 2006-11-07
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
@@ -37,7 +37,7 @@
 */
 @interface HiddenArea(PrivateMethods)
 - (void)optimizeList:(NSMutableArray*)list;
-- (BOOL)addGraphics:(NSMutableArray*)splitList inside:(VGraphic*)graphic toList:(NSMutableArray*)list;
+- (BOOL)addGraphics:(NSMutableArray*)splitList inside:(id)graphic toList:(NSMutableArray*)list;
 - (BOOL)removePartsOf:(VGraphic**)curG hiddenBy:(VGraphic*)og;
 @end
 
@@ -45,9 +45,10 @@
 
 /* remove parts of graphic objects coverd by other graphics
  * first graphic in list is covered by all following graphics in list etc.
+ * modified: 2006-06-07
  */
 - (void)removeHiddenAreas:(NSMutableArray*)list
-{   long	c, o, prevCnt;
+{   long	c, o;
 
     if ( ![list count] )
         return;
@@ -55,72 +56,140 @@
     /* beginning with the first graphic
      */
     for ( c=0; c < (int)[list count]-1; c++ )
-    {	id	cg = [list objectAtIndex:c];
-        NSColor	*cColor = [[cg color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-        NSColor	*cFillColor=nil, *oFillColor=nil, *cEndColor=nil, *oEndColor=nil;
-        NSRect	cBounds;
-        float	cw = [cg width], ow;
-        int	cf = 0, of = 0;
+    {	VGraphic    *cg = [list objectAtIndex:c];
+        NSColor     *cColor = [[cg color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+        NSColor     *cFillColor=nil, *oFillColor=nil, *cEndColor=nil, *oEndColor=nil;
+        NSRect      cBounds;
+        float       cw = [cg width], ow;
+        int         cf = 0, of = 0;
+        int         ic=0, icCnt = 1;
 
-        if ([cg respondsToSelector:@selector(fillColor)])
-        {
-            cf = [cg filled];
-            cFillColor = [[cg fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-            cEndColor  = [[cg endColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+        if ( [cg isKindOfClass:[VText class]] || [cg isKindOfClass:[VTextPath class]] )
+            continue;	// skip text	// FIXME: convert text to path
+
+        /* group must be in it self removeHiddenAreas */
+        if ( [cg isKindOfClass:[VGroup class]] )
+        {   VGroup	*grp = [VGroup group];
+            int		i, gCnt = [[(VGroup*)cg list] count];
+
+            for (i=0; i< gCnt; i++)
+                [grp addObject:[[(VGroup*)cg list] objectAtIndex:i]];
+
+            [list removeObjectAtIndex:c];
+            [list insertObject:grp atIndex:c];
+            cg = [list objectAtIndex:c];
+
+            [self removeHiddenAreas:[(VGroup*)cg list]];
         }
 
-        prevCnt = [list count];
-        cBounds = [cg bounds];
+        if ([cg isKindOfClass:[VGroup class]])
+            icCnt = [(VGroup*)cg countRecursive]; // [[cg list] count];
 
-        /* look if the cg lies under one of the other (o) graphics
-         * if the two graphics has the same colour they will be united later !
-         * else: the hidden areas of the hidden graphic will be removed or the hole hiddenG
-         */
-        for ( o=c+1; o < (int)[list count]; o++ )
-        {   id		og = [list objectAtIndex:o];
-            NSRect	oBounds;
+        for (ic=0; ic < icCnt; ic++)
+        {   VGraphic    *icg = cg;
 
-            oBounds = [og bounds];
+            if ([cg isKindOfClass:[VGroup class]])
+                icg = [(VGroup*)cg recursiveObjectAtIndex:ic]; // [[cg list] objectAtIndex:ic];
 
+            if ( [icg isKindOfClass:[VText class]] || [icg isKindOfClass:[VTextPath class]] )
+                continue;	// skip text	// FIXME: convert text to path
+
+            if ([icg respondsToSelector:@selector(fillColor)])
+            {
+                cf = [icg filled];
+                cFillColor = [[(VPath*)icg fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                cEndColor  = [[(VPath*)icg endColor]  colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+            }
+
+            cBounds = [icg bounds];
+
+            /* look if the cg lies under one of the other (o) graphics
+             * if the two graphics have the same color they will be united later !
+             * else: the hidden areas of the hidden graphic will be removed or the hole hiddenG
+             */
+            for ( o=c+1; o < (int)[list count]; o++ )
+            {   VGraphic    *og = [list objectAtIndex:o];
+                NSRect      oBounds;
+                int         io=0, ioCnt = 1;
+
+                if ( [og isKindOfClass:[VText class]] || [og isKindOfClass:[VTextPath class]] )
+                    continue;	// skip text	// FIXME: convert text to path
+/*                {   id	txtGr = [og pathRepresentation];
+
+                    [list removeObjectAtIndex:o];
+                    [list insertObject:txtGr atIndex:o];
+                    og = [list objectAtIndex:o];
+                } // konvertiert text in pfad ist dann aber definitiv ein Pfad !!!???
+*/
+                if ([og isKindOfClass:[VGroup class]])
+                    ioCnt = [(VGroup*)og countRecursive]; // [[og list] count];
+
+                for (io=0; io < ioCnt; io++)
+                {   VGraphic	*iog = og;
+
+                    if ([og isKindOfClass:[VGroup class]])
+                        iog = [(VGroup*)og recursiveObjectAtIndex:io]; // [[og list] objectAtIndex:io];
+
+                    if ( [iog isKindOfClass:[VText class]] || [iog isKindOfClass:[VTextPath class]] )
+                        continue;	// skip text	// FIXME: convert text to path
+
+                    oBounds = [iog bounds];
+                    
             //if ( !NSIsEmptyRect(NSIntersectionRect(cBounds , oBounds)) )
-            if (vhfIntersectsRect(cBounds , oBounds))
-            {   NSColor	*oColor = [[og color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                    if (vhfIntersectsRect(cBounds , oBounds))
+                    {   NSColor	*oColor = [[iog color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
 
-                of = 0;
-                ow = [og width];
-                if ([og respondsToSelector:@selector(fillColor)])
-                {
-                    of = [og filled];
-                    oFillColor = [[og fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-                    oEndColor  = [[og endColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-                }
+                        of = 0;
+                        ow = [iog width];
+                        if ([iog respondsToSelector:@selector(fillColor)])
+                        {
+                            of = [iog filled];
+                            oFillColor = [[(VPath*)iog fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                            oEndColor  = [[(VPath*)iog endColor]  colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                        }
 
-                if (!(((((cw && cw == ow) || (!cw && !ow && !cf && !of)) && [cColor isEqual:oColor]) ||
-                       (!cw && !ow && cf && cf == of)) &&
-                      ((cf >= 1 && cf == of && [cFillColor isEqual:oFillColor]) || (!cf && !of)) && // fillColors
-                      ((cf > 1 && cf == of && [cEndColor isEqual:oEndColor]) || (cf <= 1 && cf == of)))) // endColors
+                        if ( !(((((cw && cw == ow) || (!cw && !ow && !cf && !of)) && [cColor isEqual:oColor]) ||
+                               (!cw && !ow && cf && cf == of)) &&
+                              ((cf >= 1 && cf == of && [cFillColor isEqual:oFillColor]) || (!cf && !of)) &&             // fillColors
+                              ((cf > 1  && cf == of && [cEndColor  isEqual:oEndColor])  || (cf <= 1 && cf == of))) )    // endColors
 //                if ( ![cColor isEqual:oColor] )
-                {
-                    if ( [self removePartsOf:&cg hiddenBy:og] )
-                    {	/* if the hidden graphic is in the og we remove the hiddenG */
-                        [list removeObjectAtIndex:c];  /* cg is oldCg */;
-                        c--;
-                        break;
+                        {
+                            if ( [self removePartsOf:&icg hiddenBy:iog] )
+                            {	/* if the hidden graphic is in the iog we remove the hiddenG */
+
+                                if ([cg isKindOfClass:[VGroup class]])
+                                {   [(VGroup*)cg recursiveRemoveObjectAtIndex:ic]; // [[cg list] removeObjectAtIndex:ic];
+                                    icCnt--;
+                                    ic--;
+                                }
+                                else
+                                {   [list removeObjectAtIndex:c];  /* cg is oldCg */;
+                                    c--;
+                                    cg = nil;
+                                }
+                                break;
+                            }
+                            if ([cg isKindOfClass:[VGroup class]])
+                            {   [(VGroup*)cg recursiveRemoveObjectAtIndex:ic];      // [[cg list] removeObjectAtIndex:ic];
+                                [(VGroup*)cg recursiveInsertObject:icg atIndex:ic]; // [[cg list] insertObject:icg atIndex:ic];
+                            }
+                            else
+                            {   [list removeObjectAtIndex:c];
+                                [list insertObject:icg atIndex:c];	/* is a new cg */
+                                cg = icg = [list objectAtIndex:c];
+                            }
+                        }
                     }
-                    [list removeObjectAtIndex:c];
-                    [list insertObject:cg atIndex:c];	/* is a new cg */
                 }
             }
         }
-        /* if there is only one graphic behind unite/removed
-         * this cg (also for the last in list?) will not optimize -> need c < lCnt-1
-         */
-        if ( prevCnt != (int)[list count] && c < (int)[list count]-1 )
-        {   if ( c != -1 )
+        if ([cg isKindOfClass:[VGroup class]] && ![(VGroup*)cg countRecursive])
+        {
+            [list removeObjectAtIndex:c];
             c--;
         }
     }
-    return [self uniteAreas:list];
+    [self uniteAreas:list];
 }
 
 /* unite graphics in list which have the same color - remove other graphic
@@ -132,110 +201,182 @@
      */
     for ( c=0 ; c < (int)[list count] ; c++ )
     {   NSAutoreleasePool	*pool;
-	VGraphic		*cg = [list objectAtIndex:c];
-        NSColor			*cColor;
-        NSColor			*cFillColor=nil, *oFillColor=nil, *cEndColor=nil, *oEndColor=nil;
-        float			cw = [cg width], ow;
-        int			cf = 0, of = 0;
-        NSRect			cBounds;
+        VGraphic            *cg = [list objectAtIndex:c];
+        NSColor             *cColor;
+        NSColor             *cFillColor=nil, *oFillColor=nil, *cEndColor=nil, *oEndColor=nil;
+        float               cw = [cg width], ow;
+        int                 cf = 0, of = 0;
+        NSRect              cBounds;
+        int                 ic=0, icCnt = 1;
 
         /* lines, curves, full/unfilled arcs or unfilled paths can't be united */
         if ( [cg isKindOfClass:[VLine class]] || [cg isKindOfClass:[VCurve class]] ||
              ([cg isKindOfClass:[VArc class]] && (Abs([(VArc*)cg angle]) != 360.0 || ![cg filled])) ||
              ([cg isKindOfClass:[VPath class]] && ![cg filled]) ||
-             ([cg isKindOfClass:[VPolyLine class]] && ![cg filled]) )
+             ([cg isKindOfClass:[VPolyLine class]] && ![cg filled]) ||
+             ([cg isKindOfClass:[VRectangle class]] && ![cg filled]) )
             continue;
 
-        cColor = [[cg color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-        if ([cg respondsToSelector:@selector(fillColor)])
-        {
-            cf = [cg filled];
-            cFillColor = [[(VPath*)cg fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-            cEndColor  = [[(VPath*)cg endColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-        }
-
-        pool = [NSAutoreleasePool new];
-
         /* group must be in it self united */
         if ( [cg isKindOfClass:[VGroup class]] )
+        {   VGroup	*grp = [[cg copy] autorelease];
+
+            [list removeObjectAtIndex:c];
+            [list insertObject:grp atIndex:c];
+            cg = [list objectAtIndex:c];
+
             [self uniteAreas:[(VGroup*)cg list]];
+        }
 
-        prevCnt = [list count];
-        cBounds = [cg bounds];
+        if ([cg isKindOfClass:[VGroup class]])
+            icCnt = [(VGroup*)cg countRecursive]; // [[cg list] count];
 
-        for ( o=c+1; o < (int)[list count]; o++ )
-        {   NSAutoreleasePool	*pool1;
-            VGraphic		*og = [list objectAtIndex:o];
-            NSRect		oBounds;
-            BOOL		setcColorsNew = NO;
+        for (ic=0; ic < icCnt; ic++)
+        {   id	icg = cg;
 
-            if ( [og isKindOfClass:[VLine class]] || [og isKindOfClass:[VCurve class]] ||
-                 ([og isKindOfClass:[VArc class]] && (Abs([(VArc*)og angle]) != 360.0 || ![og filled])) ||
-                 ([og isKindOfClass:[VPath class]] && ![og filled]) ||
-                 ([og isKindOfClass:[VPolyLine class]] && ![cg filled]) )
-                continue;
+            if ([cg isKindOfClass:[VGroup class]])
+            {   icg = [(VGroup*)cg recursiveObjectAtIndex:ic]; // [[cg list] objectAtIndex:ic];
+                prevCnt = icCnt;
+            }
+            else
+                prevCnt = [list count];
 
-            pool1 = [NSAutoreleasePool new];
+            cColor = [[icg color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+            if ([icg respondsToSelector:@selector(fillColor)])
+            {
+                cf = [icg filled];
+                cFillColor = [[(VPath*)icg fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                cEndColor  = [[(VPath*)icg endColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+            }
 
-            // group must be in it self united
-            if ( [og isKindOfClass:[VGroup class]] )
-                [self uniteAreas:[(VGroup*)og list]];
+            pool = [NSAutoreleasePool new];
+            
+//prevCnt = [list count];
+            cBounds = [icg bounds];
 
-            oBounds = [og bounds];
-            //if ( !NSIsEmptyRect(NSIntersectionRect(cBounds , oBounds)) )
-            if (vhfIntersectsRect(cBounds , oBounds))
-            {   NSColor		*oColor = [[og color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-                VGraphic	*ng;
+            for ( o=c+1; o < (int)[list count]; o++ )
+            {   NSAutoreleasePool	*pool1;
+                id			og = [list objectAtIndex:o];
+                NSRect			oBounds;
+                BOOL			setcColorsNew = NO;
+                int			io=0, ioCnt = 1;
 
-                of = 0;
-                ow = [og width];
-                if ([og respondsToSelector:@selector(fillColor)])
-                {
-                    of = [og filled];
-                    oFillColor = [[(VPath*)og fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-                    oEndColor  = [[(VPath*)og endColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                if ( [og isKindOfClass:[VLine class]] || [og isKindOfClass:[VCurve class]] ||
+                     ([og isKindOfClass:[VArc class]] && (Abs([(VArc*)og angle]) != 360.0 || ![og filled])) ||
+                     ([og isKindOfClass:[VPath class]] && ![og filled]) ||
+                     ([og isKindOfClass:[VPolyLine class]] && ![og filled]) ||
+                     ([og isKindOfClass:[VRectangle class]] && ![og filled]) )
+                    continue;
+
+                if ([og isKindOfClass:[VGroup class]])
+                {   VGroup	*grp = [[og copy] autorelease];
+
+                    [list removeObjectAtIndex:o];
+                    [list insertObject:grp atIndex:o];
+                    og = [list objectAtIndex:o];
+
+                    ioCnt = [og countRecursive]; // [[og list] count];
                 }
 
-                if ((((((cw && cw == ow) || (!cw && !ow && !cf && !of)) && [cColor isEqual:oColor]) ||
-                      (!cw && !ow && cf && cf == of)) &&
-                     ((cf >= 1 && cf == of && [cFillColor isEqual:oFillColor]) || (!cf && !of)) && // fillColors
-                     ((cf > 1 && cf == of && [cEndColor isEqual:oEndColor]) || (cf <= 1 && cf == of))) && // endColors
-                    (ng = [cg uniteWith:og]))
-//                if ( [cColor isEqual:oColor] && (ng = [cg uniteWith:og]) )
-                {
-                    [list removeObjectAtIndex:c];
-                    [list insertObject:ng atIndex:c];
-                    [list removeObjectAtIndex:o];	/* if the graphics are united we remove the og */
-                    cg = [list objectAtIndex:c];
-                    cBounds = [cg bounds];
-                    // cg will be removed perhaps -> we must set the c colors new !!!!
-                    setcColorsNew = YES;
-                    o--;
+                for (io=0; io < ioCnt; io++)
+                {   VGraphic	*iog = og;
+
+                    if ([og isKindOfClass:[VGroup class]])
+                        iog = [og recursiveObjectAtIndex:io]; // [[og list] objectAtIndex:io];
+
+                    oBounds = [iog bounds];
+                    pool1 = [NSAutoreleasePool new];
+
+                    if (vhfIntersectsRect(cBounds , oBounds))
+                    {   NSColor		*oColor = [[og color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                        VGraphic	*ng;
+
+                        of = 0;
+                        ow = [iog width];
+                        if ([iog respondsToSelector:@selector(fillColor)])
+                        {
+                            of = [iog filled];
+                            oFillColor = [[(VPath*)iog fillColor]
+                                                      colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                            oEndColor  = [[(VPath*)iog endColor]
+                                                      colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                        }
+
+                        if ((((((cw && cw == ow) || (!cw && !ow && !cf && !of)) && [cColor isEqual:oColor]) ||
+                              (!cw && !ow && cf && cf == of)) &&
+                             ((cf >= 1 && cf == of && [cFillColor isEqual:oFillColor]) || (!cf && !of)) && // fillCol
+                             ((cf > 1 && cf == of && [cEndColor isEqual:oEndColor]) || (cf <= 1 && cf == of))) && //end
+                            (ng = [icg uniteWith:iog]))
+//                	if ( [cColor isEqual:oColor] && (ng = [icg uniteWith:iog]) )
+                        {
+                            if ([cg isKindOfClass:[VGroup class]])
+                            {
+                                [(VGroup*)cg recursiveRemoveObjectAtIndex:ic];
+                                [(VGroup*)cg recursiveInsertObject:ng atIndex:ic];
+                                if ([og isKindOfClass:[VGroup class]])
+                                {   [og recursiveRemoveObjectAtIndex:io];
+                                    io--;
+                                }
+                                else
+                                {   [list removeObjectAtIndex:o]; // if the graphics are united we remove the og
+                                    o--;
+                                }
+                                icg = [(VGroup*)cg recursiveObjectAtIndex:ic];
+                                cBounds = [icg bounds];
+                                /* cg will be removed perhaps -> we must set the c colors new !!!! */
+                                setcColorsNew = YES;
+                            }
+                            else
+                            {
+                                [list removeObjectAtIndex:c];
+                                [list insertObject:ng atIndex:c];
+                                if ([og isKindOfClass:[VGroup class]])
+                                {   [og recursiveRemoveObjectAtIndex:io];
+                                    io--;
+                                }
+                                else
+                                {   [list removeObjectAtIndex:o]; // if the graphics are united we remove the og
+                                    o--;
+                                }
+                                cg = icg = [list objectAtIndex:c];
+                                cBounds = [icg bounds];
+                                /* cg will be removed perhaps -> we must set the c colors new !!!! */
+                                setcColorsNew = YES;
+                            }
+                        }
+                    }
+                    [pool1 release];
+                    if (setcColorsNew)
+                    {   cColor = [[icg color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                        if ([icg respondsToSelector:@selector(fillColor)])
+                        {
+                            cf = [icg filled];
+                            cFillColor = [[(VPath*)icg fillColor]
+                                                       colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                            cEndColor  = [[(VPath*)icg endColor]
+                                                       colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
+                        }
+                    }
                 }
             }
-            [pool1 release];
-            if (setcColorsNew)
-            {   cColor = [[cg color] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-                if ([cg respondsToSelector:@selector(fillColor)])
-                {
-                    cf = [cg filled];
-                    cFillColor = [[(VPath*)cg fillColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-                    cEndColor  = [[(VPath*)cg endColor] colorUsingColorSpaceName:@"NSCalibratedRGBColorSpace"];
-                }
+            /* if there is only one graphic left after uniting cg
+                * (is this also true for the last in list?) we do not optimize -> need c < [list count]-1
+                */
+            if ( [cg isKindOfClass:[VGroup class]] && prevCnt != icCnt && ic < icCnt-1 && ic != -1 )
+                ic--;
+            else if ( ![cg isKindOfClass:[VGroup class]] && prevCnt != (int)[list count] && c < (int)[list count]-1 )
+            {   if ( c != -1 )
+                    c--;
             }
-        }
-        /* if there is only one graphic left after uniting cg
-         * (is this also true for the last in list?) we do not optimize -> need c < [list count]-1
-         */
-        if ( prevCnt != (int)[list count] && c < (int)[list count]-1 )
-        {   if ( c != -1 )
-                c--;
-        }
-        else if ( [[list objectAtIndex:c] isKindOfClass:[VPath class]] )
-            /* optimize list only once for each united graphic */
+            else if ( [cg isKindOfClass:[VGroup class]] && [icg isKindOfClass:[VPath class]] )
+                [icg optimizeList:[(VPath*)icg list]];
+            else if ( ![cg isKindOfClass:[VGroup class]] && [[list objectAtIndex:c] isKindOfClass:[VPath class]] )
+                /* optimize list only once for each united graphic */
             //[self optimizeList:[(VPath*)[list objectAtIndex:c] list]];
-            [[list objectAtIndex:c] optimizeList:[(VPath*)[list objectAtIndex:c] list]];
-        [pool release];
+                [[list objectAtIndex:c] optimizeList:[(VPath*)[list objectAtIndex:c] list]];
+
+            [pool release];
+        }
     }
 }
 
@@ -376,7 +517,7 @@
         for ( j=0 ; j < [list count] ; j++ )
         {   VGraphic	*g2 = [list objectAtIndex:j];
 
-            if ( j == i  || [g2 isKindOfClass:[VPath class]] || [g1 isKindOfClass:[VRectangle class]]
+            if ( j == i  || [g2 isKindOfClass:[VPath class]] || [g2 isKindOfClass:[VRectangle class]]
                  || ([g2 isKindOfClass:[VArc class]] && Abs([(VArc*)g2 angle]) == 360)
                  || ([g2 isKindOfClass:[VPolyLine class]] && [g2 filled]) ) // can't be neighbour
                 continue;
@@ -402,6 +543,7 @@
 
 /* add graphics from splitList to list which are inside graphic
  * return YES if a graphic was added
+ * modified: 2006-06-07 (respondsToSelector:)
  */
 - (BOOL)addGraphics:(NSMutableArray*)splitList inside:(id)graphic toList:(NSMutableArray*)list
 {   int		cnt, i, status=NO;
@@ -417,7 +559,7 @@
         }
         ( [g isKindOfClass:[VPath class]] ) ? [[[(VPath*)g list] objectAtIndex:0] getPoint:&p at:0.4] : [g getPoint:&p at:0.4];
 
-        if ( [graphic isPointInside:p] )
+        if ( [graphic respondsToSelector:@selector(isPointInside:)] && [graphic isPointInside:p] )
         {   LONG	j, count=[list count];
 
             for ( j=0 ; j<count; j++ )
@@ -528,10 +670,11 @@
 /* return YES if cg is completly inside og
  */
 - (BOOL)removePartsOf:(VGraphic**)curG hiddenBy:(id)og
-{   int			i, iCnt, nothingRemoved = 0;
-    id			cg = *curG, ng;
-    NSMutableArray	*splitListOg = nil;
-    NSPoint		*iPts;
+{   int             i, iCnt, nothingRemoved = 0;
+    VGraphic        *cg = *curG;
+    VPath           *ng;
+    NSMutableArray  *splitListOg = nil;
+    NSPoint         *iPts;
 
     /* cg can't be inside a line or curve or part of arc */
     if ( !([og isKindOfClass:[VPath class]] && [og filled]) && !([og isKindOfClass:[VArc class]] && Abs([og angle]) == 360 && [og filled]) && !([og isKindOfClass:[VRectangle class]] && [og filled]) && !([og isKindOfClass:[VPolyLine class]] && [og filled]) )
@@ -542,11 +685,11 @@
     ng = [VPath path];
     [ng setColor:[cg color]];
     if ([cg respondsToSelector:@selector(fillColor)])
-    {   [ng setFillColor:[cg fillColor]];
-        [ng setEndColor:[cg endColor]];
-        [ng setRadialCenter:[cg radialCenter]];
-        [ng setStepWidth:[cg stepWidth]];
-        [ng setGraduateAngle:[cg graduateAngle]];
+    {   [ng setFillColor:    [(VPath*)cg fillColor]];
+        [ng setEndColor:     [(VPath*)cg endColor]];
+        [ng setRadialCenter: [(VPath*)cg radialCenter]];
+        [ng setStepWidth:    [(VPath*)cg stepWidth]];
+        [ng setGraduateAngle:[(VPath*)cg graduateAngle]];
     }
     [ng setFilled:[cg filled] optimize:NO];
     [ng setWidth:[cg width]];
@@ -583,7 +726,7 @@
         if ( [[ng list] count] == 1 )
         {   NSPoint	p;
 
-            ( [cg isKindOfClass:[VPath class]] ) ? [[[cg list] objectAtIndex:0] getPoint:&p at:0.4] : [cg getPoint:&p at:0.4];
+            ( [cg isKindOfClass:[VPath class]] ) ? [[[(VPath*)cg list] objectAtIndex:0] getPoint:&p at:0.4] : [cg getPoint:&p at:0.4];
             if ( ![og isPointInside:p] )
             {   *curG = [[cg copy] autorelease];
                 return NO;
@@ -612,7 +755,7 @@
     /* if cg is'nt tiled
      * the hiddenG (cg) is in the otherG or the otherG is in the hiddenG else -> FALSE
      */
-    if ( ([cg isKindOfClass:[VPath class]] && [[ng list] count] == [[cg list] count]) ||
+    if ( ([cg isKindOfClass:[VPath class]] && [[ng list] count] == [[(VPath*)cg list] count]) ||
         ([cg isKindOfClass:[VArc class]] && [[ng list] count] == 1) ||
         ([cg isKindOfClass:[VRectangle class]] && [[ng list] count] == 1) ||
         ([cg isKindOfClass:[VPolyLine class]] && [[ng list] count] == 1) )
@@ -621,14 +764,14 @@
         if (iCnt) free(iPts);
         /* cg inside og - cg will removed
         */
-        ( [cg isKindOfClass:[VPath class]] ) ? [[[cg list] objectAtIndex:0] getPoint:&p at:0.4] : [cg getPoint:&p at:0.4];
+        ( [cg isKindOfClass:[VPath class]] ) ? [[[(VPath*)cg list] objectAtIndex:0] getPoint:&p at:0.4] : [cg getPoint:&p at:0.4];
         if ( [og isPointInside:p] )
             return YES;
         /* og inside cg - add og to ng -> curG
          */
         ( [og isKindOfClass:[VPath class]] ) ? [[[(VPath*)og list] objectAtIndex:0] getPoint:&p at:0.4] : [og getPoint:&p at:0.4];
         if ( ([cg isKindOfClass:[VPath class]] || [cg isKindOfClass:[VArc class]] ||
-             [cg isKindOfClass:[VPolyLine class]]) && [cg isPointInside:p] )
+              [cg isKindOfClass:[VRectangle class]] || [cg isKindOfClass:[VPolyLine class]]) && [(VRectangle*)cg isPointInside:p] )
         {
             if ( [og isKindOfClass:[VPath class]] )
                 for (i=0; i<(int)[[(VPath*)og list] count]; i++)
