@@ -1,29 +1,29 @@
 /* PerformanceMap.m
  * Map of graphic objects for optimized access
  *
- * Copyright (C) 1993-2005 by vhf interservice GmbH
- * Authors:  T+T Hennerich (1993), Georg Fleischmann (2001), Ilonka Fleischmann (2005)
+ * Copyright (C) 1993-2014 by Cenon GmbH
+ * Authors:  T+T Hennerich (1993), Georg Fleischmann (2001-), Ilonka Fleischmann (2005-)
  *
  * created:  1993 T+T Hennerich, 2001-08-17 Georg Fleischmann
- * modified: 2005-03-12
+ * modified: 2008-05-03 (autorelease pool)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.cenon.de
  */
 
 #include <Foundation/Foundation.h>
@@ -60,17 +60,17 @@
 
     [string appendFormat:@"PerformanceMap: bounds = {%.2f %.2f %.2f %.2f} segments = %d objects = %d",
         bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height,
-        [segmentList count], [graphicList count]];
+        (unsigned)[segmentList count], (unsigned)[graphicList count]];
     return string;
 }
 - (NSString*)descriptionRecursive
 {   static int		recursionCnt = 0;
     NSMutableString	*string = [NSMutableString string];
-    int			i;
+    int             i;
 
     [string appendFormat:@"%d PerformanceMap: bounds = {%.2f %.2f %.2f %.2f} segments = %d objects = %d",
         recursionCnt, bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height,
-        [segmentList count], [graphicList count]];
+        (unsigned)[segmentList count], (unsigned)[graphicList count]];
     if ([graphicList count])
         [string appendFormat:@"\n%@", [graphicList description]];
     else
@@ -92,7 +92,7 @@
 {   int		i;
 
     newFrame = NSIntegralRect(newFrame);
-    if ( bounds.size.width == newFrame.size.width &&
+    if ( bounds.size.width  == newFrame.size.width &&
          bounds.size.height == newFrame.size.height )
         return;
     for (i = [glist count]-1; i>=0; i--)	// remove all references of object to segments
@@ -123,7 +123,7 @@
     [self addObjectList:glist withReferenceList:nil];
 }
 
-/* Neue glist initialisieren. Ausdehnung darf sich nicht gendert haben.
+/* Neue glist initialisieren. Ausdehnung darf sich nicht geaendert haben.
  * Macht nur Sinn, falls das Object (self) das startSegment der GraphicView ist
  */
 - (void)initList:(NSArray*)glist
@@ -175,10 +175,10 @@
 
 /* test if object is inside bounds
  */
-- (BOOL)isObjectInside:(id)g
-{   NSRect	objectBounds = [g bounds];	// bounds including line width and vertices
-    int		i, cnt;
-    VRectangle	*gRect;
+- (BOOL)isObjectInside:(VGraphic*)g
+{   NSRect      objectBounds = [g bounds];	// bounds including line width and vertices
+    int         i, cnt;
+    VRectangle  *gRect;
 
     /* object bounds entirely inside segment */
     if ( NSContainsRect(bounds, objectBounds) )
@@ -209,7 +209,7 @@
         return YES;
     /* object if filled and point of segment is inside object */
     if ( [g filled] &&
-         [g respondsToSelector:@selector(isPointInside:)] && [g isPointInside:bounds.origin] )
+         [(VPath*)g respondsToSelector:@selector(isPointInside:)] && [(VPath*)g isPointInside:bounds.origin] )
         return YES;
     /* same test as before for images */
     if ( [g isKindOfClass:[VImage class]] && NSPointInRect(bounds.origin, [g bounds]) )
@@ -227,8 +227,8 @@
  * to achieve a better performance when drawing everything.
  */
 - (void)addObject:anObject inTryNumber:(int)try withReferenceList:(NSArray*)referenceList
-{   int 	i, count, objectIndex;
-    id 		segment;
+{   NSInteger   i, count, objectIndex;
+    id          segment;
 
     if (!segmentList && (int)[graphicList count] == capacity)			// limit of segment reached
         [self splitSegmentInTryNumber:++try withReferenceList:referenceList];	// -> request sub segments
@@ -254,7 +254,7 @@
                 {
                     for (i=0; i<count; i++)
                     {
-                        if ((int)[referenceList indexOfObject:[graphicList objectAtIndex:i]] > objectIndex)
+                        if ((NSInteger)[referenceList indexOfObject:[graphicList objectAtIndex:i]] > objectIndex)
                         {   [graphicList insertObject:anObject atIndex:i];
                             break;
                         }
@@ -292,11 +292,12 @@
 
 /* Update object, if it is a member of the map
  * removes object from map and inserts it again
- * This method should only be send to the master map !
+ * Note: This method should only be send to the master map !
+ * Note: Slow method, only use when sizes of the object change
  */
 - (void)updateObject:(VGraphic*)anObject withReferenceList:(NSArray*)refList
-{   int			i;
-    NSMutableArray	*pmList = [anObject pmList];
+{   int             i;
+    NSMutableArray  *pmList = [anObject pmList];
 
     if (![pmList count])	// only for members
         return;
@@ -691,10 +692,12 @@
         NSRectClip(rect);
 
         for (i=0, count = [graphicList count]; i<count; i++)
-        {   VGraphic	*g = [graphicList objectAtIndex:i];
+        {   VGraphic            *g = [graphicList objectAtIndex:i];
+			NSAutoreleasePool   *pool = [NSAutoreleasePool new];
 
             //[g drawInRect:rect principal:view];	// from different segments
             [g drawWithPrincipal:view];
+			[pool release];
         }
 
         PSgrestore();
@@ -716,8 +719,8 @@
     {
 	PSgsave();
         [[NSColor blackColor] set];
-        [NSBezierPath setDefaultLineWidth:0.0];
-        [NSBezierPath strokeRect:NSMakeRect(bounds.origin.x, bounds.origin.y,
+        [NSBezierPath setDefaultLineWidth:[NSBezierPath defaultLineWidth]];
+        [NSBezierPath strokeRect:NSMakeRect(bounds.origin.x,   bounds.origin.y,
                                             bounds.size.width, bounds.size.height)];
         PSgrestore();
     }
@@ -746,6 +749,7 @@
 {
     [segmentList release];
     [graphicList release];
+    [super dealloc];
 }
 
 @end
