@@ -1,33 +1,35 @@
 /* VGraphic.m
  * Graphic object - root class
  *
- * Copyright (C) 1996-2011 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
+ * WWW:      http://www.cenon.info
  *
  * created:  1996-01-29
- * modified: 2011-05-28 (excluded, -isExcluded, -setExcluded:, +initialize: version=5)
+ * modified: 2013-01-15 (-coordCenter added)
+ *           2011-05-28 (excluded, -isExcluded, -setExcluded:, +initialize: version=5)
  *           2011-04-06 (-drawStartAtScale: added)
  *           2011-03-04 (-size, -setSize: default implementation to make it work for all objects)
  *           2010-07-28 (-setLabel:, -label added, version = 4)
  *           2010-07-08 (-transform: added)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: cenon@vhf.de
+ * http://www.vhf.biz
  */
 
 #include <AppKit/AppKit.h>
@@ -256,6 +258,7 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
 - copy
 {   NSMutableData   *data = [[NSMutableData alloc] init];
     NSArchiver      *ts;
+    NSUnarchiver    *ua;
     id              g;
 
     /* writes the path to a NSData object and reads it back from it */
@@ -263,9 +266,9 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
     [ts encodeRootObject:self];
     [ts release];
 
-    ts = [[NSUnarchiver alloc] initForReadingWithData:data];
-    g = [[ts decodeObject] retain];
-    [ts release];
+    ua = [[NSUnarchiver alloc] initForReadingWithData:data];
+    g = [[ua decodeObject] retain];
+    [ua release];
     [data release];
     [g setDirty:YES];
 
@@ -452,7 +455,7 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
     return nil;
 }
 
-/* created:  05.03.97
+/* created:  1997-03-05
  */
 - (void)drawAtAngle:(float)angle in:view
 {
@@ -464,7 +467,7 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
     [self drawWithPrincipal:view];
 }
 
-/* created:  05.03.97
+/* created:  1997-03-05
  */
 - (void)rotate:(float)angle
 {
@@ -498,7 +501,7 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
 	 
 }
 
-/* modified: 05.03.97
+/* modified: 1997-03-05
  */
 - (void)mirror
 {
@@ -594,7 +597,8 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
 
 /* direction
  */
-- (BOOL)isDirectionCCW			{ return isDirectionCCW; }
+- (BOOL)isDirectionCCW              { return isDirectionCCW; }
+- (void)setDirectionCCW:(BOOL)ccw   { /*NSLog(@"setDirectionCCW: not implemented");*/ } // there are checks for respondsToSelector !
 - (void)drawDirectionAtScale:(float)scaleFactor
 {   NSPoint		p0, p;
     float		a, scale = 1.0 / scaleFactor;
@@ -657,6 +661,18 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
 {
     return NSZeroRect;
 }
+/* created: 2013-01-15
+ * purpose: needed for rotating and mirroring
+ */
+- (NSPoint)coordCenter;
+{   NSRect	rect;
+    NSPoint	p;
+
+    rect = [self coordBounds];
+    p.x = rect.origin.x + rect.size.width/2.0;
+    p.y = rect.origin.y + rect.size.height/2.0;
+    return p;
+}
 /* bounds of object including width and vertices
  * never becomes zero!
  */
