@@ -1,11 +1,13 @@
 /* VGroup.m
  * Group of graphic objects
  *
- * Copyright (C) 1996-2012 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2012-07-17 (*pts=NULL, *iPts=NULL initialized)
+ * modified: 2014-09-02 (-setDirectionCCW:, -changeDirection  new)
+ *           2012-08-12 (-setSize: use coordBounds for correct scaling)
+ *           2012-07-17 (*pts=NULL, *iPts=NULL initialized)
  *           2012-04-20 (-coordBounds zero size Bounds are not recognized)
  *           2010-07-08 (relocated -setList:)
  *
@@ -15,22 +17,22 @@
  * - if the group is handled when Alternate is pressed then the objects inside are handled
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
@@ -111,7 +113,7 @@
 
     group = [[VGroup allocWithZone:[self zone]] init];
     [group setSelected:isSelected];
-    [group setLocked:isLocked];
+    [group setLocked:NO]; // was: isLocked
 /*    [group setFilled:filled optimize:NO];
     [group setWidth:width];
     [group setColor:color];
@@ -131,7 +133,7 @@
 {
     return list;
 }
-- (void)setList:(NSArray*)aList
+- (void)setList:(NSMutableArray*)aList
 {
     [list release];
     list = [aList retain];
@@ -670,7 +672,7 @@
 }*/
 
 - (void)setSize:(NSSize)newSize
-{   NSRect	bRect = [self bounds];
+{   NSRect	bRect = [self coordBounds];
     //int		i;
 
     //for (i=[list count]-1; i>=0; i--)
@@ -1191,15 +1193,47 @@
     return (BOOL)hit;
 }
 
+/* modified: 2014-09-02
+ * must be set allways, new, we do not now the direction anyway for text !
+ * else we need two functions, the same for new paths (not Apple, text)
+ */
+- (void)setDirectionCCW:(BOOL)ccw
+{   int i;
+
+    for (i=[list count]-1; i>=0; i--)
+    {	id obj = [list objectAtIndex:i];
+
+        if ([obj respondsToSelector:@selector(setDirectionCCW:)] )
+            [obj setDirectionCCW:ccw];
+    }
+    isDirectionCCW = ccw;
+    dirty = YES;
+}
+
+/* modified: 2014-09-01
+ */
+- (void)changeDirection
+{   int i;
+
+    for (i=[list count]-1; i>=0; i--)
+    {	id obj = [list objectAtIndex:i];
+
+        if ([obj respondsToSelector:@selector(setDirectionCCW:)] )
+            [obj setDirectionCCW:(isDirectionCCW) ? 0 : 1];
+    }
+    isDirectionCCW = (isDirectionCCW) ? 0 : 1;
+    dirty = YES;
+}
+
 /*
  * return a path representing the outline of us
  * the path holds two lines and two arcs
  * if we need not build a contour a copy of self is returned
  */
 - contour:(float)w
-{   VGroup		*group;
-    NSMutableArray	*glist;
-    int			i, cnt;
+{   VGroup          *group;
+    NSMutableArray  *glist;
+    int             i, cnt;
 
     group = [VGroup group];
     [group setColor:color];
