@@ -1,29 +1,31 @@
 /* VLine.m
  * 2-D Line object
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2010-02-18 (exit with right mouse click added)
+ * modified: 2014-10-09 (-contourOpenLeftOrRight: new)
+ *           2011-07-11 (-sqrDistanceGraphic:::, -sqrDistanceGraphic:; some new graphics recognized)
+ *           2010-02-18 (exit with right mouse click added)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
@@ -76,7 +78,7 @@
  * modified: 
  */
 - copy
-{   id		line = [[VLine allocWithZone:[self zone]] init];
+{   VLine   *line = [[VLine allocWithZone:[self zone]] init];
 
     [line setWidth:width];
     [line setSelected:isSelected];
@@ -129,16 +131,16 @@
     id			window = [view window];
     VLine		*drawLineGraphic;
     BOOL		ok = YES, dragging = NO, hitEdge = NO;
-    float		grid = 1.0 / [view scaleFactor];	// minimum accepted length
+    VFloat		grid = 1.0 / [view scaleFactor];	// minimum accepted length
     int			windowNum = [event windowNumber];
     BOOL		alternate = [(App*)NSApp alternate], inTimerLoop = NO;
 
     [[(App*)NSApp inspectorPanel] loadGraphic:self];	// set the values of the inspector to self
 
     /* get start location, convert window to view coordinates */
-    start = [view convertPoint:[event locationInWindow] fromView:nil];
-    hitPoint = start;
-    hitEdge = [view hitEdge:&hitPoint spare:self];	// snap to point
+    start     = [view convertPoint:[event locationInWindow] fromView:nil];
+    hitPoint  = start;
+    hitEdge   = [view hitEdge:&hitPoint spare:self];	// snap to point
     gridPoint = [view grid:start];			// set on grid
     if ( hitEdge &&
          ((gridPoint.x == start.x && gridPoint.y == start.y)  ||
@@ -576,6 +578,15 @@
     dirty = YES;
 }
 
+- (void)transform:(NSAffineTransform*)matrix
+{   NSSize  size = NSMakeSize(width, width);
+
+    size = [matrix transformSize:size];
+    width = (Abs(size.width) + Abs(size.height)) / 2;
+    p0 = [matrix transformPoint:p0];
+    p1 = [matrix transformPoint:p1];
+}
+
 - (void)scale:(float)x :(float)y withCenter:(NSPoint)cp
 {
     width *= (x+y)/2.0;
@@ -599,7 +610,7 @@
  */
 - (void)drawWithPrincipal:principal
 {   NSColor	*oldColor = nil;
-    float	defaultWidth;
+    VFloat	defaultWidth;
 
     /* colorSeparation */
     if (!VHFIsDrawingToScreen() && [principal separationColor])
@@ -687,7 +698,7 @@
  * third points are selected, then they are used by themselves.
  */
 - (NSRect)scrollRect:(int)pt_num inView:(id)aView
-{   float	knobsize;
+{   VFloat	knobsize;
     NSRect	aRect;
 
     if (pt_num == -1)
@@ -720,7 +731,7 @@
 - (void)constrainPoint:(NSPoint *)aPt andNumber:(int)pt_num toView:(DocView*)aView
 {   NSPoint	viewMax;
     NSRect	viewRect;
-    float	margin = MARGIN / [aView scaleFactor];
+    VFloat	margin = MARGIN / [aView scaleFactor];
 
     viewRect = [aView bounds];
     viewMax.x = viewRect.origin.x + viewRect.size.width;
@@ -888,6 +899,51 @@
     return NO;
 }
 
+- (id)contourOpenLeftOrRight:(float)w
+{   VLine		*line;
+    float		r, dx, dy, c;
+    NSPoint		ps[4];
+
+    /* no correction */
+    if ( (w == 0.0 && width == 0.0) )
+    {	line = [VLine line];
+        [line setWidth:Abs(w)];
+        [line setColor:color];
+        [line setVertices:p0 :p1];
+        [line setSelected:[self isSelected]];
+        return line;
+    }
+
+    r = Abs((width + w) / 2.0);	/* the amount of growth */
+
+    dx = p1.x - p0.x;
+    dy = p1.y - p0.y;
+    if (dx == 0.0 && dy == 0.0)
+        dx = 1.0;
+    c = sqrt(dx*dx+dy*dy);
+    ps[0].x = p0.x + dy*r/c;
+    ps[0].y = p0.y - dx*r/c;
+    ps[1].x = p0.x - dy*r/c;
+    ps[1].y = p0.y + dx*r/c;
+    ps[2].x = p1.x - dy*r/c;
+    ps[2].y = p1.y + dx*r/c;
+    ps[3].x = p1.x + dy*r/c;
+    ps[3].y = p1.y - dx*r/c;
+
+    line = [VLine line];
+
+    if ( w >= 0.0 ) // right of line - outside correction
+        [line setVertices:ps[0] :ps[3]];
+    else // left of line - inside correction
+        [line setVertices:ps[1] :ps[2]];
+
+    [line setWidth:0.0];
+    [line setColor:color];
+    [line setSelected:[self isSelected]];
+    
+    return line;
+}
+
 /*
  * return a path representing the outline of us
  * the path holds two lines and two arcs
@@ -950,7 +1006,7 @@
     [list addObject:line];
 
     for (i=[list count]-1; i>=0; i--)
-    {	id	g = [list objectAtIndex:i];
+    {	VGraphic    *g = [list objectAtIndex:i];
 
         [g setWidth:0.0];
         [g setColor:color];
@@ -1015,17 +1071,17 @@
 }
 
 - getListOfObjectsSplittedFrom:(NSPoint*)pArray :(int)iCnt
-{   NSMutableArray	*splitList = nil;
-    int			i, cnt = 0;
-    NSPoint		*ps = malloc((iCnt) * sizeof(NSPoint)), bufp;
-    NSRect		bounds;
-    float		dx, dy;
-    NSAutoreleasePool	*pool;
+{   NSMutableArray      *splitList = nil;
+    int                 i, cnt = 0;
+    NSPoint             *ps = malloc((iCnt) * sizeof(NSPoint)), bufp;
+    NSRect              bounds;
+    float               dx, dy;
+    NSAutoreleasePool   *pool;
 
     bounds = [self coordBounds];
     /* only important for horicontal and vertical lines! */
-    bounds.origin.x -= 15.0*TOLERANCE;
-    bounds.origin.y -= 15.0*TOLERANCE;
+    bounds.origin.x    -= 15.0*TOLERANCE;
+    bounds.origin.y    -= 15.0*TOLERANCE;
     bounds.size.width  += 30.0*TOLERANCE;
     bounds.size.height += 30.0*TOLERANCE;
     dx = p1.x - p0.x;
@@ -1124,6 +1180,7 @@
         iCnt = [g getIntersections:ppArray with:self];
     else
     {   NSLog(@"VLine, intersection with unknown class!");
+        *ppArray = NULL;
         return 0;
     }
 
@@ -1131,7 +1188,7 @@
         sortPointArray(*ppArray, iCnt, p0);
     else
     {	free(*ppArray);
-        *ppArray = 0;
+        *ppArray = NULL;
     }
 
     return iCnt;
@@ -1145,9 +1202,9 @@
         [g getVertices:&pv0 :&pv1];
         return [self sqrDistanceLine:pv0 :pv1 :pg1 :pg2];
     }
-    else if ([g isKindOfClass:[VArc class]])
-        return [g sqrDistanceLine:p0 :p1 :pg2 :pg1];
-    else if ([g isKindOfClass:[VRectangle class]])
+    else if ( [g isKindOfClass:[VPath     class]] || [g isKindOfClass:[VGroup     class]] ||
+              [g isKindOfClass:[VPolyLine class]] || [g isKindOfClass:[VRectangle class]] ||
+              [g isKindOfClass:[VCurve    class]] || [g isKindOfClass:[VArc       class]] )
         return [g sqrDistanceGraphic:self :pg2 :pg1];
     else
     {   NSLog(@"VLine, distance (with two nearest points) with unknown class!");
@@ -1158,8 +1215,9 @@
 
 - (float)sqrDistanceGraphic:g
 {
-    if ([g isKindOfClass:[VPath class]] || [g isKindOfClass:[VGroup class]] ||
-        [g isKindOfClass:[VPolyLine class]] || [g isKindOfClass:[VRectangle class]])
+    if ( [g isKindOfClass:[VPath     class]] || [g isKindOfClass:[VGroup     class]] ||
+         [g isKindOfClass:[VPolyLine class]] || [g isKindOfClass:[VRectangle class]] ||
+         [g isKindOfClass:[VCurve    class]] || [g isKindOfClass:[VArc       class]] )
         return [g sqrDistanceGraphic:self];
     else if ([g isKindOfClass:[VLine class]])
     {	NSPoint	pv0, pv1;
@@ -1167,8 +1225,6 @@
         [g getVertices:&pv0 :&pv1];
         return [self sqrDistanceLine:pv0 :pv1];
     }
-    else if ([g isKindOfClass:[VCurve class]] || [g isKindOfClass:[VArc class]])
-        return [g sqrDistanceLine:p0 :p1];
     else
     {   NSLog(@"VLine, distance with unknown class!");
         return -1.0;
@@ -1244,32 +1300,36 @@
     [g getPoint:0 :&s2];
     [g getPoint:3 :&e2];
     if ( (Diff(p0.x, s2.x) <= TOLERANCE && Diff(p0.y, s2.y) <= TOLERANCE &&
-         Diff(p1.x, e2.x) <= TOLERANCE && Diff(p1.y, e2.y) <= TOLERANCE) ||
-        (Diff(p0.x, e2.x) <= TOLERANCE && Diff(p0.y, e2.y) <= TOLERANCE &&
-         Diff(p1.x, s2.x) <= TOLERANCE && Diff(p1.y, s2.y) <= TOLERANCE) )
+          Diff(p1.x, e2.x) <= TOLERANCE && Diff(p1.y, e2.y) <= TOLERANCE) ||
+         (Diff(p0.x, e2.x) <= TOLERANCE && Diff(p0.y, e2.y) <= TOLERANCE &&
+          Diff(p1.x, s2.x) <= TOLERANCE && Diff(p1.y, s2.y) <= TOLERANCE) )
         return YES;
     return NO;
 }
 
+- (void)encodeWithCoder:(NSCoder *)aCoder
+{
+    [super encodeWithCoder:aCoder];
+    //[aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &p0, &p1];
+    [aCoder encodePoint:p0];    // 2012-01-08
+    [aCoder encodePoint:p1];
+}
 - (id)initWithCoder:(NSCoder *)aDecoder
 {   int	version;
 
     [super initWithCoder:aDecoder];
-    version = [aDecoder versionForClassName:@"Line"];
+    version = [aDecoder versionForClassName:@"VLine"];
     if ( version < 1 )
         [aDecoder decodeValuesOfObjCTypes:"{ff}{ff}", &p0, &p1];
     else
-        [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &p0, &p1];
+    {   //[aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &p0, &p1];
+        p0 = [aDecoder decodePoint];    // 2012-01-08
+        p1 = [aDecoder decodePoint];
+    }
 
     [self setParameter];
-
     return self;
 }
-- (void)encodeWithCoder:(NSCoder *)aCoder
-{
-    [super encodeWithCoder:aCoder];
-    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}{NSPoint=ff}", &p0, &p1];
-}
 
 /* archiving with property list
  */
@@ -1285,6 +1345,7 @@
     [super initFromPropertyList:plist inDirectory:directory];
     p0 = pointFromPropertyList([plist objectForKey:@"p0"]);
     p1 = pointFromPropertyList([plist objectForKey:@"p1"]);
+    [self setParameter];
     return self;
 }
 
