@@ -1,31 +1,33 @@
 /* VRectangle.m
  * 2-D Rectangle object
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
+ * WWW:      http://www.cenon.info
  *
  * created:  1996-09-17
- * modified: 2010-07-17 (-transform added, -scale: scales width correctly with negative scales (flip))
+ * modified: 2013-08-13 (-hit:fuzz: rotate p back before let path do the job)
+ *           2010-07-17 (-transform added, -scale: scales width correctly with negative scales (flip))
  *           2010-03-03 (-copy setDirectionCCW, -pathRepresentation edges build with r, not radius)
  *           2006-02-08
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@vhf.biz
+ * http://www.vhf.biz
  */
 
 #include <AppKit/AppKit.h>
@@ -138,7 +140,7 @@
     id		window = [view window];
     VRectangle 	*drawRectangleGraphic;
     BOOL	ok = YES, dragging = NO, hitEdge = NO, inTimerLoop = NO;
-    float	grid = 1.0 / [view scaleFactor];	// minimum accepted length
+    VFloat	grid = 1.0 / [view scaleFactor];	// minimum accepted length
     int		windowNum = [event windowNumber];
     //	BOOL	alternate = [NXApp alternate];
 
@@ -863,7 +865,7 @@
  * third points are selected, then they are used by themselves.
  */
 - (NSRect)scrollRect:(int)pt_num inView:(id)aView
-{   float	knobsize;
+{   VFloat	knobsize;
     NSRect	aRect;
 
     if (pt_num == -1)
@@ -1103,8 +1105,11 @@
         if ( !Prefs_SelectByBorder && filled && !r )
             return YES;
         if ( !Prefs_SelectByBorder && filled && r )
+        {
+            if ( rotAngle )
+                vhfRotatePointAroundCenter(&p, origin, rotAngle); // rotate back
             return [[self pathRepresentation] hit:p fuzz:fuzz];
-
+        }
         for ( i=(ccw)?0:4; (ccw)?i<4:i; (ccw)?i++:i-- )
         {
             j = (ccw) ? i+1 : i-1;
