@@ -1,29 +1,38 @@
 /* VText.m
  * 2-D Text object
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-02-15
- * modified: 2005-01-11
+ * modified: 2014-09-02 (-setDirectionCCW:, -getFlattenedObjectAt:.. new, set direction to group object)
+ *           2014-07-06 (-initWithFile: dummy added)
+ *           2012-02-28 (-drawWithPrincipal: set frame twice to make mirrored text visible)
+ *           2012-01-04 (-textDidEndEditing: if locked, we don't adjust the size of the text box)
+ *           2010-07-29 (-drawWithPrincipal:...) move -setTextContainerInset: before -setFrame to make mirror on Apple visible
+ *           2010-07-28 (+valueForKey:inArray:) new
+ *           2009-12-29 (-drawWithPrincipal:): for GNUstep renew drawText after rotAngle
+ *           2009-02-12 (-textDidEndEditing: add correction to height to make text appear safely on Apple)
+ *           2009-02-11 (-getFlattenedObjectAt: set yOffset for mirrored text)
+ *           2009-01-27 (-textDidEndEditing: remove empty text)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.de
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
@@ -37,8 +46,8 @@
 #include "../FlippedView.h"
 #include "../graphicsUndo.subproj/undo.h"
 
-static NSTextView	*drawText = nil;	/* shared Text object used for drawing */
-//static NSString		*fontListNoFill;	/* list of fonts which shouldn't be filled */
+static NSTextView	*drawText = nil;	// shared Text object used for drawing
+//static NSString	*fontListNoFill;	// list of fonts which shouldn't be filled
 //static BOOL 		displayModeFilled = YES;
 
 //#define FONTLIST_NAME	@"FontListNoFill"
@@ -93,7 +102,7 @@ static void initClassVars()
 + (void)initialize
 {   //NSString	*path;
 
-    [VText setVersion:7];
+    [VText setVersion:8];
 
     /* load font list of font which we should not fill */
     /*
@@ -114,6 +123,26 @@ static void initClassVars()
     return;
 }
 
+/* search given array of Graphics objects
+ * and return the substring (value) of a text with a given key
+ * ex: "#KEY#Returned String"
+ * created: 2010-07-28
+ */
++ (NSString*)valueForKey:(NSString*)key inArray:(NSArray*)array
+{   int i, cnt = [array count];
+
+    for ( i=0; i<cnt; i++ )
+    {   VText   *g = [array objectAtIndex:i];
+
+        if ( [g isKindOfClass:[VText class]] )
+        {   NSString    *string = [g string];
+
+            if ( [string hasPrefix:key] )
+                return [string substringFromIndex:[key length]];
+        }
+    }
+    return nil;
+}
 
 - (void)renewSharedText
 {
@@ -159,8 +188,21 @@ static void initClassVars()
     return [super init];
 }
 
+/* init from file
+ *
+ * created:  2014-07-06
+ * modified: 
+ */
+- initWithFile:(NSString*)fileName
+{
+    [super init];
+    NSLog(@"TODO: VText: -initWithFile:");
+    // TODO
+    return self;
+}
+
 /*
- * created: 25.09.95
+ * created: 1995-09-25
  * purpose: initializes all the stuff needed after a -read:
  */
 - (void)setParameter
@@ -171,7 +213,8 @@ static void initClassVars()
 
 - (NSString*)description
 {
-    return [NSString stringWithFormat:@"VText: bounds=%f %f %f %f string=%@ angle=%f serial=%d fitHoricontal=%d centerVertical=%d", bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height, [self string], rotAngle, isSerialNumber, fitHorizontal, centerVertical];
+    return [NSString stringWithFormat:@"VText: bounds=%f %f %f %f string=%@ angle=%f serial=%d fitHoricontal=%d centerVertical=%d",
+            bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height, [self string], rotAngle, isSerialNumber, fitHorizontal, centerVertical];
 }
 
 - (NSString*)title
@@ -182,11 +225,30 @@ static void initClassVars()
 - (BOOL)filled			{ return filled; }
 - (void)setFilled:(BOOL)flag	{ filled = flag; dirty = YES; }
 
+/* modified: 2005-06-10
+ */
+- (void)setColor:(NSColor *)col
+{
+    [self setFillColor:col];
+}
+
+/* modified: 2005-06-10
+ */
 - (void)setFillColor:(NSColor*)col
 {
+    if (attributedString)
+    {   NSTextStorage	*ts = [drawText textStorage];
+
+        [ts beginEditing];
+        [ts setAttributedString:attributedString];
+        [ts endEditing];
+        [drawText setTextColor:col];
+        [drawText setFrame:bounds];
+        [self setAttributedString:[ts attributedSubstringFromRange:NSMakeRange(0, [ts length])]];
+    }
+    [super setColor:col];
     if (fillColor) [fillColor release];
     fillColor = [col retain];
-    [self setColor:fillColor]; // copy else redo kill
     dirty = YES;
 }
 - (NSColor*)fillColor { return fillColor; }
@@ -208,21 +270,6 @@ static void initClassVars()
 - (void)setStepWidth:(float)sw	{ stepWidth = sw; dirty = YES; }
 - (float)stepWidth		{ return stepWidth; }
 
-- (void)setColor:(NSColor *)col
-{
-    if (attributedString)
-    {   NSTextStorage	*ts = [drawText textStorage];
-
-        [ts beginEditing];
-        [ts setAttributedString:attributedString];
-        [ts endEditing];
-        [drawText setTextColor:col];
-        [drawText setFrame:bounds];
-        [self setAttributedString:[ts attributedSubstringFromRange:NSMakeRange(0, [ts length])]];
-    }
-    [super setColor:col];
-}
-
 - (void)setTextPath:tPath	{ textPath = tPath; }
 
 - (NSString*)string
@@ -248,7 +295,8 @@ static void initClassVars()
     [drawText setHorizontallyResizable:YES];
     [drawText setVerticallyResizable:YES];
     [ts beginEditing];
-    [ts setAttributedString:attributedString];
+    if (attributedString)
+        [ts setAttributedString:attributedString];
     [ts endEditing];
     [drawText setAlignment:NSLeftTextAlignment];
     [drawText setString:s];
@@ -273,7 +321,7 @@ static void initClassVars()
     }
     if (color)
         [drawText setTextColor:color];
-    [drawText sizeToFit];	/* collapses to zero for text larger bounds ??? */
+    [drawText sizeToFit];	// collapses to zero for text larger bounds ???
 
     [self setAttributedString:[ts attributedSubstringFromRange:NSMakeRange(0, [ts length])]];
     [[drawText textContainer] setWidthTracksTextView:YES];
@@ -293,7 +341,7 @@ static void initClassVars()
         font = [aFont retain];
     }
     if (aColor)
-        [super setColor:aColor];
+        [self setColor:aColor];	// 2005-06-10
     [self setString:string lineHeight:lineHeight];
     dirty = YES;
 }
@@ -341,7 +389,6 @@ static void initClassVars()
             [self kernToFitHorizontal];
     }
 }
-
 - (void)replaceCharactersInRange:(NSRange)range withString:(NSString*)string
 {
     if (string)
@@ -352,6 +399,21 @@ static void initClassVars()
             [self kernToFitHorizontal];
     }
 }
+/* created: 2005-09-01
+ */
+- (void)replaceSubstring:(NSString*)substring withString:(NSString*)string
+{
+    if (string)
+    {   NSRange	range = [[attributedString string] rangeOfString:substring];
+
+        if (range.length)
+        {   [self makeMutableAttributedString];
+            [attributedString replaceCharactersInRange:range withString:string];
+            if ( [self fitHorizontal] )
+                [self kernToFitHorizontal];
+        }
+    }
+}
 
 - (void)makeMutableAttributedString
 {
@@ -413,10 +475,27 @@ static void initClassVars()
     viewBounds = [view bounds];
     dirty = YES;
 
+    /* Here we keep the frame of the editview always 100%, whatever happens */
+    /*{   NSRect  editFrame = [editView frame], newFrame = [view frame];
+
+        editView = [view editView];
+        newFrame.size.width = NSWidth([view frame]) / [view scaleFactor];
+        printf("editView = %.1f bounds = %.1f  newFrame = %.1f  scale = %.1f\n", editFrame.size.width, viewBounds.size.width, newFrame.size.width, [view scaleFactor]);
+        if ( Diff(newFrame.size.width, editFrame.size.width) > 0.1 )
+        {
+            newFrame.size.height = NSHeight([view frame]) / [view scaleFactor];
+            NSLog(@"Note for createText: editView corrected to view size {%.1f %.1f} -> {%.1f %.1f}",
+                  editFrame.size.width, editFrame.size.height, newFrame.size.width, newFrame.size.height);
+            [[view editView] setFrame:newFrame];
+        }
+    }*/
+
     return NSMouseInRect(bounds.origin, viewBounds, NO);
 }
 
 /*
+ * modified: 2006-09-20 (scroll text to visible)
+ *
  * Here we are going to use the shared field editor for the window to
  * edit the text in the VText.  First, we must end any other editing
  * that is going on with the field editor in this window using endEditingFor:.
@@ -573,6 +652,11 @@ static void initClassVars()
     [fe setDelegate:self];
     [editView addSubview:fe];
 
+/* doesn't work ! #ifdef __APPLE__    // Workaround: display Single Line Fonts
+    if ( VHFAntialiasing() && [[font fontName] hasSuffix:@"-1Line"] )
+        VHFSetAntialiasing(NO);
+#endif*/
+
     //[fe display];	/* redraw because although we may have black background we edit on a white background*/
 
     /* Make it the first responder.
@@ -590,6 +674,7 @@ static void initClassVars()
 	[fe setSelectedRange:(NSRange){0,0}];
 
     [fe toggleRuler:self];
+    [view scrollRectToVisible:frame];	// if ruler covers text -> scroll text to visible
 
     return YES;
 }
@@ -605,21 +690,18 @@ static void initClassVars()
 }
 
 - (void)editorFrameChanged:(NSNotification *)arg
-{
-    NSRect currentEditingFrame = [[arg object] frame];
+{   NSRect  currentEditingFrame = [[arg object] frame];
 
     if (!NSEqualRects(lastEditingFrame, NSZeroRect))
     {
         if (lastEditingFrame.size.width > currentEditingFrame.size.width)
-        {
-            NSRect updateRect = lastEditingFrame;
+        {   NSRect  updateRect = lastEditingFrame;
 
             updateRect.origin.x = currentEditingFrame.origin.x + currentEditingFrame.size.width;
             [self updateEditingViewRect:updateRect];
         }
         if (lastEditingFrame.size.height > currentEditingFrame.size.height)
-        {
-            NSRect updateRect = lastEditingFrame;
+        {   NSRect updateRect = lastEditingFrame;
 
             updateRect.origin.y = currentEditingFrame.origin.y + currentEditingFrame.size.height;
             [self updateEditingViewRect:updateRect];
@@ -782,6 +864,8 @@ static void initClassVars()
     dirty = YES;
 }
 
+/* reduce kerning to fit text in width of text box
+ */
 - (void)kernToFitHorizontal
 {   NSLayoutManager	*lm = [drawText layoutManager];
     NSRect		rect;
@@ -802,10 +886,11 @@ static void initClassVars()
     {
         [self setKerning:f+0.05];
         [self updateDrawText];
+        // NOTE: on Apple this method returns heights that are less than our lineHeight
         rect = [lm boundingRectForGlyphRange:NSMakeRange(0, [lm numberOfGlyphs])
                              inTextContainer:[[lm textContainers] objectAtIndex:0]];
-        l = rect.size.height / [self lineHeight];
-        if ( l <= lines )
+        l = (rect.size.height+0.5) / Min([[self font] pointSize], [self lineHeight]);
+        if ( l <= lines )   // no line break any more -> kerning is small enough or too small
         {
             if (!i || i>10)
             {   [self setKerning:f];
@@ -813,7 +898,7 @@ static void initClassVars()
             }
             min = f;
         }
-        else
+        else                // kerning is too big
             max = f;
     }
     [drawText setFrame:bounds];
@@ -900,23 +985,25 @@ static void initClassVars()
 }
 
 /* created:  1995-10-21
- * modified: 2002-12-04
+ * modified: 2012-02-27 (make sure angle is within 0 and 360 deg)
  * purpose:  rotate the graphic around cp
  */
 - (void)setAngle:(float)angle withCenter:(NSPoint)cp
 {
+    if (angle <= -360.0) angle += 360.0; // 2012-02-27
+    if (angle >=  360.0) angle -= 360.0;
+
     if (filled)
     {   graduateAngle -= angle;
-        if (graduateAngle < 0.0)
-            graduateAngle += 360.0;
-        if (graduateAngle > 360.0)
-            graduateAngle -= 360.0;
+        if (graduateAngle <    0.0) graduateAngle += 360.0;
+        if (graduateAngle >= 360.0) graduateAngle -= 360.0;
         //vhfRotatePointAroundCenter(&radialCenter, NSMakePoint(0.5, 0.5), -angle);
     }
     vhfRotatePointAroundCenter(&bounds.origin, cp, -angle);
     rotAngle -= angle;
-    if (rotAngle < 0.0)
-        rotAngle += 360;
+    //if (rotAngle <    0.0) rotAngle += 360.0;
+    //if (rotAngle >= 360.0) rotAngle -= 360.0;   // 2012-02-27
+    rotAngle = vhfModulo(rotAngle, 360.0);
     dirty = YES;
 }
 
@@ -935,13 +1022,31 @@ static void initClassVars()
     dirty = YES;
 }
 
+- (void)setDirectionCCW:(BOOL)ccw
+{
+    if ( ccw != isDirectionCCW )
+        isDirectionCCW = (isDirectionCCW) ? 0 : 1;
+}
+
 /*
  * draws the text
  */
 - (void)drawWithPrincipal:principal
-{   id			window = [principal window];
-    BOOL		saveAutoDisplay = [window isAutodisplay];
-    NSLayoutManager	*lm = [drawText layoutManager];
+{   id              window = [principal window];
+    BOOL            saveAutoDisplay = [window isAutodisplay];
+    NSLayoutManager *lm = [drawText layoutManager];
+
+#ifdef __APPLE__    // Workaround: display Single Line Fonts
+    BOOL    antialias = VHFAntialiasing();
+    if ( antialias && [[font fontName] hasSuffix:@"-1Line"] )
+        VHFSetAntialiasing(NO);
+    /*if ( NSAppKitVersionNumber < NSAppKitVersionNumber10_6  // < 10.6
+         && [[font fontName] hasSuffix:@"-1Line"] )         // Single Line Font
+    {   VPath   *path = [self getFlattenedObjectAt:NSMakePoint(0.0, 0.0) withOffset:0];
+        [path drawWithPrincipal:principal];
+        return;
+    }*/
+#endif
 
     editView = [principal editView];	// needed in flatten object
 
@@ -951,8 +1056,8 @@ static void initClassVars()
         [[drawText textStorage] beginEditing];
         /* check color */
         if ((!VHFIsDrawingToScreen() && [principal separationColor]) || [principal mustDrawPale])
-        {   NSMutableAttributedString	*mas = [attributedString mutableCopyWithZone:[self zone]];
-            int		i, cnt = [mas length];
+        {   NSMutableAttributedString   *mas = [attributedString mutableCopyWithZone:[self zone]];
+            int                         i, cnt = [mas length];
 
             for (i=0; i<cnt; i++)
             {   NSRange	range;
@@ -963,7 +1068,7 @@ static void initClassVars()
                 if (!colorAtt)
                     colorAtt = fillColor;
                 if ([principal mustDrawPale])
-                {   float	h, s, b, a;
+                {   CGFloat h, s, b, a;
 
                     [[colorAtt colorUsingColorSpaceName:NSDeviceRGBColorSpace] getHue:&h
                                                                            saturation:&s
@@ -982,12 +1087,12 @@ static void initClassVars()
         }
 #if !defined(GNUSTEP_BASE_VERSION) && !defined(__APPLE__)	// OpenStep 4.2
         else if (VHFIsDrawingToScreen() && [[color colorSpaceName] isEqualToString:NSDeviceCMYKColorSpace])
-        {   NSMutableAttributedString	*mas = [attributedString mutableCopyWithZone:[self zone]];
-            int		i, cnt = [mas length];
+        {   NSMutableAttributedString   *mas = [attributedString mutableCopyWithZone:[self zone]];
+            int                         i, cnt = [mas length];
 
             for (i=0; i<cnt; i++)
-            {   NSRange			range;
-                NSColor			*colorAtt;
+            {   NSRange range;
+                NSColor *colorAtt;
 
                 colorAtt = [mas attribute:@"NSColor" atIndex:i longestEffectiveRange:&range
                                   inRange:NSMakeRange(0, cnt)];
@@ -1006,14 +1111,15 @@ static void initClassVars()
             [[drawText textStorage] setAttributedString:attributedString];
 
         [[drawText textStorage] endEditing];
+        [drawText setTextContainerInset:NSMakeSize(0.0, 0.0)];  // 2010-07-29: moved before setFrame: for Apple mirror
         [drawText setFrame:bounds];
-        //[drawText writeRTFDToFile:@"/tmp/text.rtf" atomically:NO];	// test
+        [drawText setFrame:bounds]; // 2012-02-28: well, without this extra invitation Apple doesn't draw mirrored text !
+        //[drawText writeRTFDToFile:@"/tmp/text.rtf" atomically:NO];	// debugging
         //[drawText setFrame:bounds];
 
-        [drawText setTextContainerInset:NSMakeSize(0.0, 0.0)];
         if ( centerVertical )	// vertical center: (bounds.size.height-textHeight)/2
         {   NSRect	rect = [lm boundingRectForGlyphRange:NSMakeRange(0, [lm numberOfGlyphs])
-                                             inTextContainer:[drawText textContainer]];
+                                         inTextContainer:[drawText textContainer]];
             [drawText setTextContainerInset:NSMakeSize(0.0, Max(0.0, (bounds.size.height-rect.size.height-1.0)/2.0))];
         }
 
@@ -1021,25 +1127,46 @@ static void initClassVars()
         [drawText scaleUnitSquareToSize:NSMakeSize(1.0, aspectRatio)];
         //displayModeFilled = filled;
         [window setAutodisplay:NO]; // don't let addSubview: cause redisplay (not wanted for resize)
-        [[NSView focusView] addSubview:drawText];
+        /* focusView is either cache or DocView
+         *   DocView        -> editView (FlippedView)
+         *                  -> drawText
+         *   cache (NSView) -> drawText
+         */
+        [[NSView focusView] addSubview:drawText];   // cache (NSView) / DocView -> drawText
         [drawText lockFocus];
         [drawText drawRect:[drawText bounds]];
         [drawText unlockFocus];
         [drawText removeFromSuperview];
         [window setAutodisplay:saveAutoDisplay];
+        //[[NSView focusView] setNeedsDisplay:NO];
         [drawText scaleUnitSquareToSize:NSMakeSize(1.0, 1.0/aspectRatio)];
+#   ifdef GNUSTEP_BASE_VERSION  // 2009-12-29 gui 0.17.1: GNUstep workaround, rotation screws up everything
+        if ( rotAngle != 0.0 )
+        {   [drawText release]; drawText = nil;
+            initClassVars();
+        }
+//#   else
+//        [drawText release]; drawText = nil;
+//        initClassVars();
+#   endif
+        //[drawText setFrameRotation:0.0];
     	/*if (DrawStatus == Resizing)
     	{   [[NSColor lightGrayColor] set];
             NSFrameRect(bounds);
     	}*/
     }
+
+#   ifdef __APPLE__    // Workaround: display Single Line Fonts
+    if (antialias)
+        VHFSetAntialiasing(antialias);
+#   endif
 }
 
 /* extract number from data, increment this number and add it to data. Then write string to drawText
  */
 - (void)setSerialTextFor:(NSTextView*)drawText withOffset:(int)o setData:(BOOL)setData
-{   int		i, lennew, lentot;
-    NSString	*string, *prefix, *valueStr;
+{   int         i, lennew, lentot;
+    NSString    *string, *prefix, *valueStr;
 
     if (attributedString && o)
     {	NSRange		range;
@@ -1192,7 +1319,7 @@ static void initClassVars()
 }
 
 - (NSRect)textBox		{ return bounds; }
-- (void)setTextBox:(NSRect)rect	{ bounds = rect; dirty = YES; }
+- (void)setTextBox:(NSRect)rect	{ bounds = rect; [self kernToFitHorizontal]; dirty = YES; } // 2008-02-05
 
 /*
  * Depending on the pt_num passed in, return the rectangle
@@ -1203,7 +1330,7 @@ static void initClassVars()
  * third points are selected, then they are used by themselves.
  */
 - (NSRect)scrollRect:(int)pt_num inView:(id)aView
-{   float	knobsize;
+{   VFloat	knobsize;
     NSPoint	p;
     NSRect	aRect;
 
@@ -1456,15 +1583,17 @@ static void initClassVars()
  * We must remove the Text object from the view heirarchy and, since
  * this Text object is going to be reused, we must set its delegate
  * back to nil.
- *
  * For further explanation of the two "ack!" lines, see edit:in: above.
+ *
+ * modified: 2012-01-04 (if isLocked, we don't adjust the size of the text box)
+ *           2009-02-12
  */
 - (void)textDidEndEditing:(NSNotification *)notification
-{   NSTextView		*fe = [notification object];
-    NSRect		oldBounds;
-    NSPoint		o;
-    float		correction = 2.0;	/* ? we use this to increase the bounds to contain the text */
-    NSAttributedString	*as;
+{   NSTextView          *fe = [notification object];
+    NSRect              oldBounds;
+    NSPoint             o;
+    float               correction = 2.0;	// ? we use this to increase the bounds that contain the text
+    NSAttributedString  *as;
 
     if ( fe )
     {
@@ -1474,12 +1603,12 @@ static void initClassVars()
         if ([fe isRulerVisible])
             [fe toggleRuler:self];
 
-        if ( ![[fe string] length] )
-            [fe setString:@"*"];
-
-        if ( [[fe string] length] )
+        if ( [[fe string] length] || attributedString )
         {   id	change = nil;
 
+            if ( ![[fe string] length] )
+                [fe setString:@"EMPTY TEXT"];
+
             if ( attributedString )
             {   change = [[EndEditingGraphicsChange alloc] initGraphicView:graphicView graphic:self];
                 correction = 0.0;
@@ -1493,11 +1622,14 @@ static void initClassVars()
 
             oldBounds = [self extendedBoundsWithScale:[graphicView scaleFactor]];
             [fe setVerticallyResizable:YES];
-            [fe sizeToFit];
-            bounds = [fe frame];
-            bounds.size = [fe bounds].size;
-            bounds.size.width += correction;
-            bounds = [editView convertRect:bounds toView:graphicView];  // ack!
+            if ( ! isLocked )
+            {   [fe sizeToFit];
+                bounds = [fe frame];
+                bounds.size = [fe bounds].size;
+                bounds.size.width  += correction;
+                bounds.size.height += correction;   // origin = upper/left here!
+                bounds = [editView convertRect:bounds toView:graphicView];  // ack!
+            }
             //bounds = [graphicView centerScanRect:bounds];	/* center pixels */
             //redrawRect = NSUnionRect(bounds, redrawRect);
 
@@ -1510,8 +1642,9 @@ static void initClassVars()
             //[[graphicView window] disableFlushWindow];
             o = bounds.origin;
             o.y += bounds.size.height;
-            vhfRotatePointAroundCenter(&bounds.origin, o, rotAngle); /* new origin */
+            vhfRotatePointAroundCenter(&bounds.origin, o, rotAngle); // new origin
             oldBounds = NSUnionRect(bounds, oldBounds);
+            [[graphicView layerOfGraphic:self] updateObject:self];  // 2008-02-05
             [(DocView*)graphicView cache:oldBounds];
             //[graphicView draw:oldBounds];	/* update cache */
             //[(NSView*)graphicView lockFocus];
@@ -1577,7 +1710,16 @@ static void initClassVars()
 
 #ifdef __APPLE__
     {   NSBezierPath	*bezierPath = [NSBezierPath bezierPath];
-        NSFont		*fnt;
+        NSFont          *fnt;
+        float           yOffset = 0.0;
+
+        /* workaround for offset in older OS X versions, not knowing exactly when it was fixed
+         * It seems vertical centered text needs this offset.
+         */
+        if ( centerVertical /*|| NSAppKitVersionNumber < 824.42*/ )   // Mac OS X < 10.4.10
+            yOffset = Max(0.0, (bounds.size.height-rect.size.height-1.0)/2.0);
+        if ( aspectRatio < 0.0 )    // mirrored text
+            yOffset -= bounds.size.height; // Apple needs that
 
         cnt = [lm numberOfGlyphs];
         if (cnt > 1)
@@ -1590,20 +1732,23 @@ static void initClassVars()
             p = [lm locationForGlyphAtIndex:i];
             [bezierPath removeAllPoints];
             [bezierPath moveToPoint:NSMakePoint(bRect.origin.x+rect.origin.x+p.x,
-                                                bRect.origin.y+bRect.size.height-rect.origin.y-p.y)];
+                                                bRect.origin.y+bRect.size.height-rect.origin.y-p.y-yOffset)];
             fnt = [[drawText textStorage] attribute:NSFontAttributeName atIndex:i effectiveRange:NULL];
             [bezierPath appendBezierPathWithGlyph:[lm glyphAtIndex:i] inFont:fnt];
             pathG = [VPath pathWithBezierPath:bezierPath];
             [pathG setWidth:0.0];
+            //printf("y=%.0f h=%.0f lh=%.0f  oberkannte=%.0f  vorstellung=%.0f\n", bRect.origin.y, bRect.size.height, [self lineHeight], bRect.origin.y+bRect.size.height, bRect.origin.y+bRect.size.height-([self lineHeight]*aspectRatio/2.0));
+             /* /2.0 war bis 2007-10-26, tut nicht mit 10.4.10 (824.42) */
             if (aspectRatio != 1.0)
                 [pathG scale:1.0 :aspectRatio
-                  withCenter:NSMakePoint(1.0, bRect.origin.x+bRect.size.height/2.0)];
+                  withCenter:NSMakePoint(1.0, bRect.origin.y+bRect.size.height/1.0)];
             [pathG setAngle:-rotAngle withCenter:bRect.origin];
             /* FIXME: stroked fonts shouldn't be filled! */
             [pathG setFilled:YES optimize:NO];
-            if (group)
+            if ( group && [pathG count] )
                 [group addObject:pathG];
         }
+        [((group) ? (id)group : (id)pathG) setDirectionCCW:isDirectionCCW];
 
         return (group) ? (id)group : (id)pathG;
     }
@@ -1611,9 +1756,9 @@ static void initClassVars()
     /*
      * we draw the text inside editView, then writing it to postscript-data for interpretation
      */
-    {	PSImportSub	*psImport;
-        NSData		*psData;
-        NSMutableArray	*list;
+    {	PSImportSub     *psImport;
+        NSData          *psData;
+        NSMutableArray  *list;
 
         if (!editView)
         {
@@ -1670,11 +1815,13 @@ static void initClassVars()
                 [pathG setFillColor:[[[list objectAtIndex:i]fillColor]
                                      colorUsingColorSpaceName:NSDeviceCMYKColorSpace]];
             if (cnt>1)
-                [[group list] addObject:pathG];
+                [group addObject:pathG];	 //[[group list] addObject:pathG];
         }
         if ( !pathG )
             pathG = [VPath path];
 
+        [((group) ? (id)group : (id)pathG) setDirectionCCW:isDirectionCCW];
+
         /*if (fontListNoFill)
         {   NSRange	range = [fontListNoFill rangeOfString:[font fontName]];
             if (fontListNoFill && range.location >= 0)
@@ -1692,19 +1839,21 @@ static void initClassVars()
     [aCoder encodeValuesOfObjCTypes:"@", &attributedString];
     [aCoder encodeValuesOfObjCTypes:"@", &font];
     [aCoder encodeValuesOfObjCTypes:"ff", &rotAngle, &aspectRatio];
-    [aCoder encodeValuesOfObjCTypes:"{NSRect=ffff}", &bounds];
+    //[aCoder encodeValuesOfObjCTypes:"{NSRect=ffff}", &bounds];    // doesn't work with 64 bit !
+    [aCoder encodeRect:bounds];                 // 2012-01-08
     [aCoder encodeValuesOfObjCTypes:"ccc", &isSerialNumber, &fitHorizontal, &centerVertical];
     [aCoder encodeObject:fillColor];
     [aCoder encodeObject:endColor];
     [aCoder encodeValuesOfObjCTypes:"ff", &graduateAngle, &stepWidth];
-    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+    //[aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+    [aCoder encodePoint:radialCenter];          // 2012-01-08
 }
 - (id)initWithCoder:(NSCoder *)aDecoder
 {   int	version;
 
     [super initWithCoder:aDecoder];
     version = [aDecoder versionForClassName:@"VText"];
-    if ( version<4 )	// 13.02.00
+    if ( version < 4 )	// 13.02.00
     {   float	length;
         [aDecoder decodeValuesOfObjCTypes:"i", &length];
     }
@@ -1721,7 +1870,8 @@ static void initClassVars()
     if ( version < 5 )	// 04.05.00
         [aDecoder decodeValuesOfObjCTypes:"{ffff}", &bounds];
     else
-        [aDecoder decodeValuesOfObjCTypes:"{NSRect=ffff}", &bounds];
+        //[aDecoder decodeValuesOfObjCTypes:"{NSRect=ffff}", &bounds];
+        bounds = [aDecoder decodeRect];         // 2012-01-08
     if ( version < 2 )
         [aDecoder decodeValuesOfObjCTypes:"c", &isSerialNumber];
     else if ( version < 3)
@@ -1730,9 +1880,10 @@ static void initClassVars()
         [aDecoder decodeValuesOfObjCTypes:"ccc", &isSerialNumber, &fitHorizontal, &centerVertical];
     if ( version >= 6)
     {   fillColor = [[aDecoder decodeObject] retain];
-        endColor = [[aDecoder decodeObject] retain];
+        endColor  = [[aDecoder decodeObject] retain];
         [aDecoder decodeValuesOfObjCTypes:"ff", &graduateAngle , &stepWidth];
-        [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+        //[aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
+        radialCenter = [aDecoder decodePoint];  // 2012-01-08
     }
     if ( version < 1 )
     {   float	f;
@@ -1750,24 +1901,24 @@ static void initClassVars()
 - (id)propertyList
 {   NSMutableDictionary	*plist = [super propertyList];
 
-    [plist setObject:propertyListFromFloat(rotAngle) forKey:@"rotAngle"];
-    [plist setObject:propertyListFromFloat(aspectRatio) forKey:@"aspectRatio"];
+    [plist setObject:propertyListFromFloat(rotAngle)            forKey:@"rotAngle"];
+    [plist setObject:propertyListFromFloat(aspectRatio)         forKey:@"aspectRatio"];
     [plist setObject:[self richTextData] forKey:@"data"];
-    [plist setObject:propertyListFromNSRect(bounds) forKey:@"bounds"];
-    if (isSerialNumber) [plist setObject:@"YES" forKey:@"isSerialNumber"];
-    if (fitHorizontal) [plist setObject:@"YES" forKey:@"fitHorizontal"];
-    if (centerVertical) [plist setObject:@"YES" forKey:@"centerVertical"];
+    [plist setObject:propertyListFromNSRect(bounds)             forKey:@"bounds"];
+    if (isSerialNumber) [plist setObject:@"YES"                 forKey:@"isSerialNumber"];
+    if (fitHorizontal)  [plist setObject:@"YES"                 forKey:@"fitHorizontal"];
+    if (centerVertical) [plist setObject:@"YES"                 forKey:@"centerVertical"];
 
     if (fillColor != [NSColor blackColor])
-        [plist setObject:propertyListFromNSColor(fillColor) forKey:@"fillColor"];
+        [plist setObject:propertyListFromNSColor(fillColor)     forKey:@"fillColor"];
     if (endColor != [NSColor blackColor])
-        [plist setObject:propertyListFromNSColor(endColor) forKey:@"endColor"];
+        [plist setObject:propertyListFromNSColor(endColor)      forKey:@"endColor"];
     if (graduateAngle)
-        [plist setObject:propertyListFromFloat(graduateAngle) forKey:@"graduateAngle"];
+        [plist setObject:propertyListFromFloat(graduateAngle)   forKey:@"graduateAngle"];
     if (stepWidth != 7)
-        [plist setObject:propertyListFromFloat(stepWidth) forKey:@"stepWidth"];
+        [plist setObject:propertyListFromFloat(stepWidth)       forKey:@"stepWidth"];
     if (radialCenter.x != 0.5 && radialCenter.y != 0.5)
-        [plist setObject:propertyListFromNSPoint(radialCenter) forKey:@"radialCenter"];
+        [plist setObject:propertyListFromNSPoint(radialCenter)  forKey:@"radialCenter"];
 
     return plist;
 }
@@ -1775,17 +1926,18 @@ static void initClassVars()
 {
     [self setParameter];
     [super initFromPropertyList:plist inDirectory:directory];
-    rotAngle = [plist floatForKey:@"rotAngle"];
-    aspectRatio = [plist floatForKey:@"aspectRatio"];
-    [self setRichTextData:[plist objectForKey:@"data"]];
+    rotAngle       = [plist floatForKey:@"rotAngle"];
+    aspectRatio    = [plist floatForKey:@"aspectRatio"];
+    [self setRichTextData:[plist objectForKey:@"data"]];    // this also loads the text colors for paragraphs !
     bounds = rectFromPropertyList([plist objectForKey:@"bounds"]);
     isSerialNumber = ([plist objectForKey:@"isSerialNumber"] ? YES : NO);
-    fitHorizontal = ([plist objectForKey:@"fitHorizontal"] ? YES : NO);
+    fitHorizontal  = ([plist objectForKey:@"fitHorizontal"]  ? YES : NO);
     centerVertical = ([plist objectForKey:@"centerVertical"] ? YES : NO);
     [self font];
 
-    if (!(fillColor = colorFromPropertyList([plist objectForKey:@"fillColor"], [self zone])))
-        [self setFillColor:[color copy]];
+    fillColor = colorFromPropertyList([plist objectForKey:@"fillColor"], [self zone]);
+    /*if (!(fillColor = colorFromPropertyList([plist objectForKey:@"fillColor"], [self zone])))
+        [self setFillColor:[color copy]];   // destroys paragraph colors !*/
     if (!(endColor = colorFromPropertyList([plist objectForKey:@"endColor"], [self zone])))
         [self setEndColor:[NSColor blackColor]];
     graduateAngle = [plist floatForKey:@"graduateAngle"];
