@@ -1,29 +1,30 @@
 /* IPAllLayers.m
  * Layer management Inspector for all objects
  *
- * Copyright (C) 2002-2005 by vhf interservice GmbH
+ * Copyright (C) 2002-2014 by Cenon GmbH
  * Author:   Ilonka Fleischmann, Georg Fleischmann
  *
  * created:  2002-06-27
- * modified: 2005-09-01 (multi page stuff added)
+ * modified: 2011-01-10 (click on layer ends editing of text)
+ *           2006-01-13
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include "../App.h"
@@ -57,7 +58,7 @@
 }
 
 /* created:  1993-07-22
- * modified: 2003-06-28
+ * modified: 2006-01-13
  * purpose:  set layer list for layer matrix (moveMatrix)
  *           this may be called from self or from the document
  */
@@ -78,7 +79,8 @@
     lastLayerList = theLayerList;
     moveCellCount = [theLayerList count];
 
-    [moveMatrix selectCellAtRow:0 column:0];	// GNUstep
+    [moveMatrix setAllowsEmptySelection:YES];	// GNUstep: allows selectedRow = -1
+    //[moveMatrix selectCellAtRow:0 column:0];	// GNUstep
     [moveMatrix renewRows:0 columns:0];
     for (i=0, cnt=[theLayerList count]; i<cnt && (layerObject=[theLayerList objectAtIndex:i]); i++)
     {	[moveMatrix addRow];
@@ -88,6 +90,7 @@
     [moveMatrix sizeToCells];
     if ([moveMatrix numberOfRows])
         [moveMatrix selectCellAtRow:[docView indexOfSelectedLayer] column:0];
+    [moveMatrix setAllowsEmptySelection:NO];	// GNUstep
     [[moveMatrix superview] display];	// redraw clipView to be sure to overdraw removed cells
 
     if ((layerObject = [[moveMatrix selectedCell] layerObject]))
@@ -111,7 +114,7 @@
  */
 - (void)update:sender
 {
-    docView = [[(App*)NSApp currentDocument] documentView];
+    docView = [window docView];
 
     /* fill the layer matrix (moveMatrix) */
     [self setLayerList:(docView) ? [docView layerList] : [self currentLayerList]];
@@ -207,8 +210,8 @@
 }
 
 - (void)removeLayer:sender
-{   int		row = [moveMatrix selectedRow];
-    LayerObject	*layerObject;
+{   int         row = [moveMatrix selectedRow];
+    LayerObject *layerObject;
 
     //if ( [[[moveMatrix selectedCell] layerObject] type] == LAYER_CLIPPING )
     //    return;
@@ -237,7 +240,6 @@
     [nameField setStringValue:[layerObject string]];
 
     [[docView document] setDirty:YES];
-    //[docView calcOutput];
     [docView drawAndDisplay];
 
     /* post notification, that layer xy has been removed, so other layer lists can update themselfes */
@@ -254,6 +256,8 @@
 
     /* update view */
     [[[(App*)NSApp currentDocument] documentView] selectLayerAtIndex:[self indexOfSelectedLayer]];
+
+    [[[(App*)NSApp currentDocument] window] endEditingFor:nil]; // end editing of text
 }
 
 - (void)doubleClickLayer:sender
@@ -266,8 +270,8 @@
  * called from IPLayerCell
  */
 - (BOOL)updateLayerLists
-{   int		i, cnt;
-    LayerObject	*layerObject;
+{   int         i, cnt;
+    LayerObject *layerObject;
 
     for ( i=0, cnt=0; (layerObject=[[moveMatrix cellAtRow:i column:0] layerObject]); i++ )
     {
@@ -283,11 +287,11 @@
  * modified: 2005-09-01
  */
 - (void)displayChanged:sender
-{   DocView		*view = [[(App*)NSApp currentDocument] documentView];
-    int			l, i;
-    NSMutableArray	*slayList = [view slayList], *layerList = [view layerList];
-    LayerObject		*selectedLayer = [[moveMatrix selectedCell] layerObject];
-    BOOL		layerStateChanged = NO;
+{   DocView         *view = [[(App*)NSApp currentDocument] documentView];
+    int             l, i;
+    NSMutableArray  *slayList = [view slayList], *layerList = [view layerList];
+    LayerObject     *selectedLayer = [[moveMatrix selectedCell] layerObject];
+    BOOL            layerStateChanged = NO;
 
     for ( l=0; l<(int)[layerList count]; l++ )
     {   LayerObject	*layerObject = [layerList objectAtIndex:l];
@@ -320,10 +324,11 @@
  * called from MoveMatrix to exchange two layers
  */
 - (void)matrixDidShuffleCellFrom:(int)from to:(int)to
-{   id		view = [[(App*)NSApp currentDocument] documentView];
-    int		i, masterCnt = [[view layerList] count];
-    id		obj, list;
-    NSArray	*array = [NSArray arrayWithObjects:[view layerList], [view slayList], nil];
+{   id              view = [[(App*)NSApp currentDocument] documentView];
+    int             i, masterCnt = [[view layerList] count];
+    id              obj;
+    NSMutableArray  *list;
+    NSArray         *array = [NSArray arrayWithObjects:[view layerList], [view slayList], nil];
 
     for ( i=0; i<(int)[array count]; i++ )
     {
