@@ -1,29 +1,29 @@
 /* IPLayerCell.m
  * part of IPAllLayers
  *
- * Copyright (C) 1996-2006 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * Created:  1996-03-07
  * Modified: 2006-11-08
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
@@ -82,12 +82,12 @@ static	NSImage	*colorImage = nil;
 }
 
 - (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
-{   NSRect	rect = cellFrame;
-    NSPoint	imageOrigin;
-    NSSize	imageSize = NSZeroSize;
-    float	textSize = 0.0;
+{   NSRect      rect = cellFrame;
+    NSPoint     imageOrigin;
+    NSSize      imageSize = NSZeroSize;
+    float       textSize = 0.0;
     static id	sharedTextCell = nil;
-    NSRect	splitRect = cellFrame;
+    NSRect      splitRect = cellFrame;
 
     [controlView lockFocus];
 
@@ -115,10 +115,26 @@ static	NSImage	*colorImage = nil;
 
     /* draw display state icon (eye) */
     if (showImage)
-    {
+    {   NSImage *img = ([layerObject state] ? altShowImage : showImage);
+
+#ifdef __APPLE__    // 10.6: composite is deprecated, drawAtPoint doesn't flip image, drawRect starts at 10.6
+        imageOrigin.y = NSMinY(cellFrame) + (NSHeight(cellFrame)-imageSize.height)/2.0;
+        //[img drawAtPoint:imageOrigin fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0];   // doesn't flip
+        if ( NSAppKitVersionNumber >= NSAppKitVersionNumber10_6 )   // >= 10.6
+        {   NSRect  dstRect;
+
+            dstRect.origin = imageOrigin;
+            dstRect.size   = [img size];
+            [img drawInRect:dstRect fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0 respectFlipped:YES hints:nil]; // >= 10.6
+        }
+        else    // <= 10.5
+        {   imageOrigin.y = NSMinY(cellFrame) + NSHeight(cellFrame) - (NSHeight(cellFrame) - imageSize.height)/2.0;
+            [img compositeToPoint:imageOrigin  operation:NSCompositeSourceOver];  // <= 10.6
+        }
+#else
         imageOrigin.y = NSMinY(cellFrame) + NSHeight(cellFrame) - (NSHeight(cellFrame) - imageSize.height)/2.0;
-        [([layerObject state] ? altShowImage : showImage) compositeToPoint:imageOrigin
-                                                                 operation:NSCompositeSourceOver];
+        [img compositeToPoint:imageOrigin  operation:NSCompositeSourceOver];  // <= 10.6
+#endif
         imageOrigin.x += IMAGEMARGIN + imageSize.width;
     }
 
@@ -126,10 +142,26 @@ static	NSImage	*colorImage = nil;
     if (editImage)
     {
         if ( [layerObject type] != LAYER_PASSIVE )
-        {
-            imageOrigin.y = NSMinY(cellFrame)+NSHeight(cellFrame)-(NSHeight(cellFrame)-imageSize.height)/2.0;
-            [([layerObject editable] ? editImage : altEditImage) compositeToPoint:imageOrigin
-                                                                        operation:NSCompositeSourceOver];
+        {   NSImage *img = ([layerObject editable] ? editImage : altEditImage);
+
+#ifdef __APPLE__    // 10.6: composite is deprecated, drawAtPoint doesn't flip image, drawRect starts at 10.6
+            imageOrigin.y = NSMinY(cellFrame) + (NSHeight(cellFrame)-imageSize.height)/2.0;
+            //[img drawAtPoint:imageOrigin fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0];   // doesn't flip
+            if ( NSAppKitVersionNumber >= NSAppKitVersionNumber10_6 )   // >= 10.6
+            {   NSRect  dstRect;
+
+                dstRect.origin = imageOrigin;
+                dstRect.size   = [img size];
+                [img drawInRect:dstRect fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0 respectFlipped:YES hints:nil]; // >= 10.6
+            }
+            else    // <= 10.5
+            {   imageOrigin.y = NSMinY(cellFrame) + NSHeight(cellFrame)-(NSHeight(cellFrame)-imageSize.height)/2.0;
+                [img compositeToPoint:imageOrigin operation:NSCompositeSourceOver];
+            }
+#else
+            imageOrigin.y = NSMinY(cellFrame) + NSHeight(cellFrame)-(NSHeight(cellFrame)-imageSize.height)/2.0;
+            [img compositeToPoint:imageOrigin operation:NSCompositeSourceOver];
+#endif
         }
         imageOrigin.x += IMAGEMARGIN + imageSize.width;
     }
