@@ -1,33 +1,34 @@
 /* ImportController.m
  * Preferences module for Cenon imports
  *
- * Copyright (C) 1996-2004 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * Created:  1999-03-15
- * Modified: 2004-12-23
+ * Modified: 2006-02-24
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
-#include <VHFShared/vhfCommonFunctions.h>
+#include <VHFShared/VHFStringAdditions.h>	// +stringWithFloat:
+//#include <VHFShared/vhfCommonFunctions.h>
 #include "ImportController.h"
 #include "../../functions.h"		// fillPopup()
 #include "../../locations.h"		// HPGL_PATH ...
@@ -113,20 +114,20 @@
 
 
 - (void)update:sender
-{   int		i;
-    id		defaults = [NSUserDefaults standardUserDefaults];
-    NSString	*string;
+{   int         i;
+    id          defaults = [NSUserDefaults standardUserDefaults];
+    NSString    *string;
 
-    [dxfResField setStringValue:vhfStringWithFloat([[defaults objectForKey:@"dxfRes"] floatValue])];
-    [psPreferArcsSwitch  setState:([[defaults objectForKey:@"psPreferArcs"] isEqual:@"YES"]) ? 1 : 0];
+    [dxfResField setStringValue:[NSString stringWithFloat:[[defaults objectForKey:@"dxfRes"] floatValue]]];
+    [psPreferArcsSwitch  setState:([[defaults objectForKey:@"psPreferArcs"]  isEqual:@"YES"]) ? 1 : 0];
     [psFlattenTextSwitch setState:([[defaults objectForKey:@"psFlattenText"] isEqual:@"YES"]) ? 1 : 0];
     [[switchMatrix cellAtRow:SWITCH_COLORTOLAYER column:0]
       setState:([[defaults objectForKey:@"colorToLayer"] isEqual:@"YES"]) ? 1 : 0];
-    [[switchMatrix cellAtRow:SWITCH_FILLOBJECTS column:0]
-      setState:([[defaults objectForKey:@"fillObjects"] isEqual:@"YES"]) ? 1 : 0];
+    [[switchMatrix cellAtRow:SWITCH_FILLOBJECTS  column:0]
+      setState:([[defaults objectForKey:@"fillObjects"]  isEqual:@"YES"]) ? 1 : 0];
 
     /* set hpgl popup */
-    fillPopup(hpglPopup, HPGLPATH, @".dev", 0 );
+    fillPopup(hpglPopup, HPGLPATH, @".dev", 0, NO);
     /* set popup button title from the cell having hpglParmsFileName as title */
     string = [defaults objectForKey:@"hpglParmsFileName"];
     for (i=[hpglPopup numberOfItems]-1;  i>0; i--)
@@ -137,7 +138,7 @@
     [hpglPopup setAction:@selector(set:)];		// set the action
 
     /* set gerber popup */
-    fillPopup(gerberPopup, GERBERPATH, @".dev", 0 );
+    fillPopup(gerberPopup, GERBERPATH, @".dev", 0, NO);
     /* set popup button title from the cell having gerberParmsFileName as title */
     string = [defaults objectForKey:@"gerberParmsFileName"];
     for (i=[gerberPopup numberOfItems]-1; i>0; i--)
@@ -148,7 +149,7 @@
     [gerberPopup setAction:@selector(set:)];		// set the action
 
     /* set DIN popup */
-    fillPopup(dinPopup, DINPATH, @".dev", 1 );
+    fillPopup(dinPopup, DINPATH, @".dev", 1, NO);
     /* set popup button title from the cell having dinParmsFileName as title */
     string = [defaults objectForKey:@"dinParmsFileName"];
     for (i=[dinPopup numberOfItems]-1; i>0; i--)
@@ -186,7 +187,7 @@
     [defaults setObject:title forKey:@"gerberParmsFileName"];
 
     /* DXF resolution */
-    string = vhfStringWithFloat([dxfResField floatValue]);
+    string = [NSString stringWithFloat:[dxfResField floatValue]];
     //string = [NSString stringWithFormat:@"%f", [dxfResField floatValue]];
     if ( ![string isEqual:[defaults objectForKey:@"dxfRes"]] )
         [defaults setObject:string forKey:@"dxfRes"];
