@@ -1,33 +1,35 @@
 /* Type1Font.m
  * Type 1 Font object used for the type 1 import
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2000-11-26
- * modified: 2005-11-23
+ * modified: 2014-07-06 ("4 random bytes" with legal characters)
+ *           2012-02-07 (-writeToFile: 2x use -writeToFile:...encoding:error:)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
 #include <VHFShared/vhfCommonFunctions.h>
+#include <VHFShared/vhfSystemAdditions.h>   // -createDirectoryAtPath:recursive:
 #include "PreferencesMacros.h"
 #include "locations.h"
 #include "Graphics.h"
@@ -138,14 +140,14 @@
     if ( [pDict objectForKey:@"blueScale"] )
         privateDict.blueScale = [[pDict objectForKey:@"blueScale"] floatValue];
     privateDict.blueShift = [[pDict objectForKey:@"blueShift"] longValue];
-    if ( [[pDict objectForKey:@"blueValues"] count] == 6 )
+    if ( [(NSArray*)[pDict objectForKey:@"blueValues"] count] == 6 )
     {   int	i;
 
         for (i=0; i<6; i++)
             privateDict.blueValues[i] = [[[pDict objectForKey:@"blueValues"] objectAtIndex:i] floatValue];
     }
 
-    if ( [[pDict objectForKey:@"minFeature"] count] == 2 )
+    if ( [(NSArray*)[pDict objectForKey:@"minFeature"] count] == 2 )
     {   privateDict.minFeature[0] = [[[pDict objectForKey:@"minFeature"] objectAtIndex:0] floatValue];
         privateDict.minFeature[1] = [[[pDict objectForKey:@"minFeature"] objectAtIndex:1] floatValue];
     }
@@ -381,6 +383,7 @@ typedef struct _MetricsState
     capHeight = 0.0;
     xHeight = 0.0;
     descender = 0.0;
+    //ascender = 0.0;
     fbox[0] = fbox[1] = 10000; fbox[2] = fbox[3] = 0.0;
     // malloc metrics
     if ( metricsCnt ) free(metrics);
@@ -427,6 +430,8 @@ typedef struct _MetricsState
                 if ( descender > metrics[m].bbox[1] )
                     descender = metrics[m].bbox[1];
             }
+            //if ( ascender < metrics[m].bbox[3] )
+            //    ascender = metrics[m].bbox[3];
         }
         else if ( !strcmp(encoding[i].name, "space") )
         {   metrics[m].bbox[0] = metrics[m].bbox[1] = metrics[m].bbox[2] = metrics[m].bbox[3] = 0.0; }
@@ -492,7 +497,7 @@ typedef struct _MetricsState
     for (i=0, charStringCnt=0; i<encodingCnt;i++)
     {
         curPoint = (NSPoint){0.0, 0.0};
-//if ( !strcmp(encoding[i].name, "space") )
+//if ( !strcmp(encoding[i].name, "hyphen") )
 //    printf("stop");
         if ([self getCharStringFromList:charStrings+charStringCnt name:encoding[i].name index:encoding[i].index list:list])
             charStringCnt++;
@@ -505,7 +510,7 @@ typedef struct _MetricsState
     fontInfo.copyright = [NSString stringWithFormat:@"%s", COPYRIGHT];
 //	fontInfo.fullName = fontName;
 //	fontInfo.familyName = fontName;
-    fontInfo.version = @"001.000";
+    fontInfo.version = @"002.000";
     /* no overshoot positions */
     privateDict.blueValues[0] = privateDict.blueValues[1];
     privateDict.blueValues[3] = privateDict.blueValues[2];
@@ -598,8 +603,16 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
             if ([[obj color] redComponent] && [[obj color] greenComponent] && [obj isKindOfClass:[VLine class]]) /* stems */
             {   NSPoint	s, e;
                 [(VLine*)obj getVertices:&s :&e];
-                s.x = floor(s.x*fontState->scale.x); s.y = floor(s.y*fontState->scale.y);
-                e.x = floor(e.x*fontState->scale.x); e.y = floor(e.y*fontState->scale.y);
+                s.x = (s.x * fontState->scale.x);
+                s.x = (((s.x) - (int)(s.x)) > 0.7501) ? floor(s.x+1.0) : floor(s.x);
+                s.y = (s.y * fontState->scale.y);
+                s.y = (((s.y) - (int)(s.y)) > 0.7501) ? floor(s.y+1.0) : floor(s.y);
+                e.x = (e.x * fontState->scale.x);
+                e.x = (((e.x) - (int)(e.x)) > 0.7501) ? floor(e.x+1.0) : floor(e.x);
+                e.y = (e.y * fontState->scale.y);
+                e.y = (((e.y) - (int)(e.y)) > 0.7501) ? floor(e.y+1.0) : floor(e.y);
+//                s.x = floor(s.x*fontState->scale.x); s.y = floor(s.y*fontState->scale.y);
+//                e.x = floor(e.x*fontState->scale.x); e.y = floor(e.y*fontState->scale.y);
                 if ((Diff(s.x, e.x) <= 1.0) && vCnt<39)	/* vertical line */
                     vStems[vCnt++] = (s.x - fontState->leftPoint); // * fontState->scale.x;
                 else if ((Diff(s.y, e.y) <= 1.0) && hCnt<39)
@@ -617,7 +630,7 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
     //int		vCnt, hCnt;
 //int		vStems[40], hStems[40];
 
-    fontState.scale.x = fontState.scale.y = 10.0; // 1/0.1
+    fontState.scale.x = fontState.scale.y = 10.0; // 1/0.01 - Fix - ist nicht zu aendern
 
 //if ( !strcmp(name, "a") || !strcmp(name, "P") )
 //    printf("stop");
@@ -735,18 +748,30 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 }
 
 - (int)setGraphic:(id)obj :(unsigned char*)charCode
-{   int	len = 0;
-    int	dx, dy;
+{   int		len = 0;
+    int		dx, dy;
 
     if ( [obj isKindOfClass:[VLine class]] )
     {   NSPoint	s, e;
 
         [(VLine*)obj getVertices:&s :&e];
+        if ( Diff(s.x, e.x) < 0.05 && Diff(s.y, e.y) < 0.05 )
+            return 0; // we do not add this short line
+        s.x = (s.x * fontState.scale.x);
+        s.x = (((s.x) - (int)(s.x)) > 0.7501) ? floor(s.x+1.0) : floor(s.x);
+        s.y = (s.y * fontState.scale.y);
+        s.y = (((s.y) - (int)(s.y)) > 0.7501) ? floor(s.y+1.0) : floor(s.y);
+        e.x = (e.x * fontState.scale.x);
+        e.x = (((e.x) - (int)(e.x)) > 0.7501) ? floor(e.x+1.0) : floor(e.x);
+        e.y = (e.y * fontState.scale.y);
+        e.y = (((e.y) - (int)(e.y)) > 0.7501) ? floor(e.y+1.0) : floor(e.y);
+/*
         s.x = floor(s.x * fontState.scale.x); s.y = floor(s.y * fontState.scale.y);
         e.x = floor(e.x * fontState.scale.x); e.y = floor(e.y * fontState.scale.y);
-        if ( Diff(s.x, curPoint.x) > 0.01 || Diff(s.y, curPoint.y) > 0.01 || nocurrentpoint)
+*/
+        if ( Diff(s.x, curPoint.x) > 1.0 || Diff(s.y, curPoint.y) > 1.0 || nocurrentpoint) // 0.01 1.0
         {
-            // if (!nocurrentpoint)
+            //if (!nocurrentpoint)
             //     code[len++] = 9; /* closepath */
             /* dx dy rmoveto (21) */
             dx = (s.x - curPoint.x); // * fontState.scale.x;
@@ -756,9 +781,16 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
             charCode[len++] = 21;
             nocurrentpoint = 0;
         }
-        /* dx dy rlineto (5) */
-        dx = (e.x - s.x); // * fontState.scale.x;
-        dy = (e.y - s.y); // * fontState.scale.y;
+        if ( Diff(s.x, curPoint.x) <= 1.0 && Diff(s.y, curPoint.y) <= 1.0
+             && (s.x != curPoint.x || s.y != curPoint.y) )
+        {   dx = (e.x - curPoint.x); // else our deltas are wrong
+            dy = (e.y - curPoint.y);
+        }
+        else
+        {   /* dx dy rlineto (5) */
+            dx = (e.x - s.x); // * fontState.scale.x;
+            dy = (e.y - s.y); // * fontState.scale.y;
+        }
         len += encodeNumber(dx, charCode+len);
         len += encodeNumber(dy, charCode+len);
         charCode[len++] = 5;
@@ -768,11 +800,29 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
     {   NSPoint	p0, p1, p2, p3;
 
         [(VCurve*)obj getVertices:&p0 :&p1 :&p2 :&p3];
+        p0.x = (p0.x * fontState.scale.x);
+        p0.x = (((p0.x) - (int)(p0.x)) > 0.7501) ? floor(p0.x+1.0) : floor(p0.x);
+        p0.y = (p0.y * fontState.scale.y);
+        p0.y = (((p0.y) - (int)(p0.y)) > 0.7501) ? floor(p0.y+1.0) : floor(p0.y);
+        p1.x = (p1.x * fontState.scale.x);
+        p1.x = (((p1.x) - (int)(p1.x)) > 0.7501) ? floor(p1.x+1.0) : floor(p1.x);
+        p1.y = (p1.y * fontState.scale.y);
+        p1.y = (((p1.y) - (int)(p1.y)) > 0.7501) ? floor(p1.y+1.0) : floor(p1.y);
+        p2.x = (p2.x * fontState.scale.x);
+        p2.x = (((p2.x) - (int)(p2.x)) > 0.7501) ? floor(p2.x+1.0) : floor(p2.x);
+        p2.y = (p2.y * fontState.scale.y);
+        p2.y = (((p2.y) - (int)(p2.y)) > 0.7501) ? floor(p2.y+1.0) : floor(p2.y);
+        p3.x = (p3.x * fontState.scale.x);
+        p3.x = (((p3.x) - (int)(p3.x)) > 0.7501) ? floor(p3.x+1.0) : floor(p3.x);
+        p3.y = (p3.y * fontState.scale.y);
+        p3.y = (((p3.y) - (int)(p3.y)) > 0.7501) ? floor(p3.y+1.0) : floor(p3.y);
+/*
         p0.x = floor(p0.x * fontState.scale.x); p0.y = floor(p0.y * fontState.scale.y);
         p1.x = floor(p1.x * fontState.scale.x); p1.y = floor(p1.y * fontState.scale.y);
         p2.x = floor(p2.x * fontState.scale.x); p2.y = floor(p2.y * fontState.scale.y);
         p3.x = floor(p3.x * fontState.scale.x); p3.y = floor(p3.y * fontState.scale.y);
-        if ( Diff(p0.x, curPoint.x) > 0.01 || Diff(p0.y, curPoint.y) > 0.01 || nocurrentpoint)
+*/
+        if ( Diff(p0.x, curPoint.x) > 1.0 || Diff(p0.y, curPoint.y) > 1.0 || nocurrentpoint) // 0.01 1.0
         {
             /* dx dy rmoveto (21) */
             dx = (p0.x - curPoint.x); //  * fontState.scale.x;
@@ -782,9 +832,17 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
             charCode[len++] = 21;
             nocurrentpoint = 0;
         }
-        /* dx1 dy1 dx2 dy2 dx3 dy3 rrcurveto (8) */
-        dx = (p1.x - p0.x); // * fontState.scale.x;
-        dy = (p1.y - p0.y); // * fontState.scale.y;
+
+        if ( Diff(p0.x, curPoint.x) <= 1.0 && Diff(p0.y, curPoint.y) <= 1.0
+             && (p0.x != curPoint.x || p0.y != curPoint.y) )
+        {   dx = (p1.x - curPoint.x); // else our deltas are wrong
+            dy = (p1.y - curPoint.y);
+        }
+        else
+        {   /* dx1 dy1 dx2 dy2 dx3 dy3 rrcurveto (8) */
+            dx = (p1.x - p0.x); // * fontState.scale.x;
+            dy = (p1.y - p0.y); // * fontState.scale.y;
+        }
         len += encodeNumber(dx, charCode+len);
         len += encodeNumber(dy, charCode+len);
         dx = (p2.x - p1.x); // * fontState.scale.x;
@@ -804,12 +862,12 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 /* fileName = "PATH/FILENAME.font" (font directory)
  */
 - (BOOL)writeToFile:(NSString*)filename
-{   BOOL		savedOk = NO;
+{   BOOL            savedOk = NO;
     NSString		*afmfilename = nil;
     NSString		*backupFilename = [filename stringByAppendingString:@"~"];
     NSString		*fileDirectory = nil;
-    NSFileManager	*fileManager = [NSFileManager defaultManager];
-    BOOL		isDirectory = YES;
+    NSFileManager   *fileManager = [NSFileManager defaultManager];
+    BOOL            isDirectory = YES;
 
 //encoding = [self standardEncoding];
 //encodingCnt = [self standardEncodingCnt];
@@ -821,12 +879,13 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
         return NO;
     }
     /* rename to backup */
-    if ( ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename handler:nil]) || ([fileManager fileExistsAtPath:filename] && ![fileManager movePath:filename toPath:backupFilename handler:nil]) )
+    if ( ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename handler:nil]) ||
+         ([fileManager fileExistsAtPath:filename]       && ![fileManager movePath:filename toPath:backupFilename handler:nil]) )
     {   NSRunAlertPanel(SAVE_TITLE, CANT_CREATE_BACKUP, nil, nil, nil);
         return NO;
     }
     /* create file directory */
-    else if ( isDirectory && [fileManager createDirectoryAtPath:filename attributes:nil] )
+    else if ( isDirectory && [fileManager createDirectoryAtPath:filename recursive:NO attributes:nil error:NULL] )
     {
         fileDirectory = filename;
         filename = [fileDirectory stringByAppendingPathComponent:[[filename stringByDeletingPathExtension] lastPathComponent]];
@@ -835,7 +894,7 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 
     /* save */
     if ([fileManager isWritableFileAtPath:fileDirectory])
-    {   NSString		*fontStr, *eexecStr, *afmStr;
+    {   NSString    *fontStr, *eexecStr, *afmStr;
         int			i, len=0;
 
         fontStr = [NSString stringWithFormat:@"%%!FontType1-1.0: %@ %@\n", fontName, fontInfo.version];
@@ -855,15 +914,15 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 	fontStr = [fontStr stringByAppendingFormat:@"/Weight (%@) readonly def\n", fontInfo.weight];
 	fontStr = [fontStr stringByAppendingFormat:@"/ItalicAngle %d def\n", 0];
 	fontStr = [fontStr stringByAppendingFormat:@"/isFixedPitch %s def\n", (fontInfo.isFixedPitch)? "true" : "false"];
-	fontStr = [fontStr stringByAppendingFormat:@"/UnderlinePosition %d def\n", fontInfo.underlinePosition];
-	fontStr = [fontStr stringByAppendingFormat:@"/UnderlineThickness %d def\n", fontInfo.underlineThickness];
+	fontStr = [fontStr stringByAppendingFormat:@"/UnderlinePosition %ld def\n", fontInfo.underlinePosition];
+	fontStr = [fontStr stringByAppendingFormat:@"/UnderlineThickness %ld def\n", fontInfo.underlineThickness];
 	fontStr = [fontStr stringByAppendingFormat:@"end readonly def\n"];
 	/* font dict */
 	fontStr = [fontStr stringByAppendingFormat:@"/FontName /%@ def\n", fontName];
 	fontStr = [fontStr stringByAppendingFormat:@"/PaintType %d def\n", paintType];
 	if (paintType == 2)
 		fontStr = [fontStr stringByAppendingFormat:@"/StrokeWidth %d def\n", 10];
-	fontStr = [fontStr stringByAppendingFormat:@"/FontType %d def\n", fontType];
+	fontStr = [fontStr stringByAppendingFormat:@"/FontType %d def\n", fontType]; // 1 for type1 fonts
 //	m = fontMatrix;
 	fontStr = [fontStr stringByAppendingFormat:@"/FontMatrix [0.001 0 0 0.001 0 0] readonly def\n"];
 //	NXPrintf(stream, "/FontMatrix [%f %f %f %f %f %f] readonly def\n", m[0], m[1], m[2], m[3], m[4], m[5]);
@@ -871,13 +930,13 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 // Encoding !!!
 	fontStr = [fontStr stringByAppendingFormat:@"/Encoding StandardEncoding def\n"];
 	fontStr = [fontStr stringByAppendingFormat:@"/FontBBox {%.0f %.0f %.0f %.0f} readonly def\n", fontBBox[0], fontBBox[1], fontBBox[2], fontBBox[3]];
-	fontStr = [fontStr stringByAppendingFormat:@"/UniqueID %d def\n", uniqueID];
+	fontStr = [fontStr stringByAppendingFormat:@"/UniqueID %ld def\n", uniqueID];
 	fontStr = [fontStr stringByAppendingFormat:@"currentdict end\n"];
 
 	/* eexec */
 	fontStr = [fontStr stringByAppendingFormat:@"currentfile eexec\n"];
 
-        eexecStr = [NSString stringWithFormat:@"%s", "]|]"]; /* 4 random bytes */
+        eexecStr = [NSString stringWithFormat:@"%s", "]|]|"]; /* 4 random bytes */
 	eexecStr = [eexecStr stringByAppendingFormat:@"userdict"];
 	eexecStr = [eexecStr stringByAppendingFormat:@"/RD{string currentfile exch readstring pop}executeonly put\n"];
 	eexecStr = [eexecStr stringByAppendingFormat:@"userdict/ND{noaccess def}executeonly put\n"];
@@ -887,7 +946,7 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 	eexecStr = [eexecStr stringByAppendingFormat:@"/MinFeature{%.0f %.0f}noaccess def\n", privateDict.minFeature[0], privateDict.minFeature[1]];
 	eexecStr = [eexecStr stringByAppendingFormat:@"/password 5839 def\n"];
 	eexecStr = [eexecStr stringByAppendingFormat:@"/Source (%@) readonly def\n", privateDict.source];
-	eexecStr = [eexecStr stringByAppendingFormat:@"/UniqueID %d def\n", uniqueID];
+	eexecStr = [eexecStr stringByAppendingFormat:@"/UniqueID %ld def\n", uniqueID];
 	eexecStr = [eexecStr stringByAppendingFormat:@"/OtherSubrs[{}{}{}\n\
   {  %% coloring subr number on stack\
      systemdict /internaldict known not\n\
@@ -922,14 +981,14 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 	//for (i=0; i<subrsCnt; i++)
 	for (i=0; i<privateDict.subrsCnt; i++)
 	{   NSString	*str = [NSString stringWithCString:(char*)privateDict.subrs[i].proc
-                                                    length:privateDict.subrs[i].length];
+                                                length:privateDict.subrs[i].length];
 
             eexecStr = [eexecStr stringByAppendingFormat:@"dup %d %d RD ", i, privateDict.subrs[i].length];
             eexecStr = [eexecStr stringByAppendingString:str];
             eexecStr = [eexecStr stringByAppendingFormat:@" NP\n"];
 	}
-	eexecStr = [eexecStr stringByAppendingFormat:@"noaccess def\n", 0];
-	eexecStr = [eexecStr stringByAppendingFormat:@"end noaccess put\n", 0];
+	eexecStr = [eexecStr stringByAppendingFormat:@"noaccess def\n"];
+	eexecStr = [eexecStr stringByAppendingFormat:@"end noaccess put\n"];
 
 	/* char strings */
 	eexecStr = [eexecStr stringByAppendingFormat:@"dup /CharStrings %d dict dup begin\n", charStringCnt];
@@ -947,8 +1006,8 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
         }
 #endif
 	for (i=0; i<charStringCnt; i++)	/* georg */
-	{   NSString	*str=[NSString stringWithCString:(char*)charStrings[i].code
-                                                  length:charStrings[i].length];
+	{   NSString    *str=[NSString stringWithCString:(char*)charStrings[i].code
+                                              length:charStrings[i].length];
 
             eexecStr = [eexecStr stringByAppendingFormat:@"/%s %d RD ", charStrings[i].name, charStrings[i].length];
             eexecStr = [eexecStr stringByAppendingString:str];
@@ -958,61 +1017,65 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 	eexecStr = [eexecStr stringByAppendingFormat:@"readonly put\n"];
 	eexecStr = [eexecStr stringByAppendingFormat:@"dup/FontName get exch definefont pop\n"];
 	eexecStr = [eexecStr stringByAppendingFormat:@"mark currentfile closefile\n"];
-#if 0
-{ NSString	*file = [NSString stringWithFormat:@"/Net/nesquick/Users/ilonka/Tempo/Test/testSubrs"];
-[eexecStr writeToFile:file atomically:YES];
-}
-#endif
-	// NXGetMemoryBuffer(eexecStream, &buf, &len, &maxlen);
-    {   char	*eExecCStr, *buffer;
+#   if 0
+        { NSString	*file = [NSString stringWithFormat:@"/Net/nesquick/Users/ilonka/Tempo/Test/testSubrs"];
+            [eexecStr writeToFile:file atomically:YES];
+        }
+#   endif
+        // NXGetMemoryBuffer(eexecStream, &buf, &len, &maxlen);
+        {   char	*eExecCStr, *buffer;
 
-        len = [eexecStr length]; // +1 -> 0
-        buffer = malloc(len+1);
-        [eexecStr getCString:buffer];
-        eExecCStr = (char*)encryptEexec((unsigned char*)buffer, len);
-        fontStr = [fontStr stringByAppendingString:[NSString stringWithCString:eExecCStr length:len*2]];
-        free(eExecCStr);
-        free(buffer);
-    }
-	// NXWrite(stream, encryptEexec(buf, len), len*2);
+            len = [eexecStr length]; // +1 -> 0
+            buffer = malloc(len+1);
+            [eexecStr getCString:buffer];   // FIXME: use UTF8String and make a copy
+            eExecCStr = (char*)encryptEexec((unsigned char*)buffer, len);
+            fontStr = [fontStr stringByAppendingString:[NSString stringWithCString:eExecCStr length:len*2]];
+            free(eExecCStr);
+            free(buffer);
+        }
+        // NXWrite(stream, encryptEexec(buf, len), len*2);
 
-	fontStr = [fontStr stringByAppendingFormat:@"\n"];
-	for (i=0;i<512;i++)
-		fontStr = [fontStr stringByAppendingFormat:@"0"];
-	fontStr = [fontStr stringByAppendingFormat:@"\ncleartomark"];
+        fontStr = [fontStr stringByAppendingFormat:@"\n"];
+        for (i=0;i<512;i++)
+            fontStr = [fontStr stringByAppendingFormat:@"0"];
+        fontStr = [fontStr stringByAppendingFormat:@"\ncleartomark"];
 
-        savedOk = [fontStr writeToFile:filename atomically:YES];
+        //savedOk = [fontStr writeToFile:filename atomically:YES];
+        savedOk = [fontStr writeToFile:filename atomically:YES
+                              encoding:NSUTF8StringEncoding error:NULL];    // >= 10.5
 
         afmStr = [NSString stringWithFormat:@"StartFontMetrics 2.0\n"];
-	afmStr = [afmStr stringByAppendingFormat:@"Comment %s\n", COPYRIGHT];
-	afmStr = [afmStr stringByAppendingFormat:@"FontName %@\n", fontName];
-	afmStr = [afmStr stringByAppendingFormat:@"EncodingScheme AdobeStandardEncoding\n"];
-	afmStr = [afmStr stringByAppendingFormat:@"FullName %@\n", fontInfo.fullName];
-	afmStr = [afmStr stringByAppendingFormat:@"FamilyName %@\n", fontInfo.familyName];
-	afmStr = [afmStr stringByAppendingFormat:@"Weight %@\n", fontInfo.weight];
-	afmStr = [afmStr stringByAppendingFormat:@"ItalicAngle 0.0\n"];
-	afmStr = [afmStr stringByAppendingFormat:@"IsFixedPitch %s\n", (fontInfo.isFixedPitch) ? "true" : "false"];
-	afmStr = [afmStr stringByAppendingFormat:@"UnderlinePosition %d\n", fontInfo.underlinePosition];
-	afmStr = [afmStr stringByAppendingFormat:@"UnderlineThickness %d\n", fontInfo.underlineThickness];
-	afmStr = [afmStr stringByAppendingFormat:@"Version %@\n", fontInfo.version];
-	afmStr = [afmStr stringByAppendingFormat:@"Notice %s\n", COPYRIGHT];
-	afmStr = [afmStr stringByAppendingFormat:@"FontBBox %.0f %.0f %.0f %.0f\n", fontBBox[0], fontBBox[1], fontBBox[2], fontBBox[3]];
-	afmStr = [afmStr stringByAppendingFormat:@"CapHeight %.0f\n", capHeight];
-	afmStr = [afmStr stringByAppendingFormat:@"XHeight %.0f\n", xHeight];
-	afmStr = [afmStr stringByAppendingFormat:@"Descender %.0f\n", descender];
-	afmStr = [afmStr stringByAppendingFormat:@"Ascender %.0f\n", capHeight];
+        afmStr = [afmStr stringByAppendingFormat:@"Comment %s\n", COPYRIGHT];
+        afmStr = [afmStr stringByAppendingFormat:@"FontName %@\n", fontName];
+        afmStr = [afmStr stringByAppendingFormat:@"EncodingScheme AdobeStandardEncoding\n"];
+        afmStr = [afmStr stringByAppendingFormat:@"FullName %@\n", fontInfo.fullName];
+        afmStr = [afmStr stringByAppendingFormat:@"FamilyName %@\n", fontInfo.familyName];
+        afmStr = [afmStr stringByAppendingFormat:@"Weight %@\n", fontInfo.weight];
+        afmStr = [afmStr stringByAppendingFormat:@"ItalicAngle 0.0\n"];
+        afmStr = [afmStr stringByAppendingFormat:@"IsFixedPitch %s\n", (fontInfo.isFixedPitch) ? "true" : "false"];
+        afmStr = [afmStr stringByAppendingFormat:@"UnderlinePosition %ld\n", fontInfo.underlinePosition];
+        afmStr = [afmStr stringByAppendingFormat:@"UnderlineThickness %ld\n", fontInfo.underlineThickness];
+        afmStr = [afmStr stringByAppendingFormat:@"Version %@\n", fontInfo.version];
+        afmStr = [afmStr stringByAppendingFormat:@"Notice %s\n", COPYRIGHT];
+        afmStr = [afmStr stringByAppendingFormat:@"FontBBox %.0f %.0f %.0f %.0f\n", fontBBox[0], fontBBox[1], fontBBox[2], fontBBox[3]];
+        afmStr = [afmStr stringByAppendingFormat:@"CapHeight %.0f\n", capHeight];
+        afmStr = [afmStr stringByAppendingFormat:@"XHeight %.0f\n", xHeight];
+        afmStr = [afmStr stringByAppendingFormat:@"Descender %.0f\n", descender];
+        afmStr = [afmStr stringByAppendingFormat:@"Ascender %.0f\n", capHeight];
 
-	afmStr = [afmStr stringByAppendingFormat:@"StartCharMetrics %d\n", metricsCnt];
-	/* metrics */
-	for (i=0; i<metricsCnt; i++)
+        afmStr = [afmStr stringByAppendingFormat:@"StartCharMetrics %d\n", metricsCnt];
+        /* metrics */
+        for (i=0; i<metricsCnt; i++)
         {
             afmStr = [afmStr stringByAppendingFormat:@"C %d ; WX %.0f ; N %@ ; B %.0f %.0f %.0f %.0f ;\n", metrics[i].index, metrics[i].width, metrics[i].name, metrics[i].bbox[0], metrics[i].bbox[1], metrics[i].bbox[2], metrics[i].bbox[3]];
         }
-	afmStr = [afmStr stringByAppendingFormat:@"EndCharMetrics\n"];
+        afmStr = [afmStr stringByAppendingFormat:@"EndCharMetrics\n"];
         // composites
-	afmStr = [afmStr stringByAppendingFormat:@"EndFontMetrics\n"];
+        afmStr = [afmStr stringByAppendingFormat:@"EndFontMetrics\n"];
 
-        savedOk = [afmStr writeToFile:afmfilename atomically:YES];
+        //savedOk = [afmStr writeToFile:afmfilename atomically:YES];
+        savedOk = [afmStr writeToFile:afmfilename atomically:YES
+                             encoding:NSUTF8StringEncoding error:NULL]; // >= 10.5
     }
     else
         NSRunAlertPanel(SAVE_TITLE, DIR_NOT_WRITABLE, nil, nil, nil);
