@@ -1,30 +1,32 @@
 /* UpdateController.m
  * Checking for Updates...
  *
- * Copyright 2010-2012 by vhf interservice GmbH
+ * Copyright 2010-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2010-05-27
- * modified: 2012-02-07 (-connectionDidFinishLoading: use -writeToFile:...encoding:error:)
+ * modified: 2013-10-15 (cleanup warnings)
+ *           2012-09-24 (remove tar archives, but not flat pkg files, fixed in 4.0.1)
+ *           2012-02-07 (-connectionDidFinishLoading: use -writeToFile:...encoding:error:)
  *           2011-03-30 (-checkForUpdates: test for Prefs_DisableAutoUpdate)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: service@vhf.de
- * http://www.vhf-interservice.com
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.de
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
@@ -35,6 +37,7 @@
 #include "CenonModuleMethods.h"
 #include "PreferencesMacros.h"  // Prefs_DisableAutoUpdate
 #include <VHFShared/VHFStringAdditions.h>       // -writeToFile:...
+#include <VHFShared/VHFSystemAdditions.h>       // -createDirectoryAtPath:recursive:...
 
 static UpdateController *sharedInstance = nil;  // default object
 static NSString         *checkMarkStr = nil;    // the checkmark
@@ -69,7 +72,8 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     if ( [components count] == 1 )
         return [self fileExistsAtPath:path];
     path = [path stringByDeletingLastPathComponent];
-    if ( !(array = [self directoryContentsAtPath:path]) || ![components count] )
+    //if ( !(array = [self directoryContentsAtPath:path]) || ![components count] )    // <= OSX 10.4
+    if ( !(array = [self contentsOfDirectoryAtPath:path error:NULL]) || ![components count] )
         return NO;
     for ( i=0, cnt = [array count]; i<cnt; i++ )
     {   NSString    *file = [array objectAtIndex:i];
@@ -192,10 +196,11 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     NSString        *skipVersion     = [userDefaults objectForKey:@"skipUpdateVersion"];
     NSDate          *lastUpdateCheck = [userDefaults objectForKey:@"lastUpdateCheck"];
     int             i;
+    BOOL            com = NO;
 
+    if ( [lastUpdateCheck isKindOfClass:[NSString class]] ) // why ?
+        lastUpdateCheck = [NSDate dateWithString:(NSString*)lastUpdateCheck];
     isAutoCheck = ([sender isKindOfClass:[NSMenuItem class]]) ? NO : YES;
-    if ( isAutoCheck && Prefs_DisableAutoUpdate )
-        return; // automatic check is disabled in preferences
     if ( isAutoCheck && lastUpdateCheck &&
          [[NSDate date] timeIntervalSinceDate:lastUpdateCheck] < 3600*24*7 )
         return; // not yet one week -> no check
@@ -205,7 +210,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     [urlStr appendFormat:@"?n=%@&v=%@&d=%@", APPNAME, appVersion, appDate];
     if ( [skipVersion length] ) // version the user wants to skip
         [urlStr appendFormat:@"&sk=%@", skipVersion];
-    //[urlStr appenFormat:@"&p="];    // TODO: pass plattform to update script
+    //[urlStr appenFormat:@"&p="];    // TODO: pass plattform+version to update script to return correct package
     for (keys = [installedModules allKeys], i=0; i < [keys count]; i++)
     {   int         n = i+1;
         NSString    *name = [keys objectAtIndex:i];
@@ -220,20 +225,28 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         if ( [date    length] ) [urlStr appendFormat:@"&d%d=%@", n, date];
         if ( [serial  length] ) [urlStr appendFormat:@"&s%d=%@", n, serial];
         if ( [netId   length] ) [urlStr appendFormat:@"&o%d=%@", n, netId];     // origin
+        if ( [serial length] )
+            com = YES;
     }
+    if ( !com && isAutoCheck && Prefs_DisableAutoUpdate )
+        return; // automatic check is disabled in preferences
+    if ( Prefs_DisableAutoUpdate ) [urlStr appendFormat:@"&off"];
     connectionRequest = [NSURLRequest requestWithURL:[NSURL URLWithString:urlStr]
                                          cachePolicy:NSURLRequestReloadIgnoringCacheData
                                      timeoutInterval:60.0];
     urlConnection = [[NSURLConnection alloc] initWithRequest:connectionRequest delegate:self];
     if ( urlConnection )
-    {   connectionData = [[NSMutableData alloc] init];
+    {
+        [userDefaults setObject:[NSDate date] forKey:@"lastUpdateCheck"];   // date of this check
+        if ( isAutoCheck && Prefs_DisableAutoUpdate )
+            return; // automatic check is disabled in preferences
+        connectionData = [[NSMutableData alloc] init];
         [progressTitleText setStringValue:@""];
         [progressNameText setStringValue:@""];
         [progressSizeText setStringValue:@""];
         [progressIndicator setIndeterminate:YES];
         [progressIndicator startAnimation:nil];
         [progressPanel makeKeyAndOrderFront:self];
-        [userDefaults setObject:[NSDate date] forKey:@"lastUpdateCheck"];   // date of this check
     }
 #if 0   // this downloads a plist file directly (what, if it ends up in a proxy ?)
     NSURL           *url = [NSURL URLWithString:@"http://www.cenon.info/update/update.plist"];
@@ -251,6 +264,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     [download setDeletesFileUponFailure:YES];
     [download setDestination:path allowOverwrite:YES];
 #endif
+    /* this will bring us to: -connectionDidFinishLoading: -> -checkForUpdateAndDisplayPanel */
 }
 
 /* check for available updates, if available display panel
@@ -387,9 +401,17 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
             if ( [v appearanceCountOfCharacter:'.'] <= 1 )  // workaround for single dot version numbers "1.11" -> "1.1.1"
                 v = @"0.0.0"; // this is old anyway
-            if ( [[iArray objectAtIndex:0] compare:(id)v options:NSNumericSearch] == NSOrderedAscending )   // installed < version
+            if ( [[iArray objectAtIndex:0] compare:(id)v options:NSNumericSearch] == NSOrderedAscending )   // installed < new version
                 updateAvailable = YES;
-            // TODO: if version is the same, compare date
+            /* if version is the same, compare date [4.0.3 2014-02-24] */
+            else if ( [iArray count] >= 2 &&    // "4.0.2, 2014-02-24"
+                      [[iArray objectAtIndex:0] compare:(id)v options:NSNumericSearch] == NSOrderedSame )   // installed = new version
+            {   NSArray     *vArray = [updateDict objectForKey:@"version"];
+                NSString    *dateStr = ([vArray count] == 2) ? [vArray objectAtIndex:1] : nil;
+
+                if ( [[iArray objectAtIndex:1] compare:(id)dateStr] == NSOrderedAscending )                 // installed < new date
+                    updateAvailable = YES;
+            }
             else    // installed >= version
                 continue;
         }
@@ -433,7 +455,8 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
             price = nil;
             if ( priceDict )
-            {   NSString    *v = ([uDict objectForKey:@"mv"]) ? [uDict objectForKey:@"mv"] : version;
+            {   //NSString    *v = ([uDict objectForKey:@"mv"]) ? [uDict objectForKey:@"mv"] : version;
+                NSString    *v = ([iArray count]) ? [iArray objectAtIndex:0] : appVersion;  // version of installed module
                 NSRange     range = [v rangeOfString:@"." options:NSBackwardsSearch];
 
                 if ( range.length )
@@ -549,8 +572,8 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         [tableView selectRowIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
 
     if ( [updateFile hasSuffix:@"update.plist"] )
-        [[NSFileManager defaultManager] removeFileAtPath:updateFile handler:nil];   // <= 10.5
-        //[[NSFileManager defaultManager] removeItemAtPath:updateFile error:NULL];  // >= 10.6
+        [[NSFileManager defaultManager] removeItemAtPath:updateFile error:NULL];  // >= 10.5
+        //[[NSFileManager defaultManager] removeFileAtPath:updateFile handler:nil];   // <= 10.4
 
     if ( updateAvailable || !isAutoCheck )  // only display panel if update is available or manual
         [panel makeKeyAndOrderFront:self];
@@ -608,7 +631,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         NSURLRequest    *request = [NSURLRequest requestWithURL:url];
         NSString        *titleStr;
 
-        pkgDownload = [[NSURLDownload alloc] initWithRequest:request delegate:self];
+        pkgDownload = [[NSURLDownload alloc] initWithRequest:request delegate:(id)self];
         [pkgDownload setDeletesFileUponFailure:YES];
         pkgPath = path;
         [pkgDownload setDestination:pkgPath allowOverwrite:YES];
@@ -673,9 +696,10 @@ static NSString         *checkMarkStr = nil;    // the checkmark
         return;
     /* Checkmark row */
     if ( colIx == 0 )   // checkmark row
-    {   NSString        *colId = [[[sender tableColumns] objectAtIndex:colIx] identifier];
-        NSMutableArray  *colArray = [[[sender dataSource] dataDict] objectForKey:colId];
-        NSMutableArray  *keyArray = [[[sender dataSource] dataDict] objectForKey:@"updateKey"];
+    {   NSString        *colId    = [[[sender tableColumns] objectAtIndex:colIx] identifier];
+        NSDictionary    *dataDict = [(UpdateTableData*)[sender dataSource] dataDict];
+        NSMutableArray  *colArray = [dataDict objectForKey:colId];
+        NSMutableArray  *keyArray = [dataDict objectForKey:@"updateKey"];
         NSString        *boolStr = @"";
         NSString        *url = [[updateDict objectForKey:[keyArray objectAtIndex:rowIx]] objectForKey:@"d"];
 
@@ -710,6 +734,10 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     [connection release];
     [connectionData release]; connectionData = nil;
     [progressPanel orderOut:self];
+
+    if ( isAutoCheck && Prefs_DisableAutoUpdate )
+        return; // automatic check is disabled in preferences
+
     if (string && [string length] > 0)
     {   NSString    *updateFile;
 
@@ -815,17 +843,20 @@ static NSString         *checkMarkStr = nil;    // the checkmark
 
             /* cleanup files: .update.plist, .update/rtf-files, packages */
             path = vhfPathWithPathComponents(vhfUserLibrary(APPNAME), @".update.plist", nil);
-            [fileManager removeFileAtPath:path handler:nil];
+            [fileManager removeItemAtPath:path error:NULL];
+            //[fileManager removeFileAtPath:path handler:nil];    // <= 10.4
             path = vhfPathWithPathComponents(vhfUserLibrary(APPNAME), @".update", nil);
-            [fileManager removeFileAtPath:path handler:nil];
+            [fileManager removeItemAtPath:path error:NULL];
+            //[fileManager removeFileAtPath:path handler:nil];    // <= 10.4
 
             for ( i=0, path=nil; i<[downloadFiles count]; i++)
             {   path = [[downloadFiles objectAtIndex:i] objectAtIndex:1];
-                if ( [fileManager fileExistsAtPath:path] )  // delete TAR-file
-                    [fileManager removeFileAtPath:path handler:nil];
+                if ( [[path pathExtension] isEqual:@"tar"] && [fileManager fileExistsAtPath:path] ) // del TAR-file
+                    [fileManager removeItemAtPath:path error:NULL];
+                    //[fileManager removeFileAtPath:path handler:nil];  // <= 10.4
                 // TODO: the following must not happen before the files are really installed !
                 /*path = [path stringByDeletingPathExtension];
-                if ( [fileManager fileExistsAtPath:path] )  // delete PKG-file
+                if ( [fileManager fileExistsAtPath:path] )  // delete PKG-file (fixed in 4.0.1)
                     [fileManager removeFileAtPath:path handler:nil];*/
             }
             if ( [[NSWorkspace sharedWorkspace] respondsToSelector:@selector(activateFileViewerSelectingURLs:)] )
@@ -851,7 +882,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
             NSURLRequest    *request = [NSURLRequest requestWithURL:url];
             NSString        *titleStr;
 
-            pkgDownload = [[NSURLDownload alloc] initWithRequest:request delegate:self];
+            pkgDownload = [[NSURLDownload alloc] initWithRequest:request delegate:(id)self];
             [pkgDownload setDeletesFileUponFailure:YES];
             pkgPath = path;
             [pkgDownload setDestination:pkgPath allowOverwrite:YES];
@@ -928,11 +959,11 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     //ext = [name pathExtension];
     NSLog(@"name = %@", name);
 
-    
+
     url = [NSURL URLWithString:[NSString stringWithFormat:@"http://www.cenon.info/update/%@", name]];
     path = vhfPathWithPathComponents(vhfUserLibrary(APPNAME), @".update", nil);
     if ( ! [fileManager fileExistsAtPath:path isDirectory:&isDir] )
-        [fileManager createDirectoryAtPath:path attributes:nil];
+        [fileManager createDirectoryAtPath:path recursive:NO attributes:nil error:NULL];
     else if ( ! isDir )
         NSLog(@"Cenon-Update: unexpected file at path '%@'", path);
     path = vhfPathWithPathComponents(path, name, nil);
@@ -940,7 +971,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     if ( [[NSFileManager defaultManager] fileExistsAtPath:path] )
         [self downloadDidFinish:(NSURLDownload*)request];
     else
-    {   download = [[NSURLDownload alloc] initWithRequest:request delegate:self];
+    {   download = [[NSURLDownload alloc] initWithRequest:request delegate:(id)self];
         [download setDeletesFileUponFailure:YES];
         [download setDestination:path allowOverwrite:YES];
     }
@@ -983,7 +1014,7 @@ static NSString         *checkMarkStr = nil;    // the checkmark
     return obj;
 }
 
-- (void)setDataDict:(NSDictionary*)newDataDict
+- (void)setDataDict:(NSMutableDictionary*)newDataDict
 {
     [dataDict release];
     dataDict = [newDataDict retain];
