@@ -1,42 +1,43 @@
 /* DXFExport.m
  * DXF export object
  *
- * Copyright (C) 2002-2003 by vhf interservice GmbH
+ * Copyright (C) 2002-2014 by Cenon GmbH
  * Author:   Ilonka Fleischmann
  *
  * created:  2002-04-25
- * modified: 2003-06-26
+ * modified: 2013-10-10 (-addLayer: lossyCString replaced, since it doesn't work on Apple anymore)
+ *           2012-02-07 (-saveToFile: use writeToFile:...encoding:error:)
  *
  * This file is part of the vhf Export Library.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include "DXFExport.h"
 #include "../VHFImport/dxfColorTable.h"
 #include "../VHFShared/types.h"
-#include "../VHFShared/VHFStringAdditions.h" // stringByReplacing -> eps export
+#include "../VHFShared/VHFStringAdditions.h" // stringByReplacing:
 
 #define RES 25.389
 
@@ -114,16 +115,18 @@ int convertNSColorToDXFColor(NSColor *color)
 }
 
 - (void)addLayer:(NSString*)name :(NSColor*)color :(int)attribut
-{   NSString		*nam = [[NSString stringWithString:name] uppercaseString];
+{   NSString		*nam = [name uppercaseString];
     NSCharacterSet	*set = [NSCharacterSet characterSetWithCharactersInString:@"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-_$"], *invSet;
-    NSRange		range;
+    NSRange         range;
 
     invSet = [set invertedSet];
 
-    // A-Z 0-9 - _ $ max 31
-    nam = [nam stringByReplacing:@"" by:@"AE"];
-    nam = [nam stringByReplacing:@"" by:@"OE"];
-    nam = [nam stringByReplacing:@"" by:@"UE"];
+    /* A-Z 0-9 - _ $ max 31 */
+    //nam = [NSString stringWithUTF8String:[nam lossyCString];    // remove all umlauts
+    nam = [[[NSString alloc] initWithData:[nam dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES]
+                                 encoding:NSASCIIStringEncoding] autorelease];    // remove all umlauts (FIXME: maybe UTF8 is ok in DXF ?)
+    if ( name && ! nam )    // make sure we have something, even if convertion method has bugs
+        nam = [name uppercaseString];
     range = [nam rangeOfCharacterFromSet:invSet];
     while (range.length)
     {   NSMutableString	*mutString = [NSMutableString string];
@@ -586,8 +589,8 @@ int convertNSColorToDXFColor(NSColor *color)
 }
 
 - (BOOL)saveToFile:(NSString*)filename
-{   BOOL	savedOk = NO;
-    NSString	*dxfStr = @"";
+{   BOOL        savedOk = NO;
+    NSString    *dxfStr = @"";
 
     /* FormatStatement mm */
     dxfStr = [dxfStr stringByAppendingFormat:@"0\n"];
@@ -627,7 +630,9 @@ int convertNSColorToDXFColor(NSColor *color)
     dxfStr = [dxfStr stringByAppendingFormat:@"0\n"]; // end of programm
     dxfStr = [dxfStr stringByAppendingFormat:@"EOF\n"]; // end of programm
 
-    savedOk = [dxfStr writeToFile:filename atomically:YES];
+    //savedOk = [dxfStr writeToFile:filename atomically:YES];
+    savedOk = [dxfStr writeToFile:filename atomically:YES
+                         encoding:NSUTF8StringEncoding error:NULL]; // >= 10.5
 
     return savedOk;
 }
