@@ -1,36 +1,37 @@
 /* DINImport.m
  * DIN import object (Drill data import)
  *
- * Copyright (C) 1996-2003 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Ilonka Fleischmann
  *
  * created:  1996-05-03
- * modified: 2002-10-26
+ * modified: 2012-06-18 (leading zero support added (LZ), clean-up)
+ *           2006-11-04
  *
  * This file is part of the vhf Import Library.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.cenon.de
  */
 
 #include <math.h>
@@ -79,9 +80,10 @@
     fileFormat = 0;
 
     res = 1000.0;
+    tz = YES;       // trailing zeros
 
-    state.inch = 1; // inch !
-    state.pa = 1;
+    state.inch = 1; // inch
+    state.pa   = 1;
 
     state.tool = 0;
     state.point.x = state.point.y = LARGENEG_COORD;
@@ -101,44 +103,45 @@
             ops.start = @"%%3000";
             res = 25400.0;
             ops.prgend = @"$";
-            ops.coordC = @"D"; // diameter of tool
+            ops.coordC = @"D";  // diameter of tool
             break;
         case DIN_SM1000:
             ops.start = @"%%1000";
             res = 25400.0;
             ops.prgend = @"$";
-            ops.coordC = @""; // diameter of tool
+            ops.coordC = @"";   // diameter of tool
             break;
         default:
-            ops.start = @"";
+            ops.start = @"%";
             res = 10000.0;
             ops.prgend = @"M30";
-            ops.coordC = @"C"; // diameter of tool
+            ops.coordC = @"C";  // diameter of tool
     }
     resMM = 25400.0;
-    ops.offset = @"M50";
-    ops.mm = @"M71";
-    ops.reset = @"";
-    ops.selectTool = @"T";
-    ops.coordX = @"X";
-    ops.coordY = @"Y";
-    ops.coordR = @"R"; // radius of circle
-    ops.plotAbs = @"G90";
-    ops.plotRel = @"G91";
-    ops.termi = @"\n";
-    //ops.polyBegin = @"G41"; // left
+    tz    = YES;                // trailing zeros
+    ops.offset      = @"M50";
+    ops.mm          = @"M71";
+    ops.reset       = @"";
+    ops.selectTool  = @"T";
+    ops.coordX      = @"X";
+    ops.coordY      = @"Y";
+    ops.coordR      = @"R";     // radius of circle
+    ops.plotAbs     = @"G90";
+    ops.plotRel     = @"G91";
+    ops.termi       = @"\n";
+    //ops.polyBegin = @"G41";   // left
     //ops.polyBegin2 = @"G42";  // right
     //ops.polyEnd = @"G40";
-    ops.comment = @"/";
-    ops.line = @"G1"; // G01
-    ops.circleCW = @"G2"; // G02
-    ops.circleCCW = @"G3"; // G03
-    ops.drill = @"G05";
+    ops.comment     = @"/";
+    ops.line        = @"G1";    // G01
+    ops.circleCW    = @"G2";    // G02
+    ops.circleCCW   = @"G3";    // G03
+    ops.drill       = @"G05";
 }
 
 - (BOOL)checkFileFormat:(NSString*)dinStr
-{   NSScanner		*scanner = [NSScanner scannerWithString:dinStr];
-    int			location;
+{   NSScanner   *scanner = [NSScanner scannerWithString:dinStr];
+    int         location;
 
     location = [scanner scanLocation];
     [scanner scanUpToString:@"%%1000" intoString:NULL];
@@ -203,6 +206,7 @@
  */
 - (BOOL)loadParameter:(NSString*)fileName
 {   NSMutableString	*parmData;
+    NSString		*val;
 
     [self setDefaultParameter];
 
@@ -221,8 +225,8 @@
     ops.init = vhfGetStringFromData(parmData, @"#INI");
     ops.reset = vhfGetStringFromData(parmData, @"#RST");
     ops.selectTool = vhfGetStringFromData(parmData, @"#ITL");
-    ops.start = vhfGetStringFromData(parmData, @"#IST");
-
+    if ( (val = vhfGetStringFromData(parmData, @"#IST")) )
+        ops.start = val;
     ops.coordX = vhfGetStringFromData(parmData, @"#IXP");
     ops.coordY = vhfGetStringFromData(parmData, @"#IYP");
     ops.coordC = vhfGetStringFromData(parmData, @"#IDM");
@@ -269,8 +273,8 @@
 }
 
 - (void)loadSM1000Apertures:(NSScanner*)scanner
-{   NSString	*parameter = @"$";
-    int		i, value, cnt = 1;
+{   NSString    *parameter = @"$";
+    int         i, value, cnt = 1;
 
     // $80$50$20$99$15$500 6 times $ at end
 
@@ -298,9 +302,9 @@
 }
 
 - (void)loadSM3000Apertures:(NSScanner*)scanner
-{   NSString	*parameter = @"$", *parmDia = @"D", *codeStr;
-    int		i, value, location = [scanner scanLocation];
-    float	dia;
+{   NSString    *parameter = @"$", *parmDia = @"D", *codeStr;
+    int         i, value, location = [scanner scanLocation];
+    float       dia;
 
     // $
     // T1D80S50F20R99N15A500T2D50...
@@ -354,29 +358,35 @@
 }
 
 - (void)loadExcellonApertures:(NSScanner*)scanner
-{   NSString	*parmDia = @"C", *codeStr;
-    int		value, location = [scanner scanLocation], inchLoc=0, mmLoc=0;
-    float	dia;
-    NSCharacterSet	*stopSet = [NSCharacterSet characterSetWithCharactersInString:@"TC"];
+{   NSString        *parmDia = @"C", *codeStr;
+    int             value, location = [scanner scanLocation], inchLoc=0, mmLoc=0;
+    float           dia;
+    NSCharacterSet  *stopSet = [NSCharacterSet characterSetWithCharactersInString:@"TC"];
 
     // get state.inch, state.pa and correct perhaps res
 
-    [scanner scanUpToString:@"G91" intoString:NULL]; // relative
+    [scanner scanUpToString:@"LZ" intoString:NULL];     // leading zeros
+    if ( [scanner scanString:@"LZ" intoString:NULL] )
+        tz = NO;    // leading zeros
+    // TODO: there is also "TZ" = trailing zeros, and "FMAT,2"
+
+    [scanner setScanLocation:location];
+    [scanner scanUpToString:@"G91" intoString:NULL];    // relative
     if ( [scanner scanString:@"G91" intoString:NULL] )
         state.pa = 0;
 
     [scanner setScanLocation:location];
-    [scanner scanUpToString:@"INCH" intoString:NULL]; // inch
+    [scanner scanUpToString:@"INCH" intoString:NULL];   // inch
     if ( [scanner scanString:@"INCH" intoString:NULL] )
         inchLoc = [scanner scanLocation];
     [scanner setScanLocation:location];
-    [scanner scanUpToString:@"M72" intoString:NULL]; // inch
+    [scanner scanUpToString:@"M72" intoString:NULL];    // inch
     if ( [scanner scanString:@"M72" intoString:NULL] )
     {   if ( !inchLoc || inchLoc > (int)[scanner scanLocation] )
             inchLoc = [scanner scanLocation];
     }
     [scanner setScanLocation:location];
-    [scanner scanUpToString:@"M71" intoString:NULL]; // mm
+    [scanner scanUpToString:@"M71" intoString:NULL];    // mm
     if ( [scanner scanString:@"M71" intoString:NULL] )
         mmLoc = [scanner scanLocation];
     [scanner setScanLocation:location];
@@ -386,7 +396,7 @@
             mmLoc = [scanner scanLocation];
     }
     [scanner setScanLocation:location];
-    [scanner scanUpToString:ops.mm intoString:NULL]; // mm (from device file)
+    [scanner scanUpToString:ops.mm intoString:NULL];    // mm (from device file)
     if ( [scanner scanString:ops.mm intoString:NULL] )
     {   if ( !mmLoc || mmLoc > (int)[scanner scanLocation] )
             mmLoc = [scanner scanLocation];
@@ -477,7 +487,7 @@
     [self loadApertures:dinStr];
 
     if ( ![tools count] )
-        NSLog(@"No apertures loaded !");
+        NSLog(@"No tools loaded !");
 
     /* interpret data
      */
@@ -490,14 +500,16 @@
 /* private methods
  */
 - (BOOL)interpret:(NSString*)dataP
-{   NSRect	bounds;
-    NSScanner	*scanner = [NSScanner scannerWithString:dataP];
+{   int         startLocation;
+    NSRect      bounds;
+    NSScanner   *scanner = [NSScanner scannerWithString:dataP];
 
     digitsSet = [NSCharacterSet characterSetWithCharactersInString:DIGITS];
     invDigitsSet = [digitsSet invertedSet];
     // NSScanner else jump over \n (default)
     [scanner setCharactersToBeSkipped:[NSCharacterSet whitespaceCharacterSet]];
 
+    startLocation = [scanner scanLocation];
     /* init bounds */
     ll.x = ll.y = LARGE_COORD;
     ur.x = ur.y = LARGENEG_COORD;
@@ -505,13 +517,17 @@
     list = [self allocateList];
 
     // set scanner to start of programm (%%1000 or %%3000)
-    if ( fileFormat != DIN_EXCELLON )
+    // if ( fileFormat != DIN_EXCELLON )
     {
         [scanner scanUpToString:ops.start intoString:NULL];
         if ( ![scanner scanString:ops.start intoString:NULL] )
         {
-            NSLog(@"DINImport: No start sign (%@) found", ops.start);
-            return NO;
+            if ( fileFormat == DIN_EXCELLON )
+                [scanner setScanLocation:startLocation];
+            else
+            {   NSLog(@"DINImport: No start sign (%@) found", ops.start);
+                return NO;
+            }
         }
     }
 
@@ -519,8 +535,8 @@
         if ( ![self getGraphicFromData:scanner :list] )
             break;
 
-    bounds.origin = ll;
-    bounds.size.width = ur.x - ll.x;
+    bounds.origin      = ll;
+    bounds.size.width  = ur.x - ll.x;
     bounds.size.height = ur.y - ll.y;
 
     [self setBounds:bounds];
@@ -530,8 +546,9 @@
 /* we need cp on a number !
  */
 - (BOOL)getGraphicFromData:(NSScanner*)scanner :cList
-{   int		location;
-    float	value;
+{   int         location;
+    float       value;
+    NSString    *str = nil;
 
     location = [scanner scanLocation];
 
@@ -589,8 +606,12 @@
     }*/
     else if ( [scanner scanString:ops.coordX intoString:NULL] )
     {
-        if ( [scanner scanFloat:&value] )
-        {  // state.point.x = state.x;
+        if ( [scanner scanCharactersFromSet:digitsSet intoString:&str] )
+        //if ( [scanner scanFloat:&value] )
+        {   //state.point.x = state.x;
+            value = [str floatValue];
+            if ( ! tz ) // leading zeros
+                value *= pow(10, 7 - [str length]);
             state.x = DeviceResToInternal(value, res);
             state.draw = 1;
         }
@@ -601,8 +622,12 @@
     }
     else if ( [scanner scanString:ops.coordY intoString:NULL] )
     {
-        if ( [scanner scanFloat:&value] )
-        {//   state.point.y = state.y;
+        if ( [scanner scanCharactersFromSet:digitsSet intoString:&str] )
+        //if ( [scanner scanFloat:&value] )
+        {   //state.point.y = state.y;
+            value = [str floatValue];
+            if ( ! tz ) // leading zeros
+                value *= pow(10, 7 - [str length]);
             state.y = DeviceResToInternal(value, res);
             state.draw = 1;
         }
@@ -613,12 +638,16 @@
     }
     else if ( [scanner scanString:ops.coordR intoString:NULL] )
     {
-        if ( [scanner scanFloat:&value] )
-        {   state.a = DeviceResToInternal(value, res);
+        if ( [scanner scanCharactersFromSet:digitsSet intoString:&str] )
+        //if ( [scanner scanFloat:&value] )
+        {   value = [str floatValue];
+            if ( ! tz ) // leading zeros
+                value *= pow(10, 7 - [str length]);
+            state.a = DeviceResToInternal(value, res);
             state.draw = 1;
         }
         else
-            NSLog(@"Coordinate R expected at location: %d", [scanner scanLocation]);
+            NSLog(@"Coordinate R expected at location: %d", (unsigned)[scanner scanLocation]);
     }
     else if ( [scanner scanString:ops.selectTool intoString:NULL] )
 //        [self getTool:scanner];
@@ -683,7 +712,7 @@
          return NO;
     else if ( val < 0 )
     {
-        NSLog(@"Gerber import, Can't find tool at location %d. Default used.", [scanner scanLocation]);
+        NSLog(@"Gerber import, Can't find tool at location %d. Default used.", (unsigned)[scanner scanLocation]);
         state.tool = state.tool + 1; //  = 0;
         return NO;
     }
@@ -879,8 +908,8 @@
 }
 
 - (void)setMark:cList
-{   NSPoint		center, pll, pur;
-    NSDictionary	*tool; // = [tools objectAtIndex:state.tool];
+{   NSPoint         center, pll, pur;
+    NSDictionary    *tool; // = [tools objectAtIndex:state.tool];
 
     tool = ([tools count]) ? [tools objectAtIndex:state.tool] : nil;
 
