@@ -1,37 +1,38 @@
 /* DXFImport.h
  * DXF import object
  *
- * Copyright (C) 1996-2011 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-01
- * modified: 2011-04-04 (DXFGroup: z coordinates added)
+ * modified: 2013-04-16 (style added to group)
+ *           2011-04-04 (DXFGroup: z coordinates added)
  *           2009-02-06 (parameter for extrusion direction added)
  *
  * This file is part of the vhf Import Library.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <Foundation/Foundation.h>
@@ -51,6 +52,7 @@
 typedef struct _DXFGroup
 {
     NSString    *text;
+    NSString    *style;
     NSString    *name;
     NSString    *layer;
     NSString    *handle;
@@ -88,20 +90,23 @@ typedef struct _DXFState
 
     NSPoint     offset;         // offset to move inserts by
     float       rotAngle;       // angle to rotate inserts by
+
+    NSString    *layer;         // layer overriding the layer in group.layer (for INSERT)
 }DXFState;
 
 @interface DXFImport:NSObject
 {
-    id          list;           // the base list for all contents
-    DXFState    state;          // the current gstate
-    NSPoint     extMin, extMax; // bounds from header
-    NSPoint     ll, ur;         // bounds of data
-    float       res;            // device resolution in pixel per inch
-    NSArray     *table;         // layer table
-    NSArray     *visibleList;   // array of visible objects (from IDBUFFER) or nil for all visible
-    DXFGroup    group;          // contents of current group
+    id              list;           // the base list for all contents
+    DXFState        state;          // the current gstate
+    NSPoint         extMin, extMax; // bounds from header
+    NSPoint         ll, ur;         // bounds of data
+    float           res;            // device resolution in pixel per inch
+    //NSArray     *table;           // layer table
+    NSDictionary    *table;         // table dictionary with entries: "layer", "style", ...
+    NSArray         *visibleList;   // array of visible objects (from IDBUFFER) or nil for all visible
+    DXFGroup        group;          // contents of current group
 
-    NSScanner   *blockScanner;  // global block scanner
+    NSScanner       *blockScanner;  // global block scanner
 }
 
 /* start import
