@@ -1,11 +1,16 @@
 /* DXFImport.m
  * DXF import object
  *
- * Copyright (C) 1996-2012 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-01
- * modified: 2012-09-13 (Diff(l, 0.0) < 0.0000001, else division with 0)
+ * modified: 2014-08-29 (-getArc:::: if group.ext transform the complete arc)
+ *           2014-08-29 (-scanGroup: group.color no init)
+ *           2014-07-04 (-getText: fontName added to internal declaration)
+ *           2013-09-13 (getInsert(), ...: place objects to layer of insert, not the layer of the blocks)
+ *           2013-04-16 (getTableFromData(): return dictionary)
+ *           2012-09-13 (Diff(l, 0.0) < 0.0000001, else division with 0)
  *           2012-02-04 (-scanGroup: transform coordinates from OCS to WCS, if necessary)
  *           2012-02-03 (InternalToDeviceRes(), DeviceResToInternal(): float -> double)
  *           2011-12-11 (-getGraphicFromData: TODO for SPLINEs added)
@@ -20,27 +25,27 @@
  * This file is part of the vhf Import Library.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <math.h>
@@ -66,8 +71,8 @@ static BOOL gotoNextId(NSScanner *scanner, NSString *dataId);
 static BOOL gotoSection(NSScanner *scanner, NSString *name);
 static BOOL gotoTable(NSScanner *scanner, NSString *name);
 static BOOL gotoGroup( NSScanner *scanner, NSString *name);
-static NSArray *getTableFromData(NSString *data);
-static NSColor *getColorFromTable(NSString *str, NSArray *table);
+static NSDictionary *getTableFromData(NSString *data);
+static NSColor *getColorFromLayer(NSString *str, NSArray *table);
 static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *angle);
 
 @interface DXFImport(PrivateMethods)
@@ -84,7 +89,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 - (BOOL)getSolid:(NSScanner*)scanner :(NSPoint*)ps :(int*)pCnt;
 - (BOOL)getCircle:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle;
 - (BOOL)getArc:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle;
-- (BOOL)getText :(NSScanner*)scanner mtext:(BOOL)mext :(NSString**)string :(float*)angle :(NSPoint*)origin :(float*)size :(float*)ar :(int*)alignment;
+- (BOOL)getText :(NSScanner*)scanner mtext:(BOOL)mext :(NSString**)string :(float*)angle :(NSPoint*)origin :(float*)size :(float*)ar :(int*)alignment :(NSString**)fontName;
 - (BOOL)get3DFace:(NSScanner*)scanner points:(V3Point*)pts color:(NSColor**)color;
 - (BOOL)getInsert:(NSScanner*)scanner :cList;
 - (void)updateBounds:(NSPoint)p;
@@ -108,8 +113,8 @@ static BOOL gotoNextLine(NSScanner *scanner)
  * scanner have to be on an id!
  */
 static BOOL gotoNextId(NSScanner *scanner, NSString* dataId)
-{   NSString	*string;
-    int		location = [scanner scanLocation];
+{   NSString    *string;
+    int         location = [scanner scanLocation];
 
     while ( ![scanner isAtEnd] )
     {
@@ -176,11 +181,11 @@ static BOOL gotoSection(NSScanner *scanner, NSString *name)
             if ( [scanner scanString:IDNAME intoString:NULL] )		/* name code */
             {
                 if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-                    NSLog(@"GotoSection: Newline expected at location: %d", [scanner scanLocation]);
+                    NSLog(@"GotoSection: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
                 if ( [scanner scanString:name intoString:NULL] )		/* check name */
                 {
                     if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-                        NSLog(@"GotoSection: Newline expected at location: %d", [scanner scanLocation]);
+                        NSLog(@"GotoSection: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
                     return YES;
                 }
             }
@@ -189,7 +194,7 @@ static BOOL gotoSection(NSScanner *scanner, NSString *name)
     return NO;
 }
 
-/* created:   12.05.93
+/* created:   1993-05-12
  * modified:  08.12.97
  *
  * purpose:   set cp to the beginning (on the name) of a named table
@@ -207,11 +212,11 @@ static BOOL gotoTable(NSScanner *scanner, NSString *name)
             if ( [scanner scanCharactersFromSet:newLineSet intoString:NULL] && [scanner scanString:@"2" intoString:NULL] )	/* newline, name code */
             {
                 if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-                    NSLog(@"GotoTable: Newline expected at location: %d", [scanner scanLocation]);
+                    NSLog(@"GotoTable: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
                 if ( [scanner scanString:name intoString:NULL] )		/* check name */
                 {
                     if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-                        NSLog(@"GotoTable: Newline expected at location: %d", [scanner scanLocation]);
+                        NSLog(@"GotoTable: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
                     return YES;
                 }
             }
@@ -227,10 +232,12 @@ static BOOL gotoTable(NSScanner *scanner, NSString *name)
  *            at the moment we only read the layer table...
  * parameter: data (dxf file)
  */
-static NSArray *getTableFromData(NSString *data)
+static NSDictionary *getTableFromData(NSString *data)
 {   NSScanner           *scanner = [NSScanner scannerWithString:data];
     NSString            *string;
-    NSMutableArray      *table = [NSMutableArray array];
+    NSMutableDictionary *table  = [NSMutableDictionary dictionary];
+    NSMutableArray      *layers = [NSMutableArray array];           // layers array
+    NSMutableDictionary *styles = [NSMutableDictionary dictionary]; // styles dict
     NSMutableDictionary *dict = nil;
 
     [scanner setCharactersToBeSkipped:skipSet];
@@ -242,11 +249,12 @@ static NSArray *getTableFromData(NSString *data)
 
     /* read the layer table
      */
+    [table setObject:layers forKey:@"layer"];
     if (!gotoTable(scanner, NAMLAYER))
     {   NSLog(@"Can't get table 'LAYER' from dxf data");
         return nil;
     }
-    /* 2
+    /* 0
      * LAYER
      * we are here
      */
@@ -255,7 +263,7 @@ static NSArray *getTableFromData(NSString *data)
 
         /* scan id */
         if ( ![scanner scanInt:&ident] )
-        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %u", (unsigned)[scanner scanLocation]);
             return NO;
         }
         gotoNextLine(scanner);	// goto value line
@@ -272,7 +280,7 @@ static NSArray *getTableFromData(NSString *data)
                 if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
                     break;
                 [dict setObject:string forKey:@"name"];
-                [table addObject:dict];
+                [layers addObject:dict];
                 break;
             case ID_COLOR:
                 [scanner scanUpToCharactersFromSet:newLineSet intoString:&string];
@@ -289,6 +297,64 @@ static NSArray *getTableFromData(NSString *data)
             break;
     }
 
+    /* read the style table
+     * tables -> style -> style1, style2, ...
+     */
+    [table setObject:styles forKey:@"style"];
+    [scanner setScanLocation:0];
+    if ( !gotoSection(scanner, NAMTABLES) )
+    {   NSLog(@"Can't get section 'TABLE' from dxf data");
+        return nil;
+    }
+    if (!gotoTable(scanner, NAMSTYLE))
+    {   NSLog(@"Can't get table 'STYLE' from dxf data");
+        return nil;
+    }
+    /* 0
+     * STYLE
+     * we are here
+     */
+    while ( ![scanner isAtEnd] )
+    {	int	ident;
+
+        /* scan id */
+        if ( ![scanner scanInt:&ident] )
+        {   NSLog(@"DXF-Import: id expected at location: %u", (unsigned)[scanner scanLocation]);
+            return NO;
+        }
+        gotoNextLine(scanner);	// goto value line
+        switch ( ident )
+        {
+            case ID_GROUP:
+                if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
+                    string = @"";
+                if (![string isEqual:GRPSTYLE]) // ENDTAB
+                    break;
+                dict = [NSMutableDictionary dictionary];
+                break;
+            case ID_NAME:
+                if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
+                    break;
+                [dict setObject:string forKey:@"name"];
+                [styles setObject:dict forKey:string];
+                //[styles addObject:dict];
+                break;
+            case ID_DESCRIPT:   // font file name ("ariblk.TTF")
+                if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
+                    break;
+                [dict setObject:string forKey:@"fileName"];
+                break;
+            case ID_EXTSTRING:  // font name ("Arial Black")
+                if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
+                    break;
+                [dict setObject:string forKey:@"fontName"];
+                break;
+        }
+        gotoNextLine(scanner);	// goto id line
+        if (ident == ID_GROUP && ![string isEqual:@"STYLE"])
+            break;
+    }
+
     return table;
 }
 
@@ -318,7 +384,7 @@ static NSArray *getVisibleListFromData(NSString *data)
 
         /* scan id */
         if ( ![scanner scanInt:&ident] )
-        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %u", (unsigned)[scanner scanLocation]);
             return nil;
         }
         gotoNextLine(scanner);	// goto value line
@@ -370,7 +436,7 @@ static BOOL gotoGroup( NSScanner *scanner, NSString *name )
         if ( [scanner scanString:name intoString:NULL] )
         {
             if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-                NSLog(@"GotoGroup: Newline expected at location: %d", [scanner scanLocation]);
+                NSLog(@"GotoGroup: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
             [scanner setCaseSensitive:caseSensitive];
             return YES;
         }
@@ -388,21 +454,21 @@ static BOOL gotoGroup( NSScanner *scanner, NSString *name )
  *            table ()
  * return:    color
  */
-static NSColor *getColorFromTable(NSString *str, NSArray *table)
+static NSColor *getColorFromLayer(NSString *str, NSArray *layers)
 {   int			i;
-    DXFColor		col;
+    DXFColor    col;
     NSColor		*color;
     //NSDictionary	*dict;
 
-    /*if (dict = [table objectForKey:str])
+    /*if (dict = [layers objectForKey:str])
     {   int	colIx = [dict intForKey:@"colorNum"];
 
         col = colorTable[(colIx > 0) ? colIx-1 : 0];
         color = [NSColor colorWithCalibratedRed:col.r green:col.g blue:col.b alpha:1.0];
         return color;
     }*/
-    for (i=[table count]-1; i>=0; i--)
-    {   NSDictionary	*dict = [table objectAtIndex:i];
+    for (i=[layers count]-1; i>=0; i--)
+    {   NSDictionary	*dict = [layers objectAtIndex:i];
 
         if ( [[dict objectForKey:@"name"] isEqual:str] )
         {   int	colIx = [[dict objectForKey:@"colorNum"] intValue];
@@ -498,7 +564,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     {   int	i;
 
         if (![scanner scanInt:&i])	// get id
-            NSLog(@"getHeader: integer expected at location: %d", [scanner scanLocation]);
+            NSLog(@"getHeader: integer expected at location: %u", (unsigned)[scanner scanLocation]);
 
         gotoNextLine(scanner);		// goto value
         switch (i)
@@ -550,7 +616,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
         return NO;
     }
 
-    list = [self allocateList:table];
+    list = [self allocateList:[table objectForKey:@"layer"]];
     cList = [NSMutableArray array];
 
     while ( ![scanner isAtEnd] )
@@ -575,7 +641,8 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 }
 
 /* we need scanner on the id line of a group
- * modified: 2005-09-26 (close polyline with arc)
+ * modified: 2013-09-13 (toLayer:((state.layer) ? state.layer : group.layer))
+ *           2005-09-26 (close polyline with arc)
  */
 - (BOOL)getGraphicFromData:(NSScanner*)scanner :(id)cList
 {   NSPoint     p0, p1;
@@ -588,7 +655,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     [scanner scanUpToCharactersFromSet:newLineSet intoString:&name];
     /* goto id line */
     if (![scanner scanCharactersFromSet:newLineSet intoString:NULL])
-        NSLog(@"getGraphicFromData:: Newline expected at location: %d", [scanner scanLocation]);
+        NSLog(@"getGraphicFromData:: Newline expected at location: %u", (unsigned)[scanner scanLocation]);
 
     switch ( state.mode )
     {
@@ -597,7 +664,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
             if ( [name isEqual:GRPLINE] )
             {
                 if ([self getLine:scanner :&p0 :&p1 :&state.color])
-                    [self addLine:p0 :p1 toLayer:group.layer];
+                    [self addLine:p0 :p1 toLayer:((state.layer) ? state.layer : group.layer)];
             }
             /* lwpolyline */
             else if ( [name isEqual:GRPLWPOLYLINE] )
@@ -621,7 +688,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                         [self updateBounds:ps[i]];
                     }
                     if ([cList count])
-                    {   [self addFillList:cList toLayer:group.layer];
+                    {   [self addFillList:cList toLayer:((state.layer) ? state.layer : group.layer)];
                         [cList removeAllObjects];
                     }
                 }
@@ -632,7 +699,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                 float	angle;
 
                 if ([self getCircle:scanner :&ctr :&start :&angle])
-                    [self addArc:ctr :start :angle toLayer:group.layer];
+                    [self addArc:ctr :start :angle toLayer:((state.layer) ? state.layer : group.layer)];
             }
             /* arc */
             else if ( [name isEqual:GRPARC] )
@@ -640,24 +707,24 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                 float	angle;
 
                 if ([self getArc:scanner :&ctr :&start :&angle])
-                    [self addArc:ctr :start :angle toLayer:group.layer];
+                    [self addArc:ctr :start :angle toLayer:((state.layer) ? state.layer : group.layer)];
             }
             /* text, mtext */
             else if ( [name isEqual:GRPTEXT] || [name isEqual:GRPMTEXT] )
             {	NSPoint     origin;
                 float       angle, size, ar;
                 int         alignment;
-                NSString    *string = nil;
+                NSString    *string = nil, *fontName = @"Courier";
 
                 if ([self getText:scanner mtext:[name isEqual:GRPMTEXT]
-                                 :&string :&angle :&origin :&size :&ar :&alignment])
+                                 :&string :&angle :&origin :&size :&ar :&alignment :&fontName])
                 {
                     if ([self respondsToSelector:@selector(addText:::::at:toLayer:)])	// old
-                        [self addText:string :@"Courier" :angle :size :ar at:origin
-                              toLayer:group.layer];
+                        [self addText:string :fontName :angle :size :ar at:origin
+                              toLayer:((state.layer) ? state.layer : group.layer)];
                     else
-                        [self addText:string :@"Courier" :angle :size :ar :alignment at:origin
-                              toLayer:group.layer];
+                        [self addText:string :fontName :angle :size :ar :alignment at:origin
+                              toLayer:((state.layer) ? state.layer : group.layer)];
                 }
                 [string autorelease];
             }
@@ -671,7 +738,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
             {
                 // TODO: SPLINEs (see example Stebner_SPLINEs*.dxf)
                 //if ([self getSpline:scanner :pts :&state.color])  // we should return/convert to bezier curve(s)
-                //    [self addCurve:pts toLayer:group.layer];
+                //    [self addCurve:pts toLayer:((state.layer) ? state.layer : group.layer)];
                 printf("TODO, DXF-Import: SPLINEs not implemented\n");
             }
             /* 3D LINE */
@@ -679,14 +746,14 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
             {   V3Point p0, p1;
 
                 if ([self getLine3D:scanner :&p0 :&p1 :&state.color])
-                    [self addLine3D:p0 :p1 toLayer:group.layer];
+                    [self addLine3D:p0 :p1 toLayer:((state.layer) ? state.layer : group.layer)];
             }
             /* 3D FACE */
             else if ( [name isEqual:GRP3DFACE] )
             {   V3Point pts[4];
 
                 if ([self get3DFace:scanner points:pts color:&state.color] )
-                    [self add3DFace:pts toLayer:group.layer];
+                    [self add3DFace:pts toLayer:((state.layer) ? state.layer : group.layer)];
             }
             break;
         case MODE_VERTEX:
@@ -721,7 +788,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                     }
                 }
                 if ([cList count])
-                {   [self addStrokeList:cList toLayer:group.layer];
+                {   [self addStrokeList:cList toLayer:((state.layer) ? state.layer : group.layer)];
                     [cList removeAllObjects];
                 }
                 state.mode = MODE_NORMAL;
@@ -845,7 +912,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 
         /* scan id */
         if ( ![scanner scanInt:&i] )
-        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %u", (unsigned)[scanner scanLocation]);
             return NO;
         }
         gotoNextLine(scanner);	// goto value line
@@ -931,7 +998,7 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
                 if ( ![visibleList count] || [visibleList containsObject:group.handle])
                 {
                     if ([cList count])
-                    {   [self addStrokeList:cList toLayer:group.layer];
+                    {   [self addStrokeList:cList toLayer:((state.layer) ? state.layer : group.layer)];
                         [cList removeAllObjects];
                     }
                 }
@@ -1219,7 +1286,8 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 }
 
 /* created:   1993-05-17
- * modified:  2010-04-03 (negative 360 deg arcs work now)
+ * modified:  2014-07-23 (if group.ext transform the complete arc)
+ *            2010-04-03 (negative 360 deg arcs work now)
  *
  * purpose:   get arc
  * parameter: cp (the dxf data)
@@ -1235,6 +1303,14 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     if ( [visibleList count] && ![visibleList containsObject:group.handle])
         return NO;
 
+    /* revert transformation of group.x0 group.y0 point - we need to transform the complete arc */
+    if ( group.extX != 0.0 || group.extY != 0.0 || group.extZ != 1.0 )
+    {   V3Point N = V3MakePoint(group.extX, group.extY, group.extZ), p;
+
+        p = transformOCS(V3MakePoint(group.x0, group.y0, 0.0), N);
+        group.x0 = p.x; group.y0 = p.y;
+    }
+
     state.color = [self currentColor];
     ctr->x = DeviceResToInternal(group.x0, res) + state.offset.x;
     ctr->y = DeviceResToInternal(group.y0, res) + state.offset.y;
@@ -1255,6 +1331,28 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     if ( *angle == 0.0 )
         *angle = ea - ba;
 
+    if ( group.extX != 0.0 || group.extY != 0.0 || group.extZ != 1.0 )
+    {   V3Point N = V3MakePoint(group.extX, group.extY, group.extZ), p;
+        NSPoint end = vhfPointAngleFromRefPoint(*ctr, *start, *angle);
+
+        p = transformOCS(V3MakePoint(ctr->x, ctr->y, 0.0), N);
+        ctr->x = p.x; ctr->y = p.y;
+        p = transformOCS(V3MakePoint(start->x, start->y, 0.0), N);
+        start->x = p.x; start->y = p.y;
+        p = transformOCS(V3MakePoint(end.x, end.y, 0.0), N);
+        end.x = p.x; end.y = p.y;
+
+        ba = vhfAngleOfPointRelativeCenter(*start, *ctr);
+        ea = vhfAngleOfPointRelativeCenter(end, *ctr);
+        if ( Abs(ea-ba) == Abs(*angle) )
+            *angle = ea - ba;
+        else
+        {   if ( ba < ea )
+            ba += 360.0;
+            *angle = ea - ba;
+        }
+    }
+
     state.width = 0.0;
 
     //[self updateBounds:*start];
@@ -1268,7 +1366,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 }
 
 /* created:   1993-06-11
- * modified:  2002-01-09
+ * modified:  2013-04-16 (fontName added)
  *
  * purpose:   get text (TEXT, MTEXT)
  *            read a sv-polygon for each character
@@ -1280,16 +1378,18 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
  *            size
  *            ar	aspect ratio
  *            alignment	1 = top/left, 2= top/center, 3=top/right, 4=middle/left, ... 9 = bottom/right
+ *            fontName
  */
 - (BOOL)getText:(NSScanner*)scanner mtext:(BOOL)mtext
                :(NSString**)string :(float*)angle :(NSPoint*)origin :(float*)size :(float*)ar :(int*)alignment
+               :(NSString**)fontName
 {   int		adjust = 0;
     NSPoint	adjustPoint,	// a point to adjust the text
-		p;
+            p;
     float	slopeAngle = 0.0;
     float	textHeight, textScale;
 
-    *ar = 1.0;	/* aspect ratio */
+    *ar = 1.0;	// aspect ratio
 
     [self scanGroup:scanner];
     if ( [visibleList count] && ![visibleList containsObject:group.handle])
@@ -1309,6 +1409,21 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     adjust = group.adjust;	// get adjust position from file
     adjustPoint.x = DeviceResToInternal(group.x1, res);
     adjustPoint.y = DeviceResToInternal(group.y1, res);
+    if ( ! group.style )
+        group.style = @"Standard";  // FIXME: this should be case insensitive
+    if ( [table objectForKey:@"style"] )
+    {   NSDictionary    *styles = [table objectForKey:@"style"];
+        NSDictionary    *style = [styles objectForKey:group.style];
+
+        if ( [style objectForKey:@"fontName"] )
+            *fontName = [style objectForKey:@"fontName"];
+        else if ( [style objectForKey:@"fileName"] )
+        {   NSString    *fn = [style objectForKey:@"fileName"];
+
+            NSLog(@"TODO: what are we going to assign to the fonbt file name: \"%@\" ? Using Default.", fn);
+            *fontName = nil;
+        }
+    }
 
     if (!mtext)
     {
@@ -1400,8 +1515,8 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     return YES;
 }
 
-/* created:   30.09.94
- * modified:  30.09.94 10.03.97
+/* created:   1994-09-30
+ * modified:  2013-09-13 (Fix: place objects to layer of insert, not the layer of the blocks)
  *
  * purpose:   get insert
  * parameter: scanner (the dxf data)
@@ -1420,6 +1535,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
         return YES;
 
     state.color = [self currentColor];
+    state.layer = [group.layer retain]; // everthing goes to this layer
     name = group.name;
     angle = group.begAngle;
     point.x = DeviceResToInternal(group.x0, res);
@@ -1488,12 +1604,14 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
      */
     state.offset = point;
     state.rotAngle = angle;
+    // FIXME: we have to put the blocks on the layer group.layer (not the layer in the blocjk !)
     while ( ![myBlockScanner isAtEnd] )
         [self getGraphicFromData:myBlockScanner :cList];
     [scanner setScanLocation:location];
     state.offset = NSMakePoint(0.0, 0.0);
     state.rotAngle = 0.0;
 
+    [state.layer release]; state.layer = nil;
     [pool release];
 
     return YES;
@@ -1502,9 +1620,9 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 /*
  */
 - (NSColor*)currentColor
-{   int		num = group.color;
-    DXFColor	col;
-    NSColor	*color;
+{   int         num = group.color;
+    DXFColor    col;
+    NSColor     *color;
 
     if ( num )
     {
@@ -1516,7 +1634,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     }
 
     /* search the relating color from the color table */
-    color = getColorFromTable(group.layer, table);
+    color = getColorFromLayer(group.layer, [table objectForKey:@"layer"]);
     return color;
 }
 
@@ -1571,18 +1689,24 @@ static V3Point transformOCS(V3Point pOCS, V3Point AZ)
  * scanner starts at id line of first entry
  * scanner scans until it reaches the next group
  * created:  1997-03-19
- * modified: 2011-04-04 (group.z0 - z3 added)
+ * modified: 2014-04-14 (group.color without init to allow color to be reached down the hierarchy)
+ *           2013-04-16 (style added)
+ *           2011-04-04 (group.z0 - z3 added)
  *           2009-02-06 (read in extrusion direction)
  */
 - (BOOL)scanGroup:(NSScanner*)scanner
 {   BOOL    exitLoop = NO;
 
-    [group.text release]; group.text = nil;
-    [group.name release]; group.name = nil;
-    [group.layer release]; group.layer = nil;
+    /* FIXME: default values may overwrite value reached down from hierarchy !
+     * first/specialy color and layer - even layer should always be set
+     */
+    [group.text release];   group.text = nil;
+    [group.style release];  group.style = nil;
+    [group.name release];   group.name = nil;
+    [group.layer release];  group.layer = nil;
     [group.handle release]; group.handle = nil;
-    group.color = 0;
-    group.width = -1.0;
+    // group.color = 0; // we use the color from higher Element, if color is not set
+    group.width    = -1.0;
     group.endWidth = -1.0;
     group.begAngle = group.endAngle = 0.0;
     group.x0 = group.y0 = group.z0 = 0.0;
@@ -1594,11 +1718,11 @@ static V3Point transformOCS(V3Point pOCS, V3Point AZ)
     group.extX = group.extY = 0.0; group.extZ = 1.0;
 
     while ( ![scanner isAtEnd] )
-    {	int	i, location = [scanner scanLocation];
+    {	int i, location = [scanner scanLocation];
 
         /* scan id */
         if ( ![scanner scanInt:&i] )
-        {   NSLog(@"DXF-Import: id expected at location: %d", [scanner scanLocation]);
+        {   NSLog(@"DXF-Import: id expected at location: %u", (unsigned)[scanner scanLocation]);
             return NO;
         }
         gotoNextLine(scanner);	// goto value line
@@ -1629,6 +1753,12 @@ static V3Point transformOCS(V3Point pOCS, V3Point AZ)
                 if (![scanner scanInt:&group.lineType])
                     group.lineType = 0.0;
                 break;
+            case ID_STYLE:
+                if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&group.style])
+                    group.style = nil;
+                else
+                    [group.style retain];
+                break;
             case ID_LAYER:
                 if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&group.layer])
                     group.layer = nil;
@@ -1707,7 +1837,7 @@ static V3Point transformOCS(V3Point pOCS, V3Point AZ)
                 break;
             case ID_COLOR:
                 if (![scanner scanInt:&group.color])
-                    group.color = 0;
+                    group.color = 0; // wrong - in ENTITIES 7 - in BLOCKS color from INSERT
                 break;
             case ID_MORE:
                 if (![scanner scanInt:&group.more])
@@ -1737,7 +1867,7 @@ static V3Point transformOCS(V3Point pOCS, V3Point AZ)
                 if (![scanner scanFloat:&group.extY])
                     group.extY = 0.0;
                 if ( group.extX != 0.0 || group.extY != 0.0 )   // FIXME: extrusion direction
-                    NSLog(@"DXF-Import: extrusion direction not supported. File-Location = %lu", [scanner scanLocation]);
+                    NSLog(@"DXF-Import: extrusion direction not supported. File-Location = %u", (unsigned)[scanner scanLocation]);
                 break;
             case ID_EXT_Z:
                 if (![scanner scanFloat:&group.extZ])
@@ -1832,16 +1962,18 @@ static V3Point transformOCS(V3Point pOCS, V3Point AZ)
 }
 
 /* allocate a text object and add it to layer
- * parameter: text	the text string
- *            font	the font name
- *            angle	rotation angle
- *            size	the font size in pt
- *            ar	aspect ratio height/width
+ * parameter: text      the text string
+ *            font      the font name
+ *            angle     rotation angle
+ *            size      the font size in pt
+ *            ar        aspect ratio height/width
+ *            alignment left, centered, right aligned
  *            layerInfo	the destination layer
  */
-- (void)addText:(NSString*)text :(NSString*)font :(float)angle :(float)size :(float)ar :(int)alignment at:(NSPoint)p toLayer:(NSString*)layerName
+- (void)addText:(NSString*)text :(NSString*)font :(float)angle :(float)size :(float)ar
+               :(int)alignment at:(NSPoint)p toLayer:(NSString*)layerName
 {
-    NSLog(@"text: %@ %f %f %f %f %d %f \"%s\" \"%s\"", layerName, p.x, p.y, angle, size, ar, alignment, text, font); 
+    NSLog(@"text: %@ %f %f %f %f %f %d \"%@\" \"%@\"", layerName, p.x, p.y, angle, size, ar, alignment, text, font); 
 }
 
 - (void)setBounds:(NSRect)bounds
