@@ -1,37 +1,40 @@
 /* GerberImport.m
  * Gerber import object (RS274X)
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *           Ilonka Fleischmann
  *
  * created:  1996-05-03
- * modified: 2005-05-12
+ * modified: 2013-12-19 (cleanup)
+ *           2010-12-30 (state.zeros == 2 Trailing Zeros fixed)
+ *           2009-07-10 (// state.g = 0; -getLayerPolarity: ![scanner isAtEnd])
+ *           2006-03-24
  *
  * This file is part of the vhf Import Library.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.cenon.de
  */
 
 #include <math.h>
@@ -521,40 +524,73 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
         state.path = 0;
     }
     else if ( [scanner scanString:ops.coordX intoString:NULL] )
-    {
-        if ( [scanner scanFloat:&value] )
-        {   state.x = DeviceResToInternal(value, res);
+    {   NSString    *str;
+
+        if ( state.zeros != 2 && [scanner scanFloat:&value] ) // Leading Zeros and dezimal points
+        {
+            state.x = DeviceResToInternal(value, res);
+            state.draw = 1;
+        }
+        else if ( state.zeros == 2 && [scanner scanCharactersFromSet:digitsSet intoString:&str] )
+        {
+            value = [str floatValue];
+            value *= pow(10, ((int)(state.formatX.x+state.formatX.y) - (int)[str length]));
+            state.x = DeviceResToInternal(value, res);
             state.draw = 1;
         }
         else
-            NSLog(@"Coordinate X expected at location: %d", [scanner scanLocation]);
+            NSLog(@"Coordinate X expected at location: %d", (unsigned)[scanner scanLocation]);
     }
     else if ( [scanner scanString:ops.coordY intoString:NULL] )
-    {
-        if ( [scanner scanFloat:&value] )
+    {   NSString    *str;
+
+        if ( state.zeros != 2 && [scanner scanFloat:&value] ) // Leading Zeros and dezimal points
         {   state.y = DeviceResToInternal(value, res);
             state.draw = 1;
         }
+        else if ( state.zeros == 2 && [scanner scanCharactersFromSet:digitsSet intoString:&str] )
+        {
+            value = [str floatValue];
+            value *= pow(10, ((int)(state.formatY.x+state.formatY.y) - (int)[str length]));
+            state.y = DeviceResToInternal(value, res);
+            state.draw = 1;
+        }
         else
-            NSLog(@"Coordinate Y expected at location: %d", [scanner scanLocation]);
+            NSLog(@"Coordinate Y expected at location: %d", (unsigned)[scanner scanLocation]);
     }
     else if ( [scanner scanString:ops.coordI intoString:NULL] )
-    {
-        if ( [scanner scanFloat:&value] )
+    {   NSString    *str;
+
+        if ( state.zeros != 2 && [scanner scanFloat:&value] ) // Leading Zeros and dezimal points
         {   state.i = DeviceResToInternal(value, res);
             state.draw = 1;
         }
-        else
-            NSLog(@"Coordinate I expected at location: %d", [scanner scanLocation]);
-    }
-    else if ( [scanner scanString:ops.coordJ intoString:NULL] )
-    {
-        if ( [scanner scanFloat:&value] )
-        {   state.j = DeviceResToInternal(value, res);
+        else if ( state.zeros == 2 && [scanner scanCharactersFromSet:digitsSet intoString:&str] )
+        {
+            value = [str floatValue];
+            value *= pow(10, ((int)(state.formatX.x+state.formatX.y) - (int)[str length]));
+            state.i = DeviceResToInternal(value, res);
             state.draw = 1;
         }
         else
-            NSLog(@"Coordinate J expected at location: %d", [scanner scanLocation]);
+            NSLog(@"Coordinate I expected at location: %d", (unsigned)[scanner scanLocation]);
+    }
+    else if ( [scanner scanString:ops.coordJ intoString:NULL] )
+    {   NSString    *str;
+
+        if ( state.zeros != 2 && [scanner scanFloat:&value] ) // Leading Zeros and dezimal points
+        {   state.j = DeviceResToInternal(value, res);
+            state.draw = 1;
+        }
+        else if ( state.zeros == 2 && [scanner scanCharactersFromSet:digitsSet intoString:&str] )
+        {
+            value = [str floatValue];
+            value *= pow(10, ((int)(state.formatX.x+state.formatX.y) - (int)[str length]));
+            state.j = DeviceResToInternal(value, res);
+            state.draw = 1;
+        }
+        else
+            NSLog(@"Coordinate J expected at location: %d", (unsigned)[scanner scanLocation]);
     }
     else if ( [scanner scanString:ops.ipolQuarter intoString:NULL] )
         state.ipolFull = 0;
@@ -605,7 +641,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
                         default:
                             [self setLine:cList];
                     }
-                    state.g = 0;
+//                    state.g = 0;
                     break;
                 case 3: /* flash */
                     [self setPad:list];
@@ -892,7 +928,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
         else // the values follows directly behind , (comma) -> we must combine them with $1 values in macroData
         {   NSScanner	*mScanner = [NSScanner scannerWithString:macroData];
             NSString	*searchStr = [NSString stringWithFormat:@"%%AM%@", macroName];
-            NSRange	range;
+            NSRange	range, rangeHack;
             int		i, cnt = 0;
             float	vals[200]; // I hope no one will write an greater macro
 
@@ -914,10 +950,48 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
             }
             // replace $1, $2 $n with values from scanner
             range = [*macroStr rangeOfString:@"="];
+            /* Eagle WorkAround */
+            rangeHack = [*macroStr rangeOfString:@"X$"];
             for (i=0; i<cnt; i++)
             {   NSString	*from = [NSString stringWithFormat:@"$%d", i+1];
                 NSString	*to = [NSString stringWithFormat:@"%.3f", vals[i]];
 
+                /* Eagle WorkAround */
+                if ( rangeHack.length )
+                {   NSScanner	*mcrScanner = [NSScanner scannerWithString:*macroStr];
+                    NSString	*string;
+
+                    NSLog(@"Eagle WorkAround wurde aktiviert\n");
+                    /*  * 5,1,8,0,0,1.08239X$1,22.5  - werte zwischen den Kommas */
+                    /* we must remove the float before/and the X */
+                    /* and we must set the last float to 0 - rotation of polygon */
+
+                    while ( ![mcrScanner isAtEnd] )
+                    {
+                        [mcrScanner scanUpToString:@"," intoString:&string];
+                        rangeHack = [string rangeOfString:@"X$"];
+                        /* this is our string part - wert zwischen den kommas */
+                        if ( rangeHack.length )
+                        {   NSScanner		*mmScanner = [NSScanner scannerWithString:string];
+                            NSMutableString	*fromHack = [NSMutableString stringWithString:string], *toHack;
+                            NSString		*restString;
+
+                            [mcrScanner scanUpToString:@"*" intoString:&restString]; // ,22.5 rotation
+                            [fromHack appendString:restString];
+
+                            [mmScanner scanUpToString:@"$" intoString:NULL]; // bis $
+                            [mmScanner scanUpToString:@"," intoString:&toHack]; // bis , -> $1 oder $3
+
+                            /* and we must set the last float to 0 - rotation of polygon */
+                            toHack = [NSMutableString stringWithString:toHack];
+                            [toHack appendString:@",0"]; // no rotation
+                            *macroStr = [*macroStr stringByReplacing:fromHack by:toHack];
+                            break;
+                        }
+                        [mcrScanner scanString:@"," intoString:NULL];
+                    }
+                }
+
                 if ( !range.length )
                     *macroStr = [*macroStr stringByReplacing:from by:to];
                 else
@@ -965,7 +1039,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
 - (BOOL)getLayerPolarity:(NSScanner*)scanner
 {
     // until %
-    while ( ![scanner scanString:ops.RS274X intoString:NULL] )
+    while ( ![scanner scanString:ops.RS274X intoString:NULL] && ![scanner isAtEnd] )
     {
         if ( [scanner scanString:@"LP" intoString:NULL] ) // no layer polarity
         {
@@ -1000,7 +1074,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
 {
     if ( (state.tool = [self toolFromString:scanner]) < 0 )
     {
-        NSLog(@"Gerber import, Can't find tool at location %d. Default used.", [scanner scanLocation]);
+        NSLog(@"Gerber import, Can't find tool at location %d. Default used.", (unsigned)[scanner scanLocation]);
         state.tool = 0;
     }
     [scanner scanUpToString:ops.termi intoString:NULL];
@@ -1559,7 +1633,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
         [self addFillPath:dList toList:mList];
         dList=[[[self allocateList] init] autorelease];
 
-        // set 4 points of horicontal (   ) line
+        // set 4 points of horicontal line
         p[0].x = cx - lenCross/2.0; p[0].y = cy - wCross/2.0;
         p[1].x = p[0].x + lenCross; p[1].y = p[0].y;
         p[2].x = p[1].x; p[2].y = p[1].y + wCross;
@@ -1656,7 +1730,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
 
         [self addFillPath:dList toList:mList];
 
-        // set 4 points of horicontal (   ) line
+        // set 4 points of horicontal ( –  ) line
         p[0].x = cx - (dOut2 + 1.0); p[0].y = cy - (wCross/2.0 + 1.0);
         p[1].x = p[0].x + dOut + 2.0; p[1].y = p[0].y;
         p[2].x = p[1].x; p[2].y = p[1].y + wCross + 2.0;
@@ -1835,16 +1909,16 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
     }
     /* macro */
     else if ( [[tool objectForKey:@"formCode"] isEqual:formM] )
-    {	NSScanner	*scanner = [NSScanner scannerWithString:[tool objectForKey:@"macro"]];
-        NSMutableArray	*myList = [[[self allocateList] init] autorelease];
-        int		location = [scanner scanLocation];
-        NSPoint		llur[2]; // ll ur of macro !
+    {	NSScanner       *scanner = [NSScanner scannerWithString:[tool objectForKey:@"macro"]];
+        NSMutableArray  *myList = [[[self allocateList] init] autorelease];
+        int             location = [scanner scanLocation];
+        NSPoint         llur[2]; // ll ur of macro !
 
         llur[0].x = llur[0].y = LARGE_COORD;
         llur[1].x = llur[1].y = LARGENEG_COORD;
 
         while ( ![scanner isAtEnd] )
-        {   int	val;
+        {   int	val = 0;
 
             [scanner setScanLocation:location];
             [scanner scanUpToString:ops.termi intoString:NULL];
@@ -1916,7 +1990,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
 
 /*
  * created:   07.05.93
- * modified:  17.06.93 05.05.96
+ * modified:  17.06.93 05.05.96 2009-07-10 (//state.g = 0)
  * purpose:   set arc ccw
  * parameter: g
  */
@@ -1925,7 +1999,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
     float		ba, ea, angle;
     NSDictionary	*tool = [tools objectAtIndex:state.tool];
 
-    state.g = 0; // else we go on with an arc
+//    state.g = 0; // else we go on with an arc
 
     if ( !state.path )
         state.width = [[tool objectForKey:@"width"] floatValue];
@@ -2107,7 +2181,7 @@ void replaceFromByTo(NSString** macroStr, NSString* from, NSString* to, int cnt,
  */
 - (void)addText:(NSString*)text :(NSString*)font :(float)angle :(float)size :(float)ar at:(NSPoint)p toList:aList
 {
-    NSLog(@"text: %f %f %f %f %f \"%s\" \"%s\"", p.x, p.y, angle, size, ar, text, font); 
+    NSLog(@"text: %f %f %f %f %f \"%@\" \"%@\"", p.x, p.y, angle, size, ar, text, font); 
 }
 
 - (void)moveListBy:(NSPoint)pt :aList
