@@ -1,36 +1,37 @@
 /* HPGLImport.m
  * HPGL import object
  *
- * Copyright (C) 1996-2006 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-03
- * modified: 2006-11-21
+ * modified: 2012-01-05 (-getLabelSize: scanDouble and make it work with all sizes of NSPoint)
+ *           2008-06-15 (-getGraphicFromData: imply PD if we get an arc)
  *
  * This file is part of the vhf Import Library.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.cenon.info
  */
 
 #include <math.h>
@@ -134,7 +135,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
     return self;
 }
 
-/* created:   03.05.96
+/* created:   1996-05-03
  * modified:  09.03.97
  * parameter: fileName
  * purpose:   load parameter file
@@ -247,10 +248,10 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
 /* private methods
  */
 - (BOOL)interpret:(NSString*)dataP
-{   id			cList;			// current list
-    NSRect		bounds;
-    NSScanner		*scanner = [NSScanner scannerWithString:dataP];
-    NSCharacterSet	*skipSet = [NSCharacterSet characterSetWithCharactersInString:NOP];
+{   id              cList;			// current list
+    NSRect          bounds;
+    NSScanner       *scanner = [NSScanner scannerWithString:dataP];
+    NSCharacterSet  *skipSet = [NSCharacterSet characterSetWithCharactersInString:NOP];
 
     digitsSet = [NSCharacterSet characterSetWithCharactersInString:DIGITS];
     invDigitsSet = [digitsSet invertedSet];
@@ -292,6 +293,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
 }
 
 /* we need cp on a number !
+ * modified: 2008-06-15
  */
 - (BOOL)getGraphicFromData:(NSScanner*)scanner :cList
 {   NSPoint	p0, p1;
@@ -338,7 +340,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
     }
     if ( [scanner scanString:ops.penDown intoString:NULL] )
     {   state.g = HPGL_LINE;
-        state.pd = 1;
+        state.pd   = 1;
         state.draw = 1;
     }
     else if ( [scanner scanString:ops.circle intoString:NULL] )
@@ -353,11 +355,13 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
     else if ( [scanner scanString:ops.arcAbs intoString:NULL] )
     {	state.g = HPGL_ARC;
         state.relArc = 0;
+        state.pd   = 1; // we can imply a pen down for this
         state.draw = 1;
     }
     else if ( [scanner scanString:ops.arcRel intoString:NULL] )
     {	state.g = HPGL_ARC;
         state.relArc = 1;
+        state.pd   = 1; // we can imply a pen down for this
         state.draw = 1;
     }
     else if ( [scanner scanString:ops.polygonDef intoString:NULL] )
@@ -383,13 +387,13 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
     {	state.draw = 1;
 	state.pa = 0;
     }
-    else if ( [scanner scanString:ops.labelSize intoString:NULL] )		/* label size */
+    else if ( [scanner scanString:ops.labelSize intoString:NULL] )  // label size
         [self getLabelSize:scanner];
-    else if ( [scanner scanString:ops.labelDir intoString:NULL] )		/* label direction */
+    else if ( [scanner scanString:ops.labelDir intoString:NULL] )   // label direction
         [self getLabelDir:scanner];
-    else if ( [scanner scanString:ops.labelSlant intoString:NULL] )	/* label slant */
+    else if ( [scanner scanString:ops.labelSlant intoString:NULL] ) // label slant
         [self getLabelSlant:scanner];
-    else if ( [scanner scanString:ops.label intoString:NULL] )		/* label */
+    else if ( [scanner scanString:ops.label intoString:NULL] )      // label
     {	NSPoint	origin;
         float		angle, size, ar;
         NSString	*string;
@@ -397,9 +401,9 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
         if ([self getLabel:scanner :&string :&angle :&origin :&size :&ar])
             [self addText:string :@"Courier" :angle :size :ar at:origin toList:list];
     }
-    else if ( [scanner scanString:ops.lineType intoString:NULL] )	/* lineType */
+    else if ( [scanner scanString:ops.lineType intoString:NULL] )   // lineType
         [self getLineType:scanner];
-    else if ( [scanner scanString:ops.inputp1p2 intoString:NULL] )	/* inputp1p2 */
+    else if ( [scanner scanString:ops.inputp1p2 intoString:NULL] )  // inputp1p2
         [self getInputP1P2:scanner];
     else if ( [scanner scanCharactersFromSet:termiSet intoString:NULL] )
     {	state.draw = 0;
@@ -445,29 +449,32 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
 }
 
 /*
- * created:   18.04.94
- * modified:  18.04.94 04.05.96 09.03.97
+ * created:   1994-04-18
+ * modified:  2012-01-05 (scanDouble and make it work with all sizes of NSPoint)
  * purpose:   get label size
  *            the size comes in cm, we store it in internal units
  * parameter: cp (at or behind command)
  */
 - (BOOL)getLabelSize:(NSScanner*)scanner
-{
+{   double  d;
+
     [scanner scanString:ops.labelSize intoString:NULL];
 
-    if ( ![scanner scanFloat:&state.labelSize.width] )
-        state.labelSize.width = state.labelSize.height = 0.0;
-    else if ( ![scanner scanFloat:&state.labelSize.height] )
-    {   NSLog(@"%@ (height) expected at location:%d", ops.labelSize, [scanner scanLocation]);
-        state.labelSize.height = 0.0;
+    state.labelSize = NSZeroSize;
+    if ( [scanner scanDouble:&d] )
+    {   state.labelSize.width = d;
+        if ( [scanner scanDouble:&d] )
+            state.labelSize.height = d;
+        else
+            NSLog(@"%@ (height) expected at location:%d", ops.labelSize, (unsigned)[scanner scanLocation]);
     }
 
-    if ( state.labelSize.width == 0.0 )	/* default size */
+    if ( state.labelSize.width == 0.0 )	// default size
         state.labelSize.width = 11.5 * PT;
     else
         state.labelSize.width *= MM*10.0;
 
-    if (state.labelSize.height == 0.0)	/* default size */
+    if (state.labelSize.height == 0.0)	// default size
         state.labelSize.height = 11.5 * PT;
     else
         state.labelSize.height *= MM*10.0;
@@ -506,7 +513,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
     if ( ![scanner scanFloat:&run] )
         run = rise = 0.0;
     else if ( ![scanner scanFloat:&rise] )
-        NSLog(@"'%@' (rise) expected at location: %d", ops.labelDir, [scanner scanLocation]);
+        NSLog(@"'%@' (rise) expected at location: %d", ops.labelDir, (unsigned)[scanner scanLocation]);
 
     if (run == 0.0 && rise == 0.0)
         state.labelDir = 0.0;
@@ -536,9 +543,9 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
 {   float	x, y;
 
     if ( ![scanner scanFloat:&x] )
-        NSLog(@"getPoint: (x) expected at location: %d", [scanner scanLocation]);
+        NSLog(@"getPoint: (x) expected at location: %d", (unsigned)[scanner scanLocation]);
     if ( ![scanner scanFloat:&y] )
-        NSLog(@"getPoint: (y) expected at location: %d", [scanner scanLocation]);
+        NSLog(@"getPoint: (y) expected at location: %d", (unsigned)[scanner scanLocation]);
     x = DeviceResToInternal(x, res);
     y = DeviceResToInternal(y, res);
     if (state.pa)
@@ -568,9 +575,9 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
 
     *p0 = state.point;
     if ( ![scanner scanFloat:&x] )
-        NSLog(@"getLine::: (x) expected at location: %d", [scanner scanLocation]);
+        NSLog(@"getLine::: (x) expected at location: %d", (unsigned)[scanner scanLocation]);
     if ( ![scanner scanFloat:&y] )
-        NSLog(@"getLine::: (y) expected at location: %d", [scanner scanLocation]);
+        NSLog(@"getLine::: (y) expected at location: %d", (unsigned)[scanner scanLocation]);
     x = DeviceResToInternal(x, res);
     y = DeviceResToInternal(y, res);
     if (state.pa)
@@ -606,7 +613,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
     *angle = 0.0;
 
     if ( ![scanner scanFloat:&r] )
-        NSLog(@"getCircle::: radius expected at location: %d", [scanner scanLocation]);
+        NSLog(@"getCircle::: radius expected at location: %d", (unsigned)[scanner scanLocation]);
     r = DeviceResToInternal(r, res);
 
     /* radius! */
@@ -641,9 +648,9 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
 
     *angle = 0.0;
     if ( ![scanner scanFloat:&x] )
-        NSLog(@"getArc::: center x expected at location: %d", [scanner scanLocation]);
+        NSLog(@"getArc::: center x expected at location: %d", (unsigned)[scanner scanLocation]);
     if ( ![scanner scanFloat:&y] )
-        NSLog(@"getArc::: center y expected at location: %d", [scanner scanLocation]);
+        NSLog(@"getArc::: center y expected at location: %d", (unsigned)[scanner scanLocation]);
     x = DeviceResToInternal(x, res);
     y = DeviceResToInternal(y, res);
     if ( !state.relArc )
@@ -658,7 +665,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
     *start = state.point;
 
     if ( ![scanner scanFloat:&value] )
-        NSLog(@"getLine::: angle expected at location: %d", [scanner scanLocation]);
+        NSLog(@"getLine::: angle expected at location: %d", (unsigned)[scanner scanLocation]);
     *angle = value;
 
     [scanner scanFloat:&value];
@@ -702,7 +709,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
     //	*ar = state.labelSize.height/state.labelSize.width;
 
     if ( ![scanner scanUpToCharactersFromSet:labelTermiSet intoString:string] )
-    {   NSLog(@"'%@' string expected at location: %d", ops.label, [scanner scanLocation]);
+    {   NSLog(@"'%@' string expected at location: %d", ops.label, (unsigned)[scanner scanLocation]);
         return NO;
     }
     /* end of text */
@@ -727,7 +734,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
 {   float	value;
 
     if ( ![scanner scanFloat:&value] )
-        NSLog(@"'%@' (polygon mode) expected at location: %d", ops.polygonDef, [scanner scanLocation]);
+        NSLog(@"'%@' (polygon mode) expected at location: %d", ops.polygonDef, (unsigned)[scanner scanLocation]);
 
     switch ((int)value)
     {
@@ -758,13 +765,13 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
     [scanner scanString:ops.inputWindow intoString:NULL];
 
     if ( ![scanner scanFloat:&value] )
-        NSLog(@"'%@' (x) expected at location: %d", ops.inputWindow, [scanner scanLocation]);
+        NSLog(@"'%@' (x) expected at location: %d", ops.inputWindow, (unsigned)[scanner scanLocation]);
     if ( ![scanner scanFloat:&value] )
-        NSLog(@"'%@' (y) expected at location: %d", ops.inputWindow, [scanner scanLocation]);
+        NSLog(@"'%@' (y) expected at location: %d", ops.inputWindow, (unsigned)[scanner scanLocation]);
     if ( ![scanner scanFloat:&value] )
-        NSLog(@"'%@' (w) expected at location: %d", ops.inputWindow, [scanner scanLocation]);
+        NSLog(@"'%@' (w) expected at location: %d", ops.inputWindow, (unsigned)[scanner scanLocation]);
     if ( ![scanner scanFloat:&value] )
-        NSLog(@"'%@' (h) expected at location: %d", ops.inputWindow, [scanner scanLocation]);
+        NSLog(@"'%@' (h) expected at location: %d", ops.inputWindow, (unsigned)[scanner scanLocation]);
 
     return YES;
 }
@@ -827,15 +834,15 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
     state.p1.x = DeviceResToInternal(value, res);
 
     if ( ![scanner scanFloat:&value] )
-        NSLog(@"'%@' (p1 y) expected at location: %d", ops.inputp1p2, [scanner scanLocation]);
+        NSLog(@"'%@' (p1 y) expected at location: %d", ops.inputp1p2, (unsigned)[scanner scanLocation]);
     state.p1.y = DeviceResToInternal(value, res);
 
     if ( ![scanner scanFloat:&value] )
-        NSLog(@"'%@' (p2 x) expected at location: %d", ops.inputp1p2, [scanner scanLocation]);
+        NSLog(@"'%@' (p2 x) expected at location: %d", ops.inputp1p2, (unsigned)[scanner scanLocation]);
     state.p2.x = DeviceResToInternal(value, res);
 
     if ( ![scanner scanFloat:&value] )
-        NSLog(@"'%@' (p2 y) expected at location: %d", ops.inputp1p2, [scanner scanLocation]);
+        NSLog(@"'%@' (p2 y) expected at location: %d", ops.inputp1p2, (unsigned)[scanner scanLocation]);
     state.p2.y = DeviceResToInternal(value, res);
 
     return YES;
@@ -896,7 +903,7 @@ static void addToBeginNew(NSString *newOp, NSMutableString *beginNew)
  */
 - (void)addText:(NSString*)text :(NSString*)font :(float)angle :(float)size :(float)ar at:(NSPoint)p toList:aList
 {
-    NSLog(@"text: %f %f %f %f %f \"%s\" \"%s\"\n", p.x, p.y, angle, size, ar, text, font); 
+    NSLog(@"text: %f %f %f %f %f \"%@\" \"%@\"\n", p.x, p.y, angle, size, ar, text, font); 
 }
 
 - (void)setBounds:(NSRect)bounds
