@@ -1,44 +1,51 @@
 /* PSImport.m
  * PostScript import object
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-25
- * modified: 2005-03-31
+ * modified: 2014-07-04 (-importPDFFromFile:, -importPS: alert, if no ghostscript is installed)
+ *           2013-12-19 (-importPS: dataIn is NSString, data is NSMutableString)
+ *           2011-09-03 (-importPS: for GhostScript: try several string loadings to get encoding right)
+ *           2009-02-02 (-importPS: for GhostScript: -dNOSAFER flag added
+ *           2009-01-31 (-importPS: for GhostScript+DPS: remove Corel Draw garbage before %!PS-ADOBE...)
+ *           2008-11-29 (init state.color in GhostScript version of importPS)
  *
  * This file is part of the vhf Import Library.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.cenon.de
  */
 
 #include <ctype.h>
 #include <math.h>
 #include "PSImport.h"
+#include "../VHFShared/vhfCommonFunctions.h"
 #include "../VHFShared/vhf2DFunctions.h"
 #include "../VHFShared/types.h"
 #include "../VHFShared/VHFScannerAdditions.h"
+#include "../VHFShared/VHFStringAdditions.h"    // -writeToFile:... for 10.4
 
 /* USEDPS: 1 = use Display PostScript, 0 = use gs (GhostScript) */
 #if defined(GNUSTEP_BASE_VERSION)
@@ -68,6 +75,20 @@
 
 @implementation PSImport
 
+/* return a temporary path
+ * created: 2012-02-07
+ */
+- (NSString*)tmpPath
+{   NSString    *tmpPath = PS_TMPPATH;
+    //NSString    *app = [[NSProcessInfo processInfo] processName];
+
+#   if defined(__APPLE__) || defined(GNUSTEP_BASE_VERSION)
+    tmpPath = NSTemporaryDirectory();
+#   endif
+    //tmpPath = [tmpPath stringByAppendingPathComponent:APPNAME];
+    return tmpPath;
+}
+
 /* created:   1996-05-03
  * modified:  
  * parameter: flag
@@ -100,14 +121,15 @@
 }
 
 /* created:   2002-01-29
- * modified:  2005-01-04
+ * modified:  2014-07-04 (alert user, if no ghostscript installed)
  * parameter: pdfData	the PDF data stream
  * purpose:   import pdf data
  */
 - importPDF:(NSData*)pdfData
-{   NSString	*app = [[NSProcessInfo processInfo] processName];
-    NSString	*pdfFile = [NSString stringWithFormat:@"/tmp/%@_pdfImport.pdf", app];
-    id		result;
+{   NSString    *app = [[NSProcessInfo processInfo] processName];
+    NSString    *pdfFile = vhfPathWithPathComponents([self tmpPath],
+                           [app stringByAppendingString:@"_pdfImport.pdf"], nil);
+    id          result;
 
     [pdfData writeToFile:pdfFile atomically:NO];
     result = [self importPDFFromFile:pdfFile];
@@ -116,13 +138,33 @@
     return result;
 }
 - importPDFFromFile:(NSString*)pdfFile
-{   NSString	*app = [[NSProcessInfo processInfo] processName], *commandLine;
-    NSString	*psFile = [NSString stringWithFormat:@"/tmp/%@_pdfImport.eps", app];
-    NSData	*psData;
+{   NSString    *app = [[NSProcessInfo processInfo] processName], *commandLine;
+    NSString    *psFile = vhfPathWithPathComponents([self tmpPath],
+                          [app stringByAppendingString:@"_pdfImport.eps"], nil);
+    NSData      *psData;
+
+    /* TODO: multiple page import: split up multiple page PDFs into single files
+     *  You might want to print each page separately. To do this, send the
+     *  output to a series of files "foo1.xyz, foo2.xyz, ..." using the
+     *  "-sOutputFile=" switch with "%d" in a filename template:
+     *  Each resulting file receives one page of output, and the files are
+     *  numbered in sequence. "%d" is a printf format specification; you can
+     *  also use a variant like "%02d".
+     *  -sOutputFile=foo%d.xyz
+     */
 
     /* convert pdf to eps */
-    commandLine = [NSString stringWithFormat:@"%@ -q -dNOPAUSE -dBATCH -dSAFER -sDEVICE=epswrite -sOutputFile=%@ -c save pop -f '%@'", [self gsPath], psFile, pdfFile];
-    system([commandLine cString]);
+    commandLine = [NSString stringWithFormat:@"%@ -q -dNOPAUSE -dBATCH -dSAFER -sDEVICE=epswrite -sOutputFile=%@ -c save pop -f '%@'",
+                            [self gsPath], psFile, pdfFile];
+    //system([commandLine UTF8String]);
+    if (system([commandLine UTF8String]) != 0)
+    {
+        NSRunAlertPanel(@"Unable to import file",
+                        @"An error occured while importing the file.\n"
+                        @"Make sure you have ghostscript installed in PATH.",
+                        @"OK", nil, nil);
+        return nil;
+    }
 
     /* import ps */
     psData = [NSData dataWithContentsOfFile:psFile];
@@ -147,19 +189,42 @@
  * We can't use NSEPSImageRep because the context is write protected
  */
 - importPS:(NSData*)psData
-{   NSAutoreleasePool	*pool = [NSAutoreleasePool new];
-    NSDPSContext	*curContext = [NSDPSContext currentContext];
-    NSDPSServerContext	*psContext = nil;
-    NSString		*app = [[NSProcessInfo processInfo] processName];
-    NSString		*path = [NSString stringWithFormat:@"/tmp/%@_psImport", app];
-    NSString		*data;
-    NSUserDefaults	*userDefaults = [NSUserDefaults standardUserDefaults];
-    NSString		*hostName = [userDefaults stringForKey:@"NSHost"];
-    NSString		*serverName = [userDefaults stringForKey:@"NSPSName"];
-    NSTimeInterval	timeout = [userDefaults floatForKey:@"NSNetTimeout"];
+{   NSAutoreleasePool   *pool = [NSAutoreleasePool new];
+    NSDPSContext        *curContext = [NSDPSContext currentContext];
+    NSDPSServerContext  *psContext = nil;
+    NSString            *app = [[NSProcessInfo processInfo] processName];
+    NSString            *path = vhfPathWithPathComponents([self tmpPath],
+                                [app stringByAppendingString:@"_psImport"], nil);
+    NSString            *data;
+    NSUserDefaults      *userDefaults = [NSUserDefaults standardUserDefaults];
+    NSString            *hostName = [userDefaults stringForKey:@"NSHost"];
+    NSString            *serverName = [userDefaults stringForKey:@"NSPSName"];
+    NSTimeInterval      timeout = [userDefaults floatForKey:@"NSNetTimeout"];
 
     if ( !psData )
         return nil;
+    /* Workaround for Windows Corel Draw: if psData contains garbage before %!PS-Adobe, we remove that.
+     * garbage at the end is only removed if file ends with "%%EOF".
+     */
+    {   NSString    *psStr, *psStr1;
+        NSRange     range, range1;
+
+        psStr = [[NSString alloc] initWithData:psData encoding:NSASCIIStringEncoding];
+        if ( (range = [psStr rangeOfString:@"%!PS"]).length && range.location > 0 )  // garbage%!PS-Adobe
+        {   int toIx;
+
+            NSLog(@"PostScript-Import: Error in PostScript file - data starts with garbage (%@) !", [psStr substringToIndex:range.location]);
+            if ( ! (range1 = [psStr rangeOfString:@"%%EOF" options:NSBackwardsSearch]).length )
+                range1 = [psStr rangeOfString:@"showpage" options:NSBackwardsSearch];
+            toIx = ((range1.length) ? range1.location+range1.length+1 : [psStr length]);
+            psStr1 = [psStr substringWithRange:NSMakeRange(range.location, toIx-range.location)];
+            [psStr release];
+            psData = [psStr1 dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES];
+        }
+        else
+            [psStr release];
+    }
+
     state.color = [[NSColor colorWithCalibratedRed:0.0 green:0.0 blue:0.0 alpha:1.0] retain];
     state.width = 0.0;
 
@@ -206,15 +271,45 @@
  */
 #else // use GhostScript
 
+/*
+ * modified:  2014-07-04 (alert user, if no ghostscript installed)
+ */
 static NSString	*prolog = nil;
 - importPS:(NSData*)psData
-{   NSString		*app = [[NSProcessInfo processInfo] processName], *commandLine, *path;
-    NSString		*inFile  = [NSString stringWithFormat:@"/tmp/%@_psImport.in", app];
-    NSString		*outFile = [NSString stringWithFormat:@"/tmp/psImport.out"]; // see psImport.prolog
-    NSMutableString	*data = [NSMutableString string];
+{   NSString            *app = [[NSProcessInfo processInfo] processName], *commandLine, *path;
+    NSString            *inFile  = vhfPathWithPathComponents([self tmpPath], [app stringByAppendingString:@"_psImport.in"],  nil);
+    //NSString            *outFile = vhfPathWithPathComponents(PS_TMPPATH, @"_psImport.out", nil);    // see psImport.prolog !
+    NSString            *outFile = vhfPathWithPathComponents([self tmpPath], [app stringByAppendingString:@"_psImport.out"], nil); // see psImport.prolog
+    NSMutableString     *data = [NSMutableString string];
+    NSString            *dataIn;
+    NSStringEncoding    enc;
 
     if ( !psData )
         return nil;
+    /* Workaround for Windows Corel Draw: if psData contains garbage before %!PS-Adobe, we remove that.
+     * garbage at the end is only removed if file ends with "%%EOF".
+     */
+    {   NSString    *psStr, *psStr1;
+        NSRange     range, range1;
+
+        psStr = [[NSString alloc] initWithData:psData encoding:NSASCIIStringEncoding];
+        if ( (range = [psStr rangeOfString:@"%!PS"]).length && range.location > 0 )  // garbage%!PS-Adobe
+        {   int toIx;
+
+            NSLog(@"PostScript-Import: Error in PostScript file - data starts with garbage (%@) !", [psStr substringToIndex:range.location]);
+            if ( ! (range1 = [psStr rangeOfString:@"%%EOF" options:NSBackwardsSearch]).length )
+                range1 = [psStr rangeOfString:@"showpage" options:NSBackwardsSearch];
+            toIx = ((range1.length) ? range1.location+range1.length+1 : [psStr length]);
+            psStr1 = [psStr substringWithRange:NSMakeRange(range.location, toIx-range.location)];
+            [psStr release];
+            psData = [psStr1 dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES];
+        }
+        else
+            [psStr release];
+    }
+
+    state.color = [[NSColor colorWithCalibratedRed:0.0 green:0.0 blue:0.0 alpha:1.0] retain];
+    state.width = 0.0;
 
     /* load prolog from framework or library directory */
     if ( !prolog )
@@ -222,7 +317,21 @@ static NSString	*prolog = nil;
         path = [[NSBundle bundleForClass:[PSImport class]] resourcePath];
         path = [path stringByAppendingPathComponent:@"psImport.prolog"];
         prolog = [[NSString stringWithContentsOfFile:path] retain];
-        if (!prolog)
+        if (prolog)
+        {   NSRange range = [prolog rangeOfString:@"/tmp/psImport.out"];
+
+            /* set file name */
+            if ( [prolog respondsToSelector:@selector(stringByReplacingCharactersInRange:withString:)] )    // SDK >= 10.5
+                prolog = [[[prolog autorelease] stringByReplacingCharactersInRange:range withString:outFile] retain];
+            else
+            {   NSMutableString *newProlog = [prolog mutableCopy];
+
+                [prolog release];
+                [newProlog replaceCharactersInRange:range withString:outFile];
+                prolog = newProlog;
+            }
+        }
+        else
             NSLog(@"Couldn't load psImport.prolog file '%@'", path);
     }
 
@@ -240,18 +349,32 @@ static NSString	*prolog = nil;
     {   NSData	*nsData;
 
         nsData = [data dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES];
-        data = [[[NSString alloc] initWithData:nsData encoding:NSASCIIStringEncoding] autorelease];
+        data = [[[NSMutableString alloc] initWithData:nsData encoding:NSASCIIStringEncoding] autorelease];
     }
-    [data writeToFile:inFile atomically:NO];
+    [data writeToFile:inFile atomically:NO encoding:NSASCIIStringEncoding error:NULL];  // >= 10.5
+    //[data writeToFile:inFile atomically:NO];    // <= 10.4
 
-    commandLine = [NSString stringWithFormat:@"%@ -dNODISPLAY -dBATCH -g1000x1000 '%@'", [self gsPath], inFile];
-    system([commandLine cString]);
+    commandLine = [NSString stringWithFormat:@"%@ -dNODISPLAY -dBATCH -dNOSAFER -g1000x1000 '%@'", [self gsPath], inFile];
+    //system([commandLine UTF8String]);
+    if (system([commandLine UTF8String]) != 0)
+    {
+        NSRunAlertPanel(@"Unable to import file",
+                        @"An error occured while importing the file.\n"
+                        @"Make sure you have ghostscript installed in PATH.",
+                        @"OK", nil, nil);
+        return nil;
+    }
 
     /* load generated file */
-    data = [[[NSString alloc] initWithContentsOfFile:outFile] autorelease];
+    //data = [[[NSString alloc] initWithContentsOfFile:outFile] autorelease];
+    dataIn = [[[NSString alloc] initWithContentsOfFile:outFile usedEncoding:&enc error:NULL] autorelease];
+    if ( !dataIn )    // files from Corel Draw, etc.
+        dataIn = [[[NSString alloc] initWithContentsOfFile:outFile encoding:NSISOLatin1StringEncoding error:NULL] autorelease];
+    if ( !dataIn )    // files from Next-Step (ex. Example file with Ducks)
+        dataIn = [[[NSString alloc] initWithContentsOfFile:outFile encoding:NSNEXTSTEPStringEncoding error:NULL] autorelease];
 
     /* interpret data and send to receiving object */
-    if ( !data || ![self interpret:data] )
+    if ( !dataIn || ![self interpret:dataIn] )
         return nil;
 
     [[NSFileManager defaultManager] removeFileAtPath:inFile  handler:nil];
@@ -281,20 +404,20 @@ static NSString	*prolog = nil;
 
 #define DESCALE	0.5
 - (BOOL)interpret:(NSString*)data
-{   NSString		*string;
-    float		value, c[MAXCOORDCNT];		// stack for coordinates
-    NSMutableArray	*s = [NSMutableArray array];	// stack for strings
-    int			cCnt = 0, poolCnt = 0;
-    NSPoint		p0, p1, p2, p3;
-    NSMutableArray	*cList = [NSMutableArray array];	// current list
-    NSRect		bounds;
-    PSState		states[MAXSTATES];	// array of states for gsave and grestore
-    int			stateCnt = 0;
-    NSScanner		*scanner = [NSScanner scannerWithString:data];
-    NSCharacterSet	*skipCharacters = [scanner charactersToBeSkipped];
-    NSCharacterSet	*noCharacters = [NSCharacterSet characterSetWithCharactersInString:@""];
-    NSAutoreleasePool	*pool = nil;
-    BOOL		ok = YES;
+{   NSString            *string;
+    float               value, c[MAXCOORDCNT];          // stack for coordinates
+    NSMutableArray      *s = [NSMutableArray array];    // stack for strings
+    int                 cCnt = 0, poolCnt = 0;
+    NSPoint             p0, p1, p2, p3;
+    NSMutableArray      *cList = [NSMutableArray array];    // current list
+    NSRect              bounds;
+    PSState             states[MAXSTATES];  // array of states for gsave and grestore
+    int                 stateCnt = 0;
+    NSScanner           *scanner = [NSScanner scannerWithString:data];
+    NSCharacterSet      *skipCharacters = [scanner charactersToBeSkipped];
+    NSCharacterSet      *noCharacters = [NSCharacterSet characterSetWithCharactersInString:@""];
+    NSAutoreleasePool   *pool = nil;
+    BOOL                ok = YES;
 
     coordinateSet = [NSCharacterSet characterSetWithCharactersInString:DIGITS];
     invCoordinateSet = [coordinateSet invertedSet];
