@@ -1,7 +1,7 @@
 /* VHFDictionaryAdditions.m
- * vhf NSDictionary additions
+ * NSDictionary / NSMutableDictionary additions
  *
- * Copyright (C) 1997-2009 by vhf interservice GmbH
+ * Copyright (C) 1997-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1997-07-08
@@ -10,27 +10,27 @@
  * This file is part of the vhf Shared Library.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.cenon.info
  */
 
 #include "VHFDictionaryAdditions.h"
@@ -65,6 +65,23 @@
     return [NSNumber numberWithInt:0];
 }
 
+/* created:  2010-05-07
+ * modified: 
+ */
+- (BOOL)boolForKey:(id)key
+{   id	obj = [self objectForKey:key];
+
+    if ( [obj isKindOfClass:[NSString class]] )
+    {   unichar ch = [obj characterAtIndex:0];
+
+        if ( ch == 'Y' || ch == 'y' || [obj intValue] )
+            return YES;
+    }
+    else if ( [obj isKindOfClass:[NSNumber class]] && [obj intValue])
+        return YES;
+    return NO;
+}
+
 /* created:  20.08.97
  * modified:
  */
@@ -98,6 +115,18 @@
     return 0.0;
 }
 
+/* created: 2013-12-19
+ */
+- (VFloat)VFloatForKey:(id)key
+{
+#   if CGFLOAT_IS_DOUBLE
+    return [self doubleForKey:key];
+#   else
+    return [self floatForKey:key];
+#   endif
+}
+
+
 /* created: 22.10.97
  */
 - (BOOL)containsPrefix:(NSString*)prefix
