@@ -1,49 +1,102 @@
 /* VHFStringAdditions.m
- * vhf NSString additions
+ * NSString additions
  *
- * Copyright (C) 1997-2008 by vhf interservice GmbH
+ * Copyright (C) 1997-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1997-07-08
- * modified: 2008-11-08 (inactive method for NSMutableArray parked here)
+ * modified: 2014-07-11 (-componentsSeparatedByCharactersInString: added)
+ *           2012-02-07 (-writeToFile:atomically:encoding:error: added for backward compatibility)
+ *           2011-09-01 (-stringWithContentsOfFile: loads flexible on Apple)
+ *           2008-11-08 (inactive method for NSMutableArray parked here)
  *
  * This file is part of the vhf Shared Library.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.de
+ * http://www.cenon.info
  */
 
 #include "VHFStringAdditions.h"
 
 @implementation NSString(VHFStringAdditions)
 
+/* Methods to replace deprecated and added methods with out own
+ *
+ * MAC_OS_X_VERSION_MIN_REQUIRED    Deployment target
+ * MAC_OS_X_VERSION_MAX_ALLOWED     Base SDK
+ */
++ (id)stringWithContentsOfFile:(NSString*)path error:(NSError **)error  // our creation (the way to go)
+{   NSStringEncoding	enc;
+    NSString            *string;
+
+    #   if MAC_OS_X_VERSION_MIN_REQUIRED < 1050 /*MAC_OS_X_VERSION_10_5*/           // deployment target < 10.5
+        if ( ! [self respondsToSelector:@selector(stringWithContentsOfFile:encoding:error:)] )
+            return [self stringWithContentsOfFile:path];                            // <= 10.4
+    #   endif
+    string = [self stringWithContentsOfFile:path usedEncoding:&enc error:NULL];     // >= 10.5
+    if ( !string )
+        string = [self stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:NULL];
+    return string;
+}
+
+/* Methods to replace deprecated methods (we still get warnings, we would fail on versions where this exists)
+ */
+#ifdef __APPLE__
+// FIXME: this should be visible for os x >= 10.4 only. It will crash < 10.4. It will give warning >= 10.4.
+//#pragma clang diagnostic push
+//#pragma clang diagnostic ignored "-Wobjc-protocol-method-implementation"
++ (id)stringWithContentsOfFile:(NSString*)path  // this is deprecated in os x <= 10.4
+{   NSStringEncoding	enc;
+    NSString            *string;
+
+    string = [self stringWithContentsOfFile:path usedEncoding:&enc error:NULL]; // >= 10.5
+    if ( !string )
+        string = [self stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:NULL]; // >= 10.5
+    return string;
+}
+//#pragma clang diagnostic pop
+
+/* Methods added in certain versions of Mac OS X (this will only be compiled with base SDK = 10.4)
+ */
+// FIXME: this should be compiled (visible) on os x <= 10.4 only !
+#   if MAC_OS_X_VERSION_MAX_ALLOWED < 1050 /*MAC_OS_X_VERSION_10_5*/
+    - (BOOL)writeToFile:(NSString *)path atomically:(BOOL)useAuxiliaryFile  // available >= 10.4
+               encoding:(NSStringEncoding)enc error:(NSError **)error
+    {
+        //printf("writeToFile: replacement\n");
+        return [self writeToFile:path atomically:useAuxiliaryFile];    // <= 10.4
+    }
+#   endif
+#endif
+
+
 /* return string from float with 4 decimals
  */
 + (NSString*)stringWithFloat:(float)value
-{   int			i;
+{   int             i;
     NSMutableString	*string;
-    NSRange		range;
-    char		c;
+    NSRange         range;
+    char            c;
 
     value += 0.000005;
     string = [NSMutableString stringWithFormat:@"%.4f", value];
@@ -82,11 +135,11 @@
  * return value: string
  */
 + (NSString*)stringWithFloat:(float)value decimals:(int)decimals
-{   int			i;
+{   int             i;
     NSString		*format = [NSString stringWithFormat:@"%%.%df", decimals];
     NSMutableString	*string;
-    NSRange		range;
-    char		c;
+    NSRange         range;
+    char            c;
 
     value += 0.000005;
 
@@ -119,6 +172,15 @@
 }
 
 
+- (VFloat)VFloatValue
+{
+#   if CGFLOAT_IS_DOUBLE
+    return [self doubleValue];
+#   else
+    return [self floatValue];
+#   endif
+}
+
 
 - (NSString*)stringByRemovingTrailingCharacters:(NSString*)chars
 {   NSRange	range;
@@ -143,9 +205,9 @@
     return [self stringByReplacing:from by:to all:YES];
 }
 - (NSString*)stringByReplacing:(NSString*)from by:(NSString*)to all:(BOOL)replaceAll;
-{   NSRange		range, searchRange;
-    NSMutableString	*mutString = [NSMutableString string];
-    int			start = 0;
+{   NSRange         range, searchRange;
+    NSMutableString *mutString = [NSMutableString string];
+    int             start = 0;
 
     searchRange = NSMakeRange(0, [self length]);
     while ( searchRange.length )
@@ -258,9 +320,9 @@
 }
 
 - (NSString*)stringWithLength:(int)length fillCharacter:(NSString*)fillChar
-{   int			i, space;
-    NSMutableString	*newString;
-    BOOL		alignLeft = YES;
+{   int             i, space;
+    NSMutableString *newString;
+    BOOL            alignLeft = YES;
 
     if (length < 0)
     {
@@ -340,13 +402,14 @@
  *
  * wildcards =
  * {
- *     skipNum   = "#";         // "#"       skip digits (0123456789)
+ *     skipNum   = "#";         // "#"       skip digits (0123456789) also skips trailing ".,-"
  *     skipAll   = "~";         // "~"       skip any characters
  *     skipSpace = "_";         // "_"       skip space
  *     skipChar  = "?";         // "?"       skip single character
  *     skipChars = "|";         // "|"       skip range of characters containing chars
  *     chars     = ".,- \t";    // ".,- \t"  characters to be skipped
  * }
+ * TODO: we should add a regexp syntax: [0-9] instead, and remove ".,-" from #
  */
 - (NSRange)rangeOfSequence:(NSString*)sequence options:(int)options
                      range:(NSRange)sRange wildcards:(NSDictionary*)wildcards
@@ -544,4 +607,14 @@
 }
 #endif
 
+
+/* convenience method to separate at charcters of a string
+ * created: 2014-07-11
+ */
+- (NSArray*)componentsSeparatedByCharactersInString:(NSString*)separator
+{   NSCharacterSet  *separatorSet = [NSCharacterSet characterSetWithCharactersInString:separator];
+
+    return [self componentsSeparatedByCharactersInSet:separatorSet];
+}
+
 @end
