@@ -1,46 +1,51 @@
 /* types.h
  * vhf definitions
  *
- * Copyright (C) 1995-2012 by vhf interservice GmbH
+ * Copyright (C) 1995-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1995-09-06
- * modified: 2012-02-03 (V3Point, VLine use VFloat, VFloat = CGFloat)
+ * modified: 2013-02-13 (define MAXFLOAT if not defined)
+ *           2012-02-03 (V3Point, VLine use VFloat, VFloat = CGFloat)
  *           2012-01-20 (TOLERANCE_DEG added for angular comparisons)
  *           2008-02-23 (**PointMMToInternal() added)
  *
  * This file is part of the vhf Shared Library.
  *
  * This library is ee software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by the vhf interservice GmbH. Among other things,
+ * modify it under the terms of the Cenon Public License as
+ * published by the Cenon GmbH. Among other things,
  * the License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this library; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this library; see the file LICENSE. If not, write to Cenon.
  *
  * If you want to link this library to your proprietary software,
  * or for other uses which are not covered by the definitions
- * laid down in the vhf Public License, vhf also offers a proprietary
+ * laid down in the Cenon Public License, vhf also offers a proprietary
  * license scheme. See the vhf internet pages or ask for details.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.cenon.info
  */
 
-#ifndef VHF_H_TYPES
-#define VHF_H_TYPES
+#ifndef CEN_H_TYPES
+#define CEN_H_TYPES
 
 #include <Foundation/Foundation.h>	// NSPoint
 #include <math.h>
 
+#ifndef MAXFLOAT
+#define MAXFLOAT	FLT_MAX
+#endif
+
 typedef struct _UPath   // DEPRECATED, do not used any more
 {
     float 	*pts;
@@ -54,10 +59,10 @@ typedef struct _UPath   // DEPRECATED, do not used any more
 #if    (defined(__APPLE__)            && defined(CGFLOAT_IS_DOUBLE) && CGFLOAT_IS_DOUBLE == 1) \
     || (defined(GNUSTEP_BASE_VERSION) && defined(CGFLOAT_IS_DBL)    && CGFLOAT_IS_DBL    == 1)
     typedef double VFloat;  // 64 bit
-#   define VHF_IS_DOUBLE 1
+#   define CEN_IS_DOUBLE 1
 #else
     typedef float  VFloat;  // 32 bit
-#   define VHF_IS_DOUBLE 0
+#   define CEN_IS_DOUBLE 0
 #endif
 
 /* 3D point */
@@ -205,4 +210,4 @@ typedef enum { LinkOnly = -1, DontLink = 0, Link = 1 } LinkType;
     #define DOUBLE  double          // Double precision float
 #endif
 
-#endif	// VHF_H_TYPES
+#endif	// CEN_H_TYPES
