@@ -1,29 +1,29 @@
 /* dvPasteboard.m
  * Pasteboard additions for Cenon DocView class
  *
- * Copyright (C) 1997-2005 by vhf interservice GmbH
+ * Copyright (C) 1997-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1997
- * modified: 2005-09-23
+ * modified: 2008-10-21 (pasting of several layers to one layer keeps order of graphics elements)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
@@ -98,9 +98,9 @@ NSArray *e2TypesCenonExports(void)
     if (!exportList)
     {
 #ifdef PB_CANSERVE_PDF
-        exportList = [[NSArray allocWithZone:NSDefaultMallocZone()] initWithObjects:e2PboardType, NSPDFPboardType, NSPostScriptPboardType, NSTIFFPboardType, nil];
+        exportList = [[NSArray allocWithZone:NSDefaultMallocZone()] initWithObjects:e2PboardType, NSPDFPboardType, NSPostScriptPboardType/*, NSTIFFPboardType*/, nil];
 #else
-        exportList = [[NSArray allocWithZone:NSDefaultMallocZone()] initWithObjects:e2PboardType, NSPostScriptPboardType, NSTIFFPboardType, nil];
+        exportList = [[NSArray allocWithZone:NSDefaultMallocZone()] initWithObjects:e2PboardType, NSPostScriptPboardType/*, NSTIFFPboardType*/, nil];
 #endif
     }
     return exportList;
@@ -232,13 +232,29 @@ NSArray *e2TypesCenonExports(void)
     return data;
 }
 #endif
+/*
+ * Images all of the objects in the glist and writes out the result in
+ * the Tagged VImage File Format (TIFF). The image will not have alpha in it.
+ */
+- (NSData*)dataForTIFF
+{   NSData	*data = nil;
+    /*NSRect	bbox;
+    NSImage	*tiffCache;
+
+    bbox = [self boundsOfArray:layerList];
+    tiffCache = [[NSImage allocWithZone:[self zone]] initWithSize:bbox.size];
+    //[self cacheList:layerList into:tiffCache];	// FIXME
+    data = [tiffCache TIFFRepresentationUsingCompression:NSTIFFCompressionLZW factor:0.0];
+    [tiffCache release];*/
+    return data;
+}
 
 /*
  * This is the same as dataForEPS, but it lets you specify the list
  * of Graphics you want to generate PostScript for (does its job by swapping
  * the glist for the list you provide temporarily).
  */
-- (NSData *)dataForEPSUsingList:list
+- (NSData *)dataForEPSUsingList:(NSArray*)list
 {   NSData	*data = nil;
     int		l;
 
@@ -251,7 +267,7 @@ NSArray *e2TypesCenonExports(void)
     return data;
 }
 #ifdef PB_CANSERVE_PDF
-- (NSData*)dataForPDFUsingList:list
+- (NSData*)dataForPDFUsingList:(NSArray*)list
 {   int	l;
 
     for (l=0; l<[list count]; l++)
@@ -259,46 +275,20 @@ NSArray *e2TypesCenonExports(void)
     return [self dataForPDF];
 }
 #endif
-
-/*
- * Images all of the objects in the glist and writes out the result in
- * the Tagged VImage File Format (TIFF). The image will not have alpha in it.
- */
-- (NSData*)dataForTIFF
-{   NSData	*data = nil;
-
-#if 0
-    NSImage	*tiffCache;
-
-    if (([glist count] == 1) && [[glist objectAtIndex:0] canEmitTIFF])
-    {
-        data = [[glist objectAtIndex:0] dataForTIFF];
-    }
-    else
-    {
-        tiffCache = [[NSImage allocWithZone:[self zone]] initWithSize:[self getBBoxOfArray:glist].size];
-        [self cacheList:glist into:tiffCache withTransparentBackground:NO];
-        data = [tiffCache TIFFRepresentationUsingCompression:NSTIFFCompressionLZW factor:NSTIFFCompressionJPEG];
-        [tiffCache release];
-    }
-#endif
-    return data;
-}
-
 /*
  * This is the same as dataForTIFF, but it lets you specify the list
  * of Graphics you want to generate TIFF for (does its job by swapping
- * the glist for the list you provide temporarily).
+ * the list of the first layer for the list you provide temporarily.
  */
-- (NSData *)dataForTIFFUsingList:list
+- (NSData*)dataForTIFFUsingList:(NSArray*)list
 {   NSData		*data = nil;
-#if 0
-    NSMutableArray	*savedglist = glist;
+    int			l;
+    //NSMutableArray	*savedList = [[[layerList objectAtIndex:0] list] retain];	// FIXME
 
-    glist = list;
+    for (l=0; l<[list count]; l++)
+        [self addList:[list objectAtIndex:l] toLayerAtIndex:0];
     data = [self dataForTIFF];
-    glist = savedglist;
-#endif
+    //[[layerList objectAtIndex:0] setList:[savedList autorelease]];	// FIXME
     return data;
 }
 
@@ -394,8 +384,8 @@ NSArray *e2TypesCenonExports(void)
 - copyToPasteboard:(NSPasteboard *)pboard types:(NSArray *)typesList
 {
     if ([slayList count])
-    {   NSMutableArray	*types = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
-        NSData		*data;
+    {   NSMutableArray  *types = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
+        NSData          *data;
 
         [types addObject:e2PboardType];
         if (!typesList || e2IncludesType(typesList, NSPostScriptPboardType))
@@ -452,28 +442,28 @@ NSArray *e2TypesCenonExports(void)
  * by the caller).
  */
 - (NSArray *)pasteFromPasteboard:(NSPasteboard *)pboard andLink:(LinkType)doLink at:(const NSPoint *)center
-{   int		i, l;
-    id		change;
-    NSArray	*pblist = nil;
-    VGraphic	*graphic = nil;
-    BOOL	pasteDrawType = NO;
+{   int         i, l;
+    id          change;
+    NSArray     *pblist = nil;
+    VGraphic    *graphic = nil;
+    BOOL        pasteDrawType = NO;
 
     //if (!linkManager)
-        doLink = DontLink;
+    doLink = DontLink;
     if (!doLink)
         pasteDrawType = e2IncludesType([pboard types], e2PboardType);
 
     if (pasteDrawType)
     {   NSData	*pbData = nil;
 
-	if ((pbData = [pboard dataForType:e2PboardType]))
+        if ((pbData = [pboard dataForType:e2PboardType]))
         {   NSUnarchiver *unarchiver = [[NSUnarchiver allocWithZone:(NSZone *)[self zone]] initForReadingWithData:pbData];
 
-	    if (unarchiver)
+            if (unarchiver)
             {   NSPoint	offset = NSMakePoint(0, 0);
                 NSRect	pbBounds, bounds = [self bounds];
 
-		pblist = [[[NSMutableArray alloc] initWithArray:[unarchiver decodeObject]] autorelease];
+                pblist = [[[NSMutableArray alloc] initWithArray:[unarchiver decodeObject]] autorelease];
 
                 /* keep graphics inside bounds (consecutive paste) or visible rect (1st paste) */
                 pbBounds = [self boundsOfArray:pblist withKnobs:NO];
@@ -518,15 +508,16 @@ NSArray *e2TypesCenonExports(void)
                             {   fixLayer = l; break; }
                         }
                     }
-                    cnt = (fixLayer<0) ? Min([layerList count], [pblist count]) : [pblist count] ;
-                    for ( l=cnt-1; l>=0; l-- )
-                    {   int		insertLayer = (fixLayer>=0) ? fixLayer : l;
-                        NSMutableArray	*slist;
-                        LayerObject	*layer;
-                        NSMutableArray	*srcList = [pblist objectAtIndex:l];
+                    cnt = (fixLayer < 0) ? Min([layerList count], [pblist count]) : [pblist count];
+                    //for ( l=cnt-1; l>=0; l-- )
+                    for ( l=0; l < cnt; l++ )
+                    {   int             insertLayer = (fixLayer >= 0) ? fixLayer : l;
+                        NSMutableArray  *slist;
+                        LayerObject     *layer;
+                        NSMutableArray  *srcList = [pblist objectAtIndex:l];
 
                         /* paste to 1st editable layer */
-                        if ( fixLayer<0 && ![[layerList objectAtIndex:l] editable])
+                        if ( fixLayer < 0 && ![[layerList objectAtIndex:l] editable])
                         {
                             for ( i=0; i<cnt; i++ )
                             {
@@ -554,10 +545,10 @@ NSArray *e2TypesCenonExports(void)
                     [change endChange];
                 }
                 else
-		    pblist = nil;
+                    pblist = nil;
                 [unarchiver release];
-	    }
-	}
+            }
+        }
     }
     else
     {   NSPoint position;
@@ -570,7 +561,7 @@ NSArray *e2TypesCenonExports(void)
         }
         else
             position = *center;
-	[self pasteForeignDataFromPasteboard:pboard andLink:doLink at:position];
+        [self pasteForeignDataFromPasteboard:pboard andLink:doLink at:position];
     }
 
     return pblist;
@@ -594,7 +585,7 @@ NSArray *e2TypesCenonExports(void)
     static NSPoint	pasteOffset;
     int			l;
 
-    /* FIXME: Wie shouldn't insert the pblist in this method.
+    /* FIXME: We shouldn't insert the pblist in this method.
      * Instead we should return pblist and insert it here
      */
     pblist = [self pasteFromPasteboard:pboard andLink:doLink at:NULL];
@@ -613,13 +604,13 @@ NSArray *e2TypesCenonExports(void)
                 return;
         }
 
-	if (lastPastedChangeCount != [pboard changeCount] || !originalPaste)
+        if (lastPastedChangeCount != [pboard changeCount] || !originalPaste)
         {   NSPoint	offset = NSMakePoint(0, 0);
             NSRect	pbBounds, bounds = [self visibleRect];
 
-	    consecutivePastes = 0;
-	    lastPastedChangeCount = [pboard changeCount];
-	    originalPaste = graphic;
+            consecutivePastes = 0;
+            lastPastedChangeCount = [pboard changeCount];
+            originalPaste = graphic;
 
             /* keep graphics inside visible rect for 1st paste */
             pbBounds = [self boundsOfArray:slayList withKnobs:NO];
@@ -638,9 +629,9 @@ NSArray *e2TypesCenonExports(void)
             if ( offset.x || offset.y )
             {
                 for (l=[slayList count]-1; l>=0; l--)
-                {   NSMutableArray	*slist = [slayList objectAtIndex:l];
-                    LayerObject		*layer = [layerList objectAtIndex:l];
-                    int			i;
+                {   NSMutableArray  *slist = [slayList objectAtIndex:l];
+                    LayerObject     *layer = [layerList objectAtIndex:l];
+                    int             i;
 
                     for (i=[slist count]-1; i>=0; i--)
                     {   VGraphic	*g = [slist objectAtIndex:i];
@@ -650,24 +641,24 @@ NSArray *e2TypesCenonExports(void)
                     }
                 }
             }
-	}
+        }
         else
         {
-	    if (consecutivePastes == 1)	// smart paste
+            if (consecutivePastes == 1)	// smart paste
             {
                 pasteOffset.x = (gridIsEnabled) ?  [self grid] : 10.0;
                 pasteOffset.y = -pasteOffset.x;
-		secondPaste = graphic;
-	    }
+                secondPaste = graphic;
+            }
             else if ((consecutivePastes == 2) && graphic)
             {
-		originalBounds = [originalPaste bounds];
-		secondBounds = [secondPaste bounds];
-		pasteOffset.x = secondBounds.origin.x - originalBounds.origin.x;
-		pasteOffset.y = secondBounds.origin.y - originalBounds.origin.y;
-	    }
-	    offset.x = pasteOffset.x * consecutivePastes;
-	    offset.y = pasteOffset.y * consecutivePastes;
+                originalBounds = [originalPaste bounds];
+                secondBounds = [secondPaste bounds];
+                pasteOffset.x = secondBounds.origin.x - originalBounds.origin.x;
+                pasteOffset.y = secondBounds.origin.y - originalBounds.origin.y;
+            }
+            offset.x = pasteOffset.x * consecutivePastes;
+            offset.y = pasteOffset.y * consecutivePastes;
 
             /* get bounds of selection, limit offset to working area */
             rect = [self boundsOfArray:slayList];
@@ -690,10 +681,10 @@ NSArray *e2TypesCenonExports(void)
                     [layer updateObject:g];
                 }
             }
-	}
+        }
         consecutivePastes++;
         [[(App*)NSApp inspectorPanel] loadList:slayList];
-//	[self cacheSelection];
+        //	[self cacheSelection];
         [self drawAndDisplay];	// redraw object bounds
     }
     //[pblist release];
