@@ -1,29 +1,31 @@
 /* dvUndo.h
  * Undo additions for Cenon DocView class
  *
- * Copyright (C) 1999-2005 by vhf interservice GmbH
+ * Copyright (C) 1999-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1999-
- * modified: 2005-07-28 (-addPointTo: added)
+ * modified: 2013-01-15 (-rotate:, rotate around coordBounds of objects)
+ *           2009-03-06 (-rotate: -deselectLockedLayers:..)
+ *           2005-07-28 (-addPointTo: added)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@vhf.biz
+ * http://www.cenon.info
  */
 
 #include <AppKit/AppKit.h>
@@ -39,8 +41,8 @@
     NSRect	affectedBounds = NSZeroRect;
 
     for ( l=0, cnt = [slayList count]; l<cnt; l++ )
-    {	NSMutableArray *slist = [slayList objectAtIndex:l];
-        LayerObject	*layer = [layerList objectAtIndex:l];
+    {	NSMutableArray  *slist = [slayList  objectAtIndex:l];
+        LayerObject     *layer = [layerList objectAtIndex:l];
 
         if (![layer editable])
             continue;
@@ -100,9 +102,9 @@
     [change startChange];
         switch (colorNum)
         {
-            default: [self graphicsPerform:@selector(setColor:) with:color]; break;
+            default: [self graphicsPerform:@selector(setColor:)     with:color]; break;
             case 1:  [self graphicsPerform:@selector(setFillColor:) with:color]; break;
-            case 2:  [self graphicsPerform:@selector(setEndColor:) with:color];
+            case 2:  [self graphicsPerform:@selector(setEndColor:)  with:color];
         }
         [document setDirty:YES];
     [change endChange];
@@ -148,25 +150,49 @@
  */
 - (void)takeFillFrom:sender
 {   id		change;
+    int		l, cnt, i;
+    NSRect	affectedBounds = NSZeroRect;
     int		filled = [sender indexOfSelectedItem];
 
     change = [[FillGraphicsChange alloc] initGraphicView:self fill:filled];
     [change startChange];
-	[self graphicsPerform:@selector(setFilled:) with:(void*)filled];
-        [document setDirty:YES];
+        //[self graphicsPerform:@selector(setFilled:) with:(void*)filled];
+        //[document setDirty:YES];
+        for (l=0, cnt = [slayList count]; l<cnt; l++)
+        {   NSMutableArray	*slist = [slayList  objectAtIndex:l];
+            LayerObject		*layer = [layerList objectAtIndex:l];
+
+            if (![layer editable])
+                continue;
+            for (i=[slist count]-1; i>=0; i--)
+            {   VGraphic    *g = [slist objectAtIndex:i];
+
+                if ( [g respondsToSelector:@selector(setFilled:)] )
+                {
+                    affectedBounds = (affectedBounds.size.width)
+                                        ? NSUnionRect([g extendedBoundsWithScale:scale], affectedBounds)
+                                        : [g extendedBoundsWithScale:scale];
+                    [g setFilled:filled];
+                    [layer updateObject:g];
+                    affectedBounds = NSUnionRect([g extendedBoundsWithScale:scale], affectedBounds);
+                }
+            }
+        }
+    [document setDirty:YES];
+    [self cache:affectedBounds];
     [change endChange];
 }
 
 - (void)takeStepWidth:(float)stepWidth
 {   id		change;
     int		l, cnt, i;
-    NSRect	affectedBounds = NSMakeRect(0, 0, 0, 0);
+    NSRect	affectedBounds = NSZeroRect;
 
     change = [[StepWidthGraphicsChange alloc] initGraphicView:self stepWidth:stepWidth];
     [change startChange];
         cnt = [slayList count];
         for (l=0; l<cnt; l++)
-        {   NSMutableArray	*slist = [slayList objectAtIndex:l];
+        {   NSMutableArray	*slist = [slayList  objectAtIndex:l];
             LayerObject		*layer = [layerList objectAtIndex:l];
 
             if (![layer editable])
@@ -229,13 +255,13 @@
         //[self graphicsPerform:@selector(setWidth_ptr:) with:&width];
         cnt = [slayList count];
         for (l=0; l<cnt; l++)
-        {   NSMutableArray	*slist = [slayList objectAtIndex:l];
+        {   NSMutableArray	*slist = [slayList  objectAtIndex:l];
             LayerObject		*layer = [layerList objectAtIndex:l];
 
             if (![layer editable])
                 continue;
             for (i=[slist count]-1; i>=0; i--)
-            {   id	g = [slist objectAtIndex:i];
+            {   VGraphic    *g = [slist objectAtIndex:i];
 
                 affectedBounds = (affectedBounds.size.width) ? NSUnionRect([g extendedBoundsWithScale:scale], affectedBounds) : [g extendedBoundsWithScale:scale];
                 if ( [g respondsToSelector:@selector(setWidth:)] )
@@ -461,14 +487,31 @@
 }
 
 - (void)rotate:(float)rotAngle
-{   int		i, l;
-    NSRect	rect, drawRect;
-    NSPoint	rotCenter;
-    id		change;
+{   int         i, l, selCount  = 0;
+    NSRect      rect, drawRect;
+    NSPoint     rotCenter;
+    id          change;
+    VGraphic    *g = nil;
 
     drawRect = [self boundsOfArray:slayList];
-    rotCenter.x = drawRect.origin.x + drawRect.size.width  / 2.0;
-    rotCenter.y = drawRect.origin.y + drawRect.size.height / 2.0;
+    [self deselectLockedLayers:YES lockedObjects:YES];
+
+    /* check for single selected object */
+    for (l=[slayList count]-1; l>=0; l--)
+    {   NSMutableArray	*slist = [slayList objectAtIndex:l];
+
+        selCount += [slist count];
+        if ( [slist count] == 1 )
+            g = [slist objectAtIndex:0];
+    }
+
+    if ( selCount == 1 && [g isKindOfClass:[VArc class]] && Abs([g angle]) >= 90.0 )    // single arc >= 90 deg
+        rotCenter = [g center];
+    else
+    {   rect = [self coordBoundsOfArray:slayList]; //rect = [self boundsOfArray:slayList withKnobs:NO];
+        rotCenter.x = rect.origin.x + rect.size.width  / 2.0;
+        rotCenter.y = rect.origin.y + rect.size.height / 2.0;
+    }
 
     change = [[RotateGraphicsChange alloc] initGraphicView:self angle:-rotAngle center:rotCenter];
     [change startChange];
