@@ -1,35 +1,36 @@
 /* functions.m
  * Common Cenon functions
  *
- * Copyright (C) 1996-2012 by vhf interservice GmbH
+ * Copyright (C) 1996-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-25
  * modified: 2012-01-05 (NSMenuItem)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.cenon.info
  */
 
 #include <AppKit/AppKit.h>
 #include <VHFShared/types.h>
 #include <VHFShared/vhfCommonFunctions.h>
 #include <VHFShared/VHFDictionaryAdditions.h>
+#include <VHFShared/VHFSystemAdditions.h>   // NSFileManager additions
 #include "functions.h"
 #include "locations.h"
 
@@ -44,8 +45,9 @@ NSString *localLibrary(void)
 
     /* we return the local path or the system path, whichever exists - local has priority */
     /* "/Library/Application Support/Cenon" */
-    lPath = vhfPathWithPathComponents([NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSLocalDomainMask, YES) objectAtIndex:0],
-                                      @"Application Support", APPNAME, nil);
+    lPath = vhfPathWithPathComponents(
+            [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSLocalDomainMask, YES) objectAtIndex:0],
+            @"Application Support", APPNAME, nil);
     if ( lPath && [fileManager fileExistsAtPath:lPath] )
         return lPath;
     /* "/Library/Cenon" (Default) */
@@ -165,7 +167,8 @@ void fillPopup(NSPopUpButton *popup, NSString *folder, NSString *ext, int remove
             break;
         path = [path stringByAppendingPathComponent:folder];
 
-        array = [fileManager directoryContentsAtPath:path];
+        //array = [fileManager directoryContentsAtPath:path];
+        array = [fileManager contentsOfDirectoryAtPath:path];
         for ( j = [array count]-1; j >= 0; j-- )
         {   NSString	*name = [array objectAtIndex:j];
             NSString    *filePath = [path stringByAppendingPathComponent:name];
@@ -185,7 +188,7 @@ void fillPopup(NSPopUpButton *popup, NSString *folder, NSString *ext, int remove
             /* B. files in sub-folders */
             else if ( searchSubFolders )    // sub-folder
             {   NSString    *subpath = vhfPathWithPathComponents(path, name, nil);
-                NSArray     *subArray = [fileManager directoryContentsAtPath:subpath];
+                NSArray     *subArray = [fileManager contentsOfDirectoryAtPath:subpath];
                 int         s;
 
                 for ( s = [subArray count]-1; s >= 0; s-- )
