@@ -1,6 +1,6 @@
 /* ChangeDetail.h
  *
- * Copyright (C) 1993-2012 by vhf interservice GmbH
+ * Copyright (C) 1993-2014 by Cenon GmbH
  * Authors:  Georg Fleischmann
  *
  * created:  1993
@@ -52,7 +52,9 @@
 @interface DimensionsChangeDetail: ChangeDetail
 {
     NSRect 	oldBounds;
+    VFloat  oldWidth;   // stroke width
     NSRect 	newBounds;
+    VFloat  newWidth;
 }
 - (BOOL)useNestedDetails;
 - (void)recordDetail;
