@@ -1,33 +1,34 @@
 /* messages.h
  * common project messages
  *
- * Copyright (C) 1993-2010 by vhf interservice GmbH
+ * Copyright (C) 1993-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1993-08-19
- * modified: 2010-04-23
+ * modified: 2011-07-05 (PSIMPORT_INSTALLGS_STRING added)
+ *           2011-03-30
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
-#ifndef VHF_H_MESSAGES
-#define VHF_H_MESSAGES
+#ifndef CEN_H_MESSAGES
+#define CEN_H_MESSAGES
 
 #define HIDE_GRID NSLocalizedString(@"Turn Grid Off", NULL)
 #define SHOW_GRID NSLocalizedString(@"Turn Grid On", NULL)
@@ -43,7 +44,7 @@
 #define OPEN_ERROR NSLocalizedString(@"I/O error.  Can't open file '%@'.", "Message in alert given to user when he tries to open a draw document and there is an I/O error.  This usually happens when the document being opened is not really a draw document but somehow got renamed to have the .draw extension.")
 #define CANT_CREATE_BACKUP NSLocalizedString(@"Can't create backup file. File not saved!", "Message indicating that a backup file could not be created during save.")
 #define DIR_NOT_WRITABLE NSLocalizedString(@"Directory is not writeable.", "Message indicating that the directory the user is trying to save to is not writable.")
-#define CANT_SAVE NSLocalizedString(@"Can't save file.", "This alert appears when the user has asked to save his file somewhere, but Draw was unable to create that file.  This can occur for many reasons, the most common of which is that the file or directory is not writable.")
+#define CANT_SAVE NSLocalizedString(@"Can't save file.", "This alert appears when the user has asked to save his file somewhere, but Cenon was unable to create that file.  This can occur for many reasons, the most common of which is that the file or directory is not writable.")
 #define EXPORTLOCK_STRING NSLocalizedString(@"This document is not supposed to be exported\nPlease respect the efforts of the author.", "This alert appears when the user has asked to save his file, but it is protected.")
 
 /* In these next few items the space in the name is intentional, to distinguish them from the same strings which occur on buttons */
@@ -64,7 +65,6 @@
 #define SELECT2FORJOIN_STRING NSLocalizedString(@"Select two objects for joining!", NULL)
 
 #define UNTITLED_STRING NSLocalizedString(@"UNTITLED", "The name of a document which the user has not yet given a name to.")
-#define CANTSAVEFILE_STRING NSLocalizedString(@"Can't save file.", NULL)
 
 #define CANTSTARTMODUL NSLocalizedString(@"Could not start module.", "The specified program could not be started")
 #define CANTOPENFILE_STRING NSLocalizedString(@"Could not open file.", "The user-specified file could not be opened")
@@ -75,7 +75,8 @@
 #define QUITANYWAY_STRING NSLocalizedString(@"Quit Anyway", "Choice (on a button) given to user which allows him/her to quit the application even though there are unsaved documents.")
 #define QUIT_STRING NSLocalizedString(@"Quit", "The operation of exiting the application.")
 #define	DEFAULTSTRING	"Default"
-#define CANTWRITEFILE_STRING NSLocalizedString(@"Can't write file.", "Document could not be saved")
+#define CANTSAVEFILE_STRING  NSLocalizedString(@"Can't save file.", NULL)   // CANT_SAVE is used
+#define CANTWRITEFILE_STRING NSLocalizedString(@"Can't write file.", "Document could not be saved")   // CANT_SAVE is used
 #define NULLGRAPHIC_STRING NSLocalizedString(@"Graphic has zero size.", "")
 #define BAD_IMAGE @"Unable to import that image into Cenon."
 #define SAVECHANGES_STRING NSLocalizedString(@"%@ has changes. Save them?", "Question asked of user when he/she tries to close a window containing an unsaved document.  The %@ is the name of the document.")
@@ -112,5 +113,6 @@
 #define DXFIMPORTOUTOFBOUNDS_STRING NSLocalizedString(@"Some Objects are exceeding drawing bounds!", NULL)
 #define IMPORTTONOTEXISTINGLAYER_STRING NSLocalizedString(@"No layer available with name/color '%@'!", NULL)
 #define CREATELAYER_STRING NSLocalizedString(@"Create Layer", NULL)
+#define PSIMPORT_INSTALLGS_STRING NSLocalizedString(@"You need to install GhostScript (gs) to import PDF or PostScript!", NULL)
 
-#endif // VHF_H_MESSAGES
+#endif // CEN_H_MESSAGES
