@@ -1,34 +1,36 @@
 /* propertyList.m
  * Functions helping to read and write property lists
  *
- * Copyright (C) 2002-2008 by vhf interservice GmbH
+ * Copyright (C) 2002-2014 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  ?
- * modified: 2008-01-12 (propertyListFromNSColor() Apple >= 10.4 workaround)
+ * modified: 2009-12-12 (save coordinated with 9 digits precision instead of 6 before)
+ *           2008-01-12 (propertyListFromNSColor() Apple >= 10.4 workaround)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
 #include <VHFShared/VHFDictionaryAdditions.h>
 #include <VHFShared/vhfCommonFunctions.h>	// vhfStringWithFloat()
+#include <VHFShared/vhfCompatibility.h>     // CGFloat for OS X 10.4
 #include "propertyList.h"
 
 #ifndef NSAppKitVersionNumber10_4
@@ -36,13 +38,38 @@
 #endif
 
 /* we don't always have VGraphic objects, so we declare the used method here */
-@interface PropertyList
+@interface PropertyList:NSObject
 - (id)initFromPropertyList:(id)plist inDirectory:(NSString*)directory;
 @end
 @implementation PropertyList
 - (id)initFromPropertyList:(id)plist inDirectory:(NSString*)directory	{ return nil; }
 @end
 
+/* we could also include StringAdditions and NumberAdditions */
+@implementation NSNumber(CGFloatAdditions)
+- (VFloat)VFloatValue
+{
+#   if CGFLOAT_IS_DOUBLE
+    return [self doubleValue];
+#   else
+    return [self floatValue];
+#   endif
+}
+@end
+@implementation NSString(CGFloatAdditions)
+- (VFloat)VFloatValue
+{
+#   if CGFLOAT_IS_DOUBLE
+    return [self doubleValue];
+#   else
+    return [self floatValue];
+#   endif
+}
+@end
+
+
+/* backward compatibility before moving to new classnames
+ */
 NSString *newClassName(NSString *className)
 {
     if ([className isEqual:@"TextGraphic"])
@@ -51,9 +78,9 @@ NSString *newClassName(NSString *className)
 }
 
 id propertyListFromArray(NSArray *array)
-{   id realObject;
-    NSMutableArray *plistArray;
-    NSEnumerator *enumerator;
+{   id              realObject;
+    NSMutableArray  *plistArray;
+    NSEnumerator    *enumerator;
 
     plistArray = [NSMutableArray arrayWithCapacity:[array count]];
     enumerator = [array objectEnumerator];
@@ -63,16 +90,19 @@ id propertyListFromArray(NSArray *array)
             [plistArray addObject:[realObject propertyList]];
         else
         {
-	    /* Should probably raise here. */
-	}
+            /* Should probably raise here. */
+        }
     }
-
     return plistArray;
 }
 
-id propertyListFromFloat(float f)
+id propertyListFromFloat(float f)   // TODO: make VFloat
 {
-    return [NSString stringWithFormat:@"%g", f];
+    return [NSString stringWithFormat:@"%.12g", f];
+}
+id propertyListFromVFloat(VFloat f)
+{
+    return [NSString stringWithFormat:@"%.12g", f];
 }
 
 id propertyListFromInt(int i)
@@ -134,7 +164,7 @@ id propertyListFromNSColor(NSColor *color)
     else if ( NSAppKitVersionNumber >= NSAppKitVersionNumber10_4 &&   // >= 10.4
               [[color colorSpaceName] isEqualToString:NSCustomColorSpace] )
     {   int     cnt = [color numberOfComponents];
-        float   compo[cnt];
+        CGFloat compo[cnt];
 
         [color getComponents:compo];
         switch ([[color colorSpace] colorSpaceModel])
@@ -176,47 +206,49 @@ id propertyListFromNSColor(NSColor *color)
 }
 
 id propertyListFromNSPrintInfo(NSPrintInfo *printInfo)
-{   NSMutableDictionary *dictionary = [NSMutableDictionary dictionaryWithCapacity:5];
+{   NSMutableDictionary *dictionary = [NSMutableDictionary dictionaryWithCapacity:7];
 
     if (!printInfo)
         return dictionary;
-    [dictionary setFloat:[printInfo leftMargin] forKey:@"leftMargin"];
-    [dictionary setFloat:[printInfo rightMargin] forKey:@"rightMargin"];
-    [dictionary setFloat:[printInfo topMargin] forKey:@"topMargin"];
-    [dictionary setFloat:[printInfo bottomMargin] forKey:@"bottomMargin"];
+    [dictionary setFloat:[printInfo leftMargin]     forKey:@"leftMargin"];
+    [dictionary setFloat:[printInfo rightMargin]    forKey:@"rightMargin"];
+    [dictionary setFloat:[printInfo topMargin]      forKey:@"topMargin"];
+    [dictionary setFloat:[printInfo bottomMargin]   forKey:@"bottomMargin"];
     [dictionary setObject:propertyListFromNSSize([printInfo paperSize]) forKey:@"paperSize"];
-    [dictionary setObject:[printInfo paperName] forKey:@"paperName"];
-    [dictionary setInt:[printInfo orientation] forKey:@"orientation"];
-
+    [dictionary setObject:[printInfo paperName]     forKey:@"paperName"];
+    [dictionary setInt:[printInfo orientation]      forKey:@"orientation"];
     return dictionary;
 }
 
 id propertyListFromNSRect(NSRect rect)
 {
-    return [NSString stringWithFormat:@"%g %g %g %g", rect.origin.x, rect.origin.y, rect.size.width, rect.size.height];
+    return [NSString stringWithFormat:@"%.12g %.12g %.12g %.12g", rect.origin.x, rect.origin.y, rect.size.width, rect.size.height];
 }
 
 id propertyListFromNSSize(NSSize size)
 {
-    return [NSString stringWithFormat:@"%g %g", size.width, size.height];
+    return [NSString stringWithFormat:@"%.12g %.12g", size.width, size.height];
 }
 
 id propertyListFromNSPoint(NSPoint point)
-{
-    return [NSString stringWithFormat:@"%g %g", point.x, point.y];
+{   //int digX = 9;
+
+    //if ( floor(point.x*1000.0)/1000.0 == point.x )
+    //    digX = 6;
+    return [NSString stringWithFormat:@"%.12g %.12g", point.x, point.y];
 }
 
 id propertyListFromV3Point(V3Point point)
 {
-    return [NSString stringWithFormat:@"%g %g %g", point.x, point.y, point.z];
+    return [NSString stringWithFormat:@"%.12g %.12g %.12g", point.x, point.y, point.z];
 }
 
 NSMutableArray *arrayFromPropertyList(id plist, NSString *directory, NSZone *zone)
-{   id realObject;
-    NSMutableArray *realArray;
-    NSDictionary *plistObject;
-    NSEnumerator *enumerator;
-    NSString *className;
+{   id              realObject;
+    NSMutableArray  *realArray;
+    NSDictionary    *plistObject;
+    NSEnumerator    *enumerator;
+    NSString        *className;
 
     realArray = [[NSMutableArray allocWithZone:zone] initWithCapacity:[plist count]];
     enumerator = [plist objectEnumerator];
@@ -250,41 +282,66 @@ NSColor *colorFromPropertyList(id plist, NSZone *zone)
         if ([colorSpaceName isEqualToString:@"CalWhite"] || [colorSpaceName isEqualToString:@"NSCalibratedWhiteColorSpace"])
         {
             if ([plist objectForKey:@"W"])
-                return [[NSColor colorWithCalibratedWhite:[[plist objectForKey:@"W"] floatValue] alpha:[[plist objectForKey:@"A"] floatValue]] retain];
-            else
-                return [[NSColor colorWithCalibratedWhite:[[plist objectForKey:@"White"] floatValue] alpha:[[plist objectForKey:@"Alpha"] floatValue]] retain];
+                return [[NSColor colorWithCalibratedWhite:[[plist objectForKey:@"W"] floatValue]
+                                                    alpha:[[plist objectForKey:@"A"] floatValue]] retain];
+            else    // stone old
+                return [[NSColor colorWithCalibratedWhite:[[plist objectForKey:@"White"] floatValue]
+                                                    alpha:[[plist objectForKey:@"Alpha"] floatValue]] retain];
         }
         else if ([colorSpaceName isEqualToString:@"CalRGB"] || [colorSpaceName isEqualToString:@"NSCalibratedRGBColorSpace"])
         {
             if ([plist objectForKey:@"R"])
-            return [[NSColor colorWithCalibratedRed:[[plist objectForKey:@"R"] floatValue] green:[[plist objectForKey:@"G"] floatValue] blue:[[plist objectForKey:@"B"] floatValue] alpha:[[plist objectForKey:@"A"] floatValue]] retain];
+            return [[NSColor colorWithCalibratedRed:[[plist objectForKey:@"R"] floatValue]
+                                              green:[[plist objectForKey:@"G"] floatValue]
+                                               blue:[[plist objectForKey:@"B"] floatValue]
+                                              alpha:[[plist objectForKey:@"A"] floatValue]] retain];
             else
-                return [[NSColor colorWithCalibratedRed:[[plist objectForKey:@"Red"] floatValue] green:[[plist objectForKey:@"Green"] floatValue] blue:[[plist objectForKey:@"Blue"] floatValue] alpha:[[plist objectForKey:@"Alpha"] floatValue]] retain];
+                return [[NSColor colorWithCalibratedRed:[[plist objectForKey:@"Red"] floatValue]
+                                                  green:[[plist objectForKey:@"Green"] floatValue]
+                                                   blue:[[plist objectForKey:@"Blue"] floatValue]
+                                                  alpha:[[plist objectForKey:@"Alpha"] floatValue]] retain];
         }
         else if ([colorSpaceName isEqualToString:@"DevWhite"] || [colorSpaceName isEqualToString:@"NSDeviceWhiteColorSpace"])
         {
             if ([plist objectForKey:@"W"])
-                return [[NSColor colorWithDeviceWhite:[[plist objectForKey:@"W"] floatValue] alpha:[[plist objectForKey:@"A"] floatValue]] retain];
+                return [[NSColor colorWithDeviceWhite:[[plist objectForKey:@"W"] floatValue]
+                                                alpha:[[plist objectForKey:@"A"] floatValue]] retain];
             else
-                return [[NSColor colorWithDeviceWhite:[[plist objectForKey:@"White"] floatValue] alpha:[[plist objectForKey:@"Alpha"] floatValue]] retain];
+                return [[NSColor colorWithDeviceWhite:[[plist objectForKey:@"White"] floatValue]
+                                                alpha:[[plist objectForKey:@"Alpha"] floatValue]] retain];
         }
         else if ([colorSpaceName isEqualToString:@"DevRGB"] || [colorSpaceName isEqualToString:@"NSDeviceRGBColorSpace"])
         {
             if ([plist objectForKey:@"R"])
-                return [[NSColor colorWithDeviceRed:[[plist objectForKey:@"R"] floatValue] green:[[plist objectForKey:@"G"] floatValue] blue:[[plist objectForKey:@"B"] floatValue] alpha:[[plist objectForKey:@"A"] floatValue]] retain];
+                return [[NSColor colorWithDeviceRed:[[plist objectForKey:@"R"] floatValue]
+                                              green:[[plist objectForKey:@"G"] floatValue]
+                                               blue:[[plist objectForKey:@"B"] floatValue]
+                                              alpha:[[plist objectForKey:@"A"] floatValue]] retain];
             else
-                return [[NSColor colorWithDeviceRed:[[plist objectForKey:@"Red"] floatValue] green:[[plist objectForKey:@"Green"] floatValue] blue:[[plist objectForKey:@"Blue"] floatValue] alpha:[[plist objectForKey:@"Alpha"] floatValue]] retain];
+                return [[NSColor colorWithDeviceRed:[[plist objectForKey:@"Red"] floatValue]
+                                              green:[[plist objectForKey:@"Green"] floatValue]
+                                               blue:[[plist objectForKey:@"Blue"] floatValue]
+                                              alpha:[[plist objectForKey:@"Alpha"] floatValue]] retain];
         }
         else if ([colorSpaceName isEqualToString:@"DevCMYK"] || [colorSpaceName isEqualToString:@"NSDeviceCMYKColorSpace"])
         {
             if ([plist objectForKey:@"C"])
-                return [[NSColor colorWithDeviceCyan:[[plist objectForKey:@"C"] floatValue] magenta:[[plist objectForKey:@"M"] floatValue] yellow:[[plist objectForKey:@"Y"] floatValue] black:[[plist objectForKey:@"B"] floatValue] alpha:[[plist objectForKey:@"A"] floatValue]] retain];
+                return [[NSColor colorWithDeviceCyan:[[plist objectForKey:@"C"] floatValue]
+                                             magenta:[[plist objectForKey:@"M"] floatValue]
+                                              yellow:[[plist objectForKey:@"Y"] floatValue]
+                                               black:[[plist objectForKey:@"B"] floatValue]
+                                               alpha:[[plist objectForKey:@"A"] floatValue]] retain];
             else
-                return [[NSColor colorWithDeviceCyan:[[plist objectForKey:@"Cyan"] floatValue] magenta:[[plist objectForKey:@"Magenta"] floatValue] yellow:[[plist objectForKey:@"Yellow"] floatValue] black:[[plist objectForKey:@"Black"] floatValue] alpha:[[plist objectForKey:@"Alpha"] floatValue]] retain];
+                return [[NSColor colorWithDeviceCyan:[[plist objectForKey:@"Cyan"] floatValue]
+                                             magenta:[[plist objectForKey:@"Magenta"] floatValue]
+                                              yellow:[[plist objectForKey:@"Yellow"] floatValue]
+                                               black:[[plist objectForKey:@"Black"] floatValue]
+                                               alpha:[[plist objectForKey:@"Alpha"] floatValue]] retain];
         }
         else if ([colorSpaceName isEqualToString:@"Named"] || [colorSpaceName isEqualToString:@"NSNamedColorSpace"])
         {
-            return [[NSColor colorWithCatalogName:[plist objectForKey:@"CId"] colorName:[plist objectForKey:@"NId"]] retain];
+            return [[NSColor colorWithCatalogName:[plist objectForKey:@"CId"]
+                                        colorName:[plist objectForKey:@"NId"]] retain];
         }
         else if ([colorSpaceName isEqualToString:@"Unknown"])
             return [[NSUnarchiver unarchiveObjectWithData:[plist objectForKey:@"Data"]] retain];
@@ -304,25 +361,30 @@ NSPrintInfo *printInfoFromPropertyList(id plist, NSZone *zone)
     //[printInfo setHorizontallyCentered:NO];
     //[printInfo setHorizontalPagination:NSFitPagination];
 
-    [printInfo setLeftMargin:[plist floatForKey:@"leftMargin"]];
-    [printInfo setRightMargin:[plist floatForKey:@"rightMargin"]];
-    [printInfo setTopMargin:[plist floatForKey:@"topMargin"]];
-    [printInfo setBottomMargin:[plist floatForKey:@"bottomMargin"]];
+    [printInfo setLeftMargin:   [plist VFloatForKey:@"leftMargin"]];
+    [printInfo setRightMargin:  [plist VFloatForKey:@"rightMargin"]];
+    [printInfo setTopMargin:    [plist VFloatForKey:@"topMargin"]];
+    [printInfo setBottomMargin: [plist VFloatForKey:@"bottomMargin"]];
 
     [printInfo setPaperSize:sizeFromPropertyList([plist objectForKey:@"paperSize"])];
-    [printInfo setPaperName:[plist objectForKey:@"paperName"]];
-    [printInfo setOrientation:[plist intForKey:@"orientation"]];
+    [printInfo setPaperName:    [plist objectForKey:@"paperName"]];
+    [printInfo setOrientation:  [plist intForKey:@"orientation"]];
     return printInfo;
 }
 
+VFloat vfloatFromPropertyList(NSString *plist)
+{   VFloat	retval = [plist VFloatValue];
+    return retval;
+}
+
 NSRect rectFromPropertyList(NSString *plist)
 {   NSRect	retval;
     NSArray	*components = [plist componentsSeparatedByString:@" "];
 
-    retval.origin.x = [[components objectAtIndex:0] floatValue];
-    retval.origin.y = [[components objectAtIndex:1] floatValue];
-    retval.size.width  = [[components objectAtIndex:2] floatValue];
-    retval.size.height = [[components objectAtIndex:3] floatValue];
+    retval.origin.x     = [[components objectAtIndex:0] VFloatValue];
+    retval.origin.y     = [[components objectAtIndex:1] VFloatValue];
+    retval.size.width   = [[components objectAtIndex:2] VFloatValue];
+    retval.size.height  = [[components objectAtIndex:3] VFloatValue];
     return retval;
 }
 
@@ -330,8 +392,8 @@ NSSize sizeFromPropertyList(id plist)
 {   NSSize	retval;
     NSArray	*components = [plist componentsSeparatedByString:@" "];
 
-    retval.width = [[components objectAtIndex:0] floatValue];
-    retval.height = [[components objectAtIndex:1] floatValue];
+    retval.width  = [[components objectAtIndex:0] VFloatValue];
+    retval.height = [[components objectAtIndex:1] VFloatValue];
     return retval;
 }
 
@@ -341,8 +403,8 @@ NSPoint pointFromPropertyList(id plist)
 
     if ([components count] < 2)
         return NSMakePoint(0.0, 0.0);
-    retval.x = [[components objectAtIndex:0] floatValue];
-    retval.y = [[components objectAtIndex:1] floatValue];
+    retval.x = [[components objectAtIndex:0] VFloatValue];
+    retval.y = [[components objectAtIndex:1] VFloatValue];
     return retval;
 }
 
@@ -352,8 +414,8 @@ V3Point v3pointFromPropertyList(id plist)
 
     if ([components count] < 3)
         return V3MakePoint(0.0, 0.0, 0.0);
-    retval.x = [[components objectAtIndex:0] floatValue];
-    retval.y = [[components objectAtIndex:1] floatValue];
-    retval.z = [[components objectAtIndex:2] floatValue];
+    retval.x = [[components objectAtIndex:0] VFloatValue];
+    retval.y = [[components objectAtIndex:1] VFloatValue];
+    retval.z = [[components objectAtIndex:2] VFloatValue];
     return retval;
 }
