@@ -1,28 +1,28 @@
 /* ChangeManager.m
  *
- * Copyright (C) 1993-2006 by vhf interservice GmbH
+ * Copyright (C) 1993-2014 by Cenon GmbH
  * Authors:  Georg Fleischmann
  *
  * created:  1993 based on the Draw example files
  * modified: 2006-11-07
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the vhf Public License as
- * published by vhf interservice GmbH. Among other things, the
+ * modify it under the terms of the Cenon Public License as
+ * published by Cenon GmbH. Among other things, the
  * License requires that the copyright notices and this notice
  * be preserved on all copies.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the vhf Public License for more details.
+ * See the Cenon Public License for more details.
  *
- * You should have received a copy of the vhf Public License along
- * with this program; see the file LICENSE. If not, write to vhf.
+ * You should have received a copy of the Cenon Public License along
+ * with this program; see the file LICENSE. If not, write to Cenon.
  *
- * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
- * eMail: info@vhf.de
- * http://www.vhf.de
+ * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include "undochange.h"
@@ -75,9 +75,9 @@
 {
     if (_lastChange == nil)
         return NO;
-    NSAssert1(![_lastChange changeInProgress], @"%@", "Fault in Undo system: Code 1");
-    NSAssert1(![_lastChange disabled], @"%@", "Fault in Undo system: Code 2");
-    NSAssert1([_lastChange hasBeenDone], @"%@", "Fault in Undo system: Code 3");
+    NSAssert1(![_lastChange changeInProgress], @"%@", @"Fault in Undo system: Code 1");
+    NSAssert1(![_lastChange disabled],         @"%@", @"Fault in Undo system: Code 2");
+    NSAssert1([_lastChange hasBeenDone],       @"%@", @"Fault in Undo system: Code 3");
     return YES;
 }
 
@@ -85,9 +85,9 @@
 {
     if (_nextChange == nil)
         return NO;
-    NSAssert1(![_nextChange changeInProgress], @"%@", "Fault in Undo system: Code 4");
-    NSAssert1(![_nextChange disabled], @"%@", "Fault in Undo system: Code 5");
-    NSAssert1(![_nextChange hasBeenDone], @"%@", "Fault in Undo system: Code 6");
+    NSAssert1(![_nextChange changeInProgress], @"%@", @"Fault in Undo system: Code 4");
+    NSAssert1(![_nextChange disabled],         @"%@", @"Fault in Undo system: Code 5");
+    NSAssert1(![_nextChange hasBeenDone],      @"%@", @"Fault in Undo system: Code 6");
     return YES;
 }
 
@@ -317,9 +317,9 @@
 - (BOOL)changeComplete:change
 {   int i;
 
-    NSAssert1(![change changeInProgress], @"%@", "Fault in Undo system: Code 7");
-    NSAssert1(![change disabled], @"%@", "Fault in Undo system: Code 8");
-    NSAssert1([change hasBeenDone], @"%@", "Fault in Undo system: Code 9");
+    NSAssert1(![change changeInProgress], @"%@", @"Fault in Undo system: Code 7");
+    NSAssert1(![change disabled],         @"%@", @"Fault in Undo system: Code 8");
+    NSAssert1([change hasBeenDone],       @"%@", @"Fault in Undo system: Code 9");
     if (change != _changeInProgress)
     {
         /* Actually, we come here whenever a change is 
@@ -332,7 +332,7 @@
 
     if (_numberOfUndoneChanges > 0)
     {
-        NSAssert1(_numberOfDoneChanges != N_LEVEL_UNDO, @"%@", "Fault in Undo system: Code 10");
+        NSAssert1(_numberOfDoneChanges != N_LEVEL_UNDO, @"%@", @"Fault in Undo system: Code 10");
         /* Remove and free all undone changes */
         for (i = (_numberOfDoneChanges + _numberOfUndoneChanges); i > _numberOfDoneChanges; i--)
             [_changeList removeObjectAtIndex:(i - 1)];
@@ -343,8 +343,8 @@
     }
     if (_numberOfDoneChanges == N_LEVEL_UNDO)
     {
-        NSAssert1(_numberOfUndoneChanges == 0, @"%@", "Fault in Undo system: Code 11");
-        NSAssert1(_nextChange == nil, @"%@", "Fault in Undo system: Code 12");
+        NSAssert1(_numberOfUndoneChanges == 0, @"%@", @"Fault in Undo system: Code 11");
+        NSAssert1(_nextChange == nil,          @"%@", @"Fault in Undo system: Code 12");
         [_changeList removeObjectAtIndex:0];
         _numberOfDoneChanges--;
         _someChangesForgotten = YES;
